package com.example.tdl.callabletask;

import com.example.tdl.config.mqtt.MqttTemlateAsync;
import com.example.tdl.domain.vo.CheckInVo;
import com.example.tdl.domain.vo.ConfigCMDVo;
import com.example.tdl.domain.vo.ConfigVo;
import com.example.tdl.domain.vo.RespVo;
import com.example.tdl.entity.Config;
import com.example.tdl.service.ConfigService;
import com.google.gson.Gson;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import com.google.gson.JsonSyntaxException;
import org.eclipse.paho.client.mqttv3.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.Callable;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * @Author: lelouch
 * @Description:
 * @Date: Created in 9:34 2018/3/13
 * @Modified By:
 **/
public class GWConfigWorker implements Callable<String>, MqttCallback {

    private Logger logger = LoggerFactory.getLogger(this.getClass());

    @Autowired
    private ConfigService configService;

    private static final int INIT_DELAY_DEFAULT = 1000; // unit:ms
    private static final int SCH_PERIOD_DEFAULT = 10 * INIT_DELAY_DEFAULT; // unit:ms

    private static final String PUB_TOPIC = "/Config";
    private static final String SUB_TOPIC = "/Config/Resp";

    private String SN;
    private String Type;
    private Boolean NeedResp = false;
    private String Device;
    private String ConfigCMD;//发送的数据

    private String ConfigTopic;//发送的主题
    private String RespTopic;//返回的主题

    private String ret_data=null;

    private MqttTemlateAsync mqttAsyncClient;
    private Gson gson = new Gson();

    private ScheduledExecutorService mReconnectScheduler;

    private long mInitDelay = INIT_DELAY_DEFAULT;
    private long mSchedulePeriod = SCH_PERIOD_DEFAULT;

    private String url;
    private String port;
    private String username;
    private String password;
    private int qos;
    private String cacrt;
    private String clientkey;
    private String clientcrt;
    private String clientpwd;

    public void SetMqttConfig(String url, String port, String user, String password, int qos,
                              String mcacrt, String mclientkey, String mclientcrt, String mclientpwd) {
        this.url = url;
        this.port = port;
        this.username = user;
        this.password = password;
        this.qos = qos;
        this.cacrt = mcacrt;
        this.clientkey = mclientkey;
        this.clientcrt = mclientcrt;
        this.clientpwd = mclientpwd;
    }

    public GWConfigWorker(String type,String sn, String CMD,Boolean flag) throws Exception{
        this.SN = sn;
        this.Type = type;
        this.Device = Type + "_" + SN;
        this.ConfigCMD = CMD;
        this.NeedResp = flag;
        this.ConfigTopic = "GW/" + Type + "/" + SN + PUB_TOPIC;
        this.RespTopic = "GW/" + Type + "/" + SN + SUB_TOPIC;
    }

    @Override
    public String call() throws Exception {
        try {
            mqttAsyncClient = new MqttTemlateAsync(this.url + ":" + this.port,
                    Device + "_" + String.valueOf(System.currentTimeMillis()) + "_config");
                mqttAsyncClient.connect(MqttTemlateAsync.setSSLOptions(this.username, this.password,
                        this.cacrt, this.clientcrt, this.clientkey, this.clientpwd), null,
                        new IMqttActionListener() {//监听
                            @Override
                            public void onSuccess(IMqttToken iMqttToken) {
                                System.out.println("connection successfull ");
                                try {
                                    if(NeedResp){
                                        mqttAsyncClient.subscribe(RespTopic, qos);//订阅主题
                                    }
                                    //获取数据
                                    ConfigVo configVo = configService.getConfig(SN,Type);
                                    ConfigCMD = configVo.getMessage();
                                    //发送数据
                                    publish(ConfigCMD,ConfigTopic);
                                } catch (MqttException e){
                                    e.printStackTrace();
                                    System.out.println("Error: " + e.getMessage());
                                }
                            }
                            @Override
                            public void onFailure(IMqttToken iMqttToken, Throwable throwable) {
                                System.out.println("connection fail with client id");
                            }
                        });
            this.mqttAsyncClient.setCallback(this);
        } catch (Exception e) {
            e.printStackTrace();
            logger.info(e.toString());
        }
        return ret_data;
    }

    private void clientReconnect(){
        if (mReconnectScheduler != null) {
            // mReconnectScheduler is running.
            return;
        }
        mReconnectScheduler = Executors.newSingleThreadScheduledExecutor();
        mReconnectScheduler.scheduleAtFixedRate(new Runnable() {
            @Override
            public void run() {
                // TODO Auto-generated method stub
                if (mqttAsyncClient != null && !mqttAsyncClient.isConnected()) {
                    try {
                        mqttAsyncClient.reconnect();
                    } catch (MqttSecurityException e) {
                        // TODO: handle exception
                        e.printStackTrace();
                        logger.info(e.toString());
                    } catch (MqttException e) {
                        // TODO: handle exception
                        e.printStackTrace();
                        logger.info(e.toString());
                    }
                } else {
                    mReconnectScheduler.shutdown();
                    mReconnectScheduler = null;
                }
            }
        }, mInitDelay, mSchedulePeriod, TimeUnit.MILLISECONDS);
    }

    @Override
    public void connectionLost(Throwable cause) {
        System.out.println(cause);
        clientReconnect();
    }

    @Override
    public void messageArrived(String topic, MqttMessage mqttMessage) throws Exception {
        String Message = mqttMessage.toString();
        try {
            this.ret_data = Message;
            //解析Message消息
            RespVo respVo = new Gson().fromJson(Message,RespVo.class);
            if(respVo.getResponse().equals("config")){
                //配置回复
                configService.bindiSuccess(SN,Type,respVo.getStatus());
            }else{
                //解绑回复
                if(respVo.getStatus() ==0 || respVo.getStatus() == 1){
                    configService.delConfig(SN,Type);
                }
            }
            if(NeedResp){
                mqttAsyncClient.unsubscribe(RespTopic);//不监听回复主题
            }
            mqttAsyncClient.disconnect();
        } catch (JsonSyntaxException e){
            e.printStackTrace();
            System.out.println("mqttAsyncClient: message " + Message + " received syntax error.");
        } catch (IllegalStateException e){
            e.printStackTrace();
            System.out.println("mqttAsyncClient: " + e);
        } catch (Exception e){
            e.printStackTrace();
            logger.info(e.toString());
        }
    }

    @Override
    public void deliveryComplete(IMqttDeliveryToken iMqttDeliveryToken) {
    }

    //发送数据的接口
    public void publish(String ConfigCMD,String ConfigTopic) throws MqttException {
        //发送数据
        MqttMessage msg = new MqttMessage();
        msg.setQos(1);
        msg.setRetained(true);
        msg.setPayload(ConfigCMD.getBytes());
        mqttAsyncClient.publish(ConfigTopic,msg);
    }

}
