package com.example.tdl.config;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.PropertyAccessor;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.cache.CacheManager;
import org.springframework.cache.annotation.CachingConfigurerSupport;
import org.springframework.cache.annotation.EnableCaching;
import org.springframework.cache.interceptor.KeyGenerator;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.PropertySource;
import org.springframework.data.redis.cache.RedisCacheManager;
import org.springframework.data.redis.connection.RedisConnectionFactory;
import org.springframework.data.redis.connection.jedis.JedisConnectionFactory;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.data.redis.serializer.Jackson2JsonRedisSerializer;
import redis.clients.jedis.JedisPoolConfig;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;


@Configuration
@EnableCaching
@ConfigurationProperties(prefix = "tdl.redis")
public class RedisConfig extends CachingConfigurerSupport{

    private String host;
    private String password;
    private int port;
    private int timeout;

    private int index;
    private int poolmaxactive;
    private int poolmaxwait;
    private int poolmaxidle;
    private int poolminidle;

    public int getIndex() {
        return index;
    }

    public void setIndex(int index) {
        this.index = index;
    }

    public int getPoolmaxactive() {
        return poolmaxactive;
    }

    public void setPoolmaxactive(int poolmaxactive) {
        this.poolmaxactive = poolmaxactive;
    }

    public int getPoolmaxwait() {
        return poolmaxwait;
    }

    public void setPoolmaxwait(int poolmaxwait) {
        this.poolmaxwait = poolmaxwait;
    }

    public int getPoolmaxidle() {
        return poolmaxidle;
    }

    public void setPoolmaxidle(int poolmaxidle) {
        this.poolmaxidle = poolmaxidle;
    }

    public int getPoolminidle() {
        return poolminidle;
    }

    public void setPoolminidle(int poolminidle) {
        this.poolminidle = poolminidle;
    }

    public String getHost() {
        return host;
    }

    public void setHost(String host) {
        this.host = host;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public int getPort() {
        return port;
    }

    public void setPort(int port) {
        this.port = port;
    }

    public int getTimeout() {
        return timeout;
    }

    public void setTimeout(int timeout) {
        this.timeout = timeout;
    }

    /**
     * 生产key的策略
     *
     * @return
     */
    @Bean
    @Override
    public KeyGenerator keyGenerator() {
        return new KeyGenerator() {
            @Override
            public Object generate(Object target, Method method, Object... params) {
                StringBuilder sb = new StringBuilder();
                sb.append(target.getClass().getName());
                sb.append(method.getName());
                for (Object obj : params) {
                    sb.append(obj.toString());
                }
                return sb.toString();
            }
        };
    }

    /**
     * 管理缓存
     *
     * @param redisTemplate
     * @return
     */
    @Bean
    public CacheManager CacheManager(RedisTemplate<?, ?> redisTemplate) {
        RedisCacheManager rcm = new RedisCacheManager(redisTemplate);
        // 设置cache过期时间,时间单位是秒
        rcm.setDefaultExpiration(60);
        Map<String, Long> map = new HashMap<String, Long>();
        map.put("test", 60L);
        rcm.setExpires(map);
        return rcm;
    }

    /**
     * redis 数据库连接池
     *
     * @return
     */
    public RedisConnectionFactory  redisConnectionFactory(int dbIndex) {
        JedisConnectionFactory factory = new JedisConnectionFactory();
        setIndex(dbIndex);
        factory.setHostName(host);
        factory.setPort(Integer.valueOf(port));
        factory.setDatabase(index);
        factory.setTimeout(10000); // 设置连接超时时间
        factory.setPoolConfig(poolConfig());
        factory.afterPropertiesSet();
        return factory;
    }

    public JedisPoolConfig poolConfig() {
        JedisPoolConfig poolCofig = new JedisPoolConfig();
        poolCofig.setMaxIdle(poolmaxidle);
        poolCofig.setMaxTotal(poolmaxactive);
        poolCofig.setMaxWaitMillis(poolmaxwait);
        poolCofig.setTestOnBorrow(false);
        return poolCofig;
    }

    /**
     * redisSerializer
     */
    private void JsonRedisSerializer(StringRedisTemplate template, boolean isHash) {
        Jackson2JsonRedisSerializer<Object> jackson2JsonRedisSerializer =
                new Jackson2JsonRedisSerializer<Object>(Object.class);
        ObjectMapper om = new ObjectMapper();
        om.setVisibility(PropertyAccessor.ALL, JsonAutoDetect.Visibility.ANY);
        om.enableDefaultTyping(ObjectMapper.DefaultTyping.NON_FINAL);
        jackson2JsonRedisSerializer.setObjectMapper(om);
        template.setValueSerializer(jackson2JsonRedisSerializer);
        if (isHash) {
            template.setHashValueSerializer(jackson2JsonRedisSerializer);
            template.setHashKeySerializer(jackson2JsonRedisSerializer);
        }
        template.afterPropertiesSet();
    }

    /**
     * redisTemplate配置
     *
     * @param
     * @return
     */
    @Bean
    public RedisTemplate<String, String> infoRedisTemplate() {
        StringRedisTemplate template = new StringRedisTemplate(redisConnectionFactory(0));
        JsonRedisSerializer(template, false);
        return template;
    }

    @Bean
    public RedisTemplate<String, String> realRedisTemplate() {
        StringRedisTemplate template = new StringRedisTemplate(redisConnectionFactory(1));
        JsonRedisSerializer(template, true);
        return template;
    }

    @Bean
    public RedisTemplate<String, String> alarmRedisTemplate() {
        StringRedisTemplate template = new StringRedisTemplate(redisConnectionFactory(3));
        JsonRedisSerializer(template, false);
        return template;
    }

    @Bean
    public RedisTemplate<String, String> tokenRedisTemplate() {
        StringRedisTemplate template = new StringRedisTemplate(redisConnectionFactory(15));
        JsonRedisSerializer(template, false);
        return template;
    }



}

