package com.example.tdl.mqtt;

import com.example.tdl.callabletask.GWConfigWorker;
import com.example.tdl.config.mqtt.MqttConfig;
import com.example.tdl.config.mqtt.MqttTemlateAsync;
import com.example.tdl.domain.vo.CheckInVo;
import com.example.tdl.domain.vo.ConfigCMDVo;
import com.example.tdl.domain.vo.ConfigVo;
import com.example.tdl.service.ConfigService;
import com.google.gson.Gson;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import com.google.gson.JsonSyntaxException;
import org.apache.commons.lang3.StringUtils;
import org.eclipse.paho.client.mqttv3.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.expression.spel.ast.NullLiteral;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * @Author: lelouch
 * @Description:
 * @Date: Created in 9:41 2018/3/13
 * @Modified By:
 **/
@Component
public class MqttListener implements MqttCallback {
    private static final int INIT_DELAY_DEFAULT = 1000; // unit:ms
    private static final int SCH_PERIOD_DEFAULT = 10 * INIT_DELAY_DEFAULT; // unit:ms
    private static final String CHECKIN_TOPIC = "TDL/+/+/CheckIn";

    private MqttTemlateAsync mqttAsyncClient;

    @Autowired
    private MqttConfig mqttconfig;

    @Autowired
    ThreadPoolTaskExecutor configthreadPool;

    @Autowired
    private ConfigService configService;

    Map<String,Future<String>> futureMap = new HashMap<String,Future<String>>();

    private ScheduledExecutorService mReconnectScheduler;
    private long mInitDelay = INIT_DELAY_DEFAULT;
    private long mSchedulePeriod = SCH_PERIOD_DEFAULT;
    private String listener_topic = CHECKIN_TOPIC;

    @PostConstruct
    public void initialize() throws MqttException {
        try {
            mqttAsyncClient = new MqttTemlateAsync(mqttconfig.getUrl() + ":" + mqttconfig.getPort(),
                    "TDL_CheckIn" + "_" + String.valueOf(System.currentTimeMillis()) + "_sub");
            mqttAsyncClient.connect(MqttTemlateAsync.setSSLOptions(mqttconfig.getUsername(), mqttconfig.getPassword(),
                    mqttconfig.getCacrt(), mqttconfig.getClientcrt(),mqttconfig.getClientkey(), mqttconfig.getClientpwd()), null,
                    new IMqttActionListener() {
                        @Override
                        public void onSuccess(IMqttToken iMqttToken) {
                            System.out.println("connection successfull ");
                            try {
                                mqttAsyncClient.subscribe(listener_topic, mqttconfig.getQos());
                            } catch (MqttException e) {
                                System.out.println("Error: " + e.getMessage());
                            }
                        }

                        @Override
                        public void onFailure(IMqttToken iMqttToken, Throwable throwable) {
                            System.out.println("connection fail with client id");
                        }
                    });
            this.mqttAsyncClient.setCallback(this);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void clientReconnect() {
        if (mReconnectScheduler != null) {
            // mReconnectScheduler is running.
            return;
        }

        mReconnectScheduler = Executors.newSingleThreadScheduledExecutor();
        mReconnectScheduler.scheduleAtFixedRate(new Runnable() {
            @Override
            public void run() {
                // TODO Auto-generated method stub
                if (mqttAsyncClient != null && !mqttAsyncClient.isConnected()) {
                    try {
                        mqttAsyncClient.reconnect();
                    } catch (MqttSecurityException e) {
                        // TODO: handle exception
                        e.printStackTrace();
                    } catch (MqttException e) {
                        // TODO: handle exception
                        e.printStackTrace();
                    }
                } else {
                    mReconnectScheduler.shutdown();
                    mReconnectScheduler = null;
                }
            }
        }, mInitDelay, mSchedulePeriod, TimeUnit.MILLISECONDS);
    }

    @Override
    public void connectionLost(Throwable cause) {
        System.out.println(cause);
        clientReconnect();
    }

    @Override
    public void messageArrived(String topic, MqttMessage mqttMessage) throws Exception {
        String Message = mqttMessage.toString();
        Long timestamp = System.currentTimeMillis();
        try {
            String[] tmparray = topic.split("/");
            String Type = tmparray[1];
            String SN = tmparray[2];
            Integer flag = parseData(Message,SN,Type);
            //            String ConfigData ="{\"action\":\"config\"}";
            String ConfigData;
            Future<String> result = null;
            GWConfigWorker gcconfig = null;
            if(flag == 0){
                ConfigCMDVo configCMDVo = new ConfigCMDVo();
                configCMDVo.setAction("sleep");
                configCMDVo.setT(System.currentTimeMillis()/1000l);
                ConfigData = new Gson().toJson(configCMDVo);
                gcconfig = new GWConfigWorker(Type,SN,ConfigData, false);
            }else if(flag == 2){
                ConfigCMDVo configCMDVo = new ConfigCMDVo();
                configCMDVo.setAction("end");
                configCMDVo.setT(System.currentTimeMillis()/1000l);
                ConfigData = new Gson().toJson(configCMDVo);
                gcconfig = new GWConfigWorker(Type,SN,ConfigData, true);
            }else if(flag == 1){
                ConfigData = configService.getConfig(SN,Type).getMessage();
                gcconfig = new GWConfigWorker(Type,SN,ConfigData, true);
            }

            gcconfig.SetMqttConfig(mqttconfig.getUrl(),mqttconfig.getPort(),
                    mqttconfig.getUsername(), mqttconfig.getPassword(),mqttconfig.getQos(),
                    mqttconfig.getCacrt(),mqttconfig.getClientkey(), mqttconfig.getClientcrt(),mqttconfig.getClientpwd());
            result = configthreadPool.submit(gcconfig);

//            while (result.isDone())
//            {
//                String respcmd = result.get();
//            }
//            futureMap.put(timestamp.toString(),result);

        } catch (JsonSyntaxException e) {
            e.printStackTrace();
            System.out.println("mqttAsyncClient: message " + Message + " received syntax error.");
        } catch (IllegalStateException e) {
            e.printStackTrace();
            System.out.println("mqttAsyncClient: " + e);
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println(e.toString());
        }
    }

    @Override
    public void deliveryComplete(IMqttDeliveryToken iMqttDeliveryToken) {

    }

    //checkIn数据解析,true即发送休眠信息；false发送config信息
    public Integer parseData(String message,String SN,String Type){
        CheckInVo checkInVo = new Gson().fromJson(message,CheckInVo.class);
        if(StringUtils.isEmpty(checkInVo.getDevList()==null ? "" :checkInVo.getDevList().toString())){
            return 1;//重发
        }
//        有设备信息
        ConfigVo configVo = configService.getConfig(SN,Type);
        ConfigCMDVo configCMDVo = new Gson().fromJson(configVo.getMessage(),ConfigCMDVo.class);
        if(configVo.getUntie()){
            //解绑状态
            return 2;//解绑
        }
        //判断两者的devList是否一致
        if (configCMDVo.getDevList().containsAll(checkInVo.getDevList())
                && checkInVo.getDevList().containsAll(configCMDVo.getDevList())
                && checkInVo.getGprsPeriod().equals(configCMDVo.getGprsPeriod())
                && checkInVo.getGpsPeriod().equals(configCMDVo.getGpsPeriod())
                && checkInVo.getMode().equals(configCMDVo.getMode())) {
            //一致
            return 0;//发送休眠信息
        } else {
            //不一致
            return 1;//重发
        }
    }

}
