package com.example.tdl.service.redis;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.redis.core.HashOperations;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.io.Serializable;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.TimeUnit;

@Service
public class RedisService {
    final static Logger logger = LoggerFactory.getLogger(RedisService.class);

    protected RedisTemplate redisTemplate;

    public RedisService(RedisTemplate redisTemplate) {
    	this.redisTemplate = redisTemplate;
    }

    @SuppressWarnings({ "unchecked", "rawtypes" })
    public boolean setHash(String key, Map map, long expireTime) {
        boolean result = false;
        try {
            HashOperations<Serializable, Object, Object> hash = redisTemplate.opsForHash();
            hash.putAll(key, map);
            redisTemplate.expire(key, expireTime, TimeUnit.SECONDS);
            result = true;
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("redis插入hash值发生错误", this);
        }
        return result;
    }
    
    public boolean setHash(String key, String hashKey, String value, long expireTime) {
        boolean result = false;
        try {
            HashOperations<Serializable, Object, Object> hash = redisTemplate.opsForHash();
            hash.put(key, hashKey, value);
            redisTemplate.expire(key, expireTime, TimeUnit.SECONDS);
            result = true;
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("redis插入hash值发生错误", this);
        }
        return result;
    }
    
    public boolean setHashs(String key, Map<? extends String, ? extends Object> map, long expireTime) {
        boolean result = false;
        try {
            HashOperations<Serializable, Object, Object> hash = redisTemplate.opsForHash();
            hash.putAll(key, map);
            redisTemplate.expire(key, expireTime, TimeUnit.SECONDS);
            result = true;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return result;
    }

    public Object getHash(String key, Object mapKey) {
        Object result = null;
        try {
            HashOperations<Serializable, Object, Object> hash = redisTemplate.opsForHash();
            result = hash.get(key, mapKey);
        }catch (Exception e){
            e.printStackTrace();
            logger.error("redis获取hash值发生错误", this);
        }
        return result;
    }

    public Object getHashKeys(String key) {
        String ret="{error}";
        Object result = ret;
        try {
            HashOperations<String, Object, Object> hash= redisTemplate.opsForHash();
            //HashOperations<Serializable, Object, Object> hash = redisTemplate.opsForHash();
            result = hash.keys(key);
        }catch (Exception e){
            e.printStackTrace();
            logger.error("redis获取hash 所有key发生错误", this);
        }
        return result;
    }

    public Map<Object, Object> getHashAll(String key) {
    	Map<Object, Object> result = null;
        try {
            HashOperations<Serializable, Object, Object> hash = redisTemplate.opsForHash();
            result = hash.entries(key);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("redis获取hash 所有key value发生错误", this);
        }
        return result;
    }

    public boolean hasHashKey(String key, String Hashkey) {
    	boolean result = false;
        try {
            HashOperations<Serializable, Object, Object> hash = redisTemplate.opsForHash();
            result = hash.hasKey(key, Hashkey);
        }catch (Exception e){
            e.printStackTrace();
            logger.error("判断key值是否存在", this);
        }
        return result;
    }

    public long delHashKey(String key, String Hashkey) {
        long result = 0;
        try {
            HashOperations<Serializable, Object, Object> hash = redisTemplate.opsForHash();
            result = hash.delete(key, Hashkey);
        }catch (Exception e){
            e.printStackTrace();
            logger.error("删除指定key值", this);
        }
        return result;
    }

    public Object getAllKeys() {
        Object ret = null;
        try {
            ret = redisTemplate.keys("*");
        } catch (Exception e) {
            e.printStackTrace();
        }
        return ret;
    }

    public void remove(final String... keys) {
        for (String key : keys) {
            remove(key);
        }
    }

    public void removePattern(final String pattern) {
        Set<Serializable> keys = redisTemplate.keys(pattern);
        if (keys.size() > 0)
            redisTemplate.delete(keys);
    }

    public void remove(final String key) {
        if (exists(key)) {
            redisTemplate.delete(key);
        }
    }

    public boolean exists(final String key) {
        return redisTemplate.hasKey(key);
    }

    public String get(final String key) {
        String result = null;
        try {
            ValueOperations<Serializable, String> operations = redisTemplate.opsForValue();
            result = operations.get(key);
            return result;
        }catch (Exception e){
            e.printStackTrace();
        }
        return result;
    }

    public boolean set(final String key, Object value) {
        boolean result = false;
        try {
            ValueOperations<Serializable, Object> operations = redisTemplate.opsForValue();
            operations.set(key, value);
            result = true;
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("redis插入String失败", this);
        }
        return result;
    }

    public boolean set(final String key, Object value, long expireTime) {
        boolean result = false;
        try {
            ValueOperations<Serializable, Object> operations = redisTemplate.opsForValue();
            operations.set(key, value);
            redisTemplate.expire(key, expireTime, TimeUnit.SECONDS);
            result = true;
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("redis插入String失败", this);
        }
        return result;
    }

    public boolean refeshTimeOut(String key, long expireTime) {
        boolean f = false;
        try {
            redisTemplate.expire(key, expireTime, TimeUnit.SECONDS);
            f = true;
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("redis更新过期时间失败", this);
        }
        return f;
    }

    public void delKey(final String... key){
        if(key!=null && key.length > 0){
            if(key.length == 1){
                redisTemplate.delete(key[0]);
            }else{
                redisTemplate.delete(CollectionUtils.arrayToList(key));
            }
        }
    }

    public void hmSet(String key, Object hashKey, Object value) {
        HashOperations<String, Object, Object> hash = redisTemplate.opsForHash();
        hash.put(key, hashKey, value);
    }

}
