package com.example.tdl.web;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.AddAlarmVo;
import com.example.tdl.domain.vo.DelAlarmVo;
import com.example.tdl.domain.vo.UpdateAlarmVo;
import com.example.tdl.service.AlarmService;

import com.example.tdl.service.WarehouseService;
import com.example.tdl.service.redis.TokenRedisService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;


@RestController
@RequestMapping("/alarm")
public class AlarmController {

    private Logger logger = LoggerFactory.getLogger(AlarmController.class);

    CommFeedback fb=new CommFeedback();

    Gson gson=new Gson();

    @Autowired
    private AlarmService alarmService=new AlarmService();

    @Autowired
    private WarehouseService warehouseService;

    @Autowired
    private TokenRedisService tokenRedisService;

    //查询所有预警信息
    @ApiOperation(value = "查询所有预警信息",notes = "查询所有预警信息,返回值说明：" +
            "   alarmType：预警类型，" +
            "   alarmScene：预警场景，" +
            "   temMax：温度上限，" +
            "   temMin：温度下限，" +
            "   humidityMax：湿度上限，" +
            "   humidityMin：湿度下限，" +
            "   tilt：倾斜度，" +
            "   shock：震度，" +
            "   remark：备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/getAll",method = RequestMethod.GET)
    public Object getAll(){
        return alarmService.getAll();
    }

    //根据预警类型模糊查询
    @ApiOperation(value = "根据预警类型模糊查询",notes = "查询所有预警信息,返回值说明：" +
            "   alarmType：预警类型，" +
            "   alarmScene：预警场景，" +
            "   temMax：温度上限，" +
            "   temMin：温度下限，" +
            "   humidityMax：湿度上限，" +
            "   humidityMin：湿度下限，" +
            "   tilt：倾斜度，" +
            "   shock：震度，" +
            "   remark：备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/getAllByAlarmType",method = RequestMethod.POST)
    public Object getAllByAlarmType(@RequestBody String alarmType){
        JSONObject jsonObject= JSON.parseObject(alarmType);
        alarmType=(String)jsonObject.get("alarmType");
        return alarmService.getAllByAlarmType(alarmType);
    }

    //添加预警信息
    @ApiOperation(value = "添加预警信息",notes = "添加预警信息,传递参数：" +
            "   alarmType：预警类型，" +
            "   alarmScene：预警场景，" +
            "   temMax：温度上限，" +
            "   temMin：温度下限，" +
            "   humidityMax：湿度上限，" +
            "   humidityMin：湿度下限，" +
            "   tilt：倾斜度，" +
            "   shock：震度，" +
            "   remark：备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/addAlarm",method = RequestMethod.POST)
    public Object addAlarm(@RequestBody AddAlarmVo addAlarmVo){
        if (StringUtils.isEmpty(addAlarmVo.getAlarmType())){
            fb.setCode(0);
            fb.setMessage("预警类型不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addAlarmVo.getAlarmScene())){
            fb.setCode(0);
            fb.setMessage("预警场景不能为空");
            return gson.toJson(fb);
        }
        if (!addAlarmVo.getAlarmScene().equals("仓库")&&!addAlarmVo.getAlarmScene().equals("运输")){
            fb.setCode(0);
            fb.setMessage("请选择正确的预警场景");
            return gson.toJson(fb);
        }
        if (addAlarmVo.getTemMax()== null){
            fb.setCode(0);
            fb.setMessage("温度上限不能为空");
            return gson.toJson(fb);
        }
        if (addAlarmVo.getTemMin()== null){
            fb.setCode(0);
            fb.setMessage("温度下限不能为空");
            return gson.toJson(fb);
        }
        if (addAlarmVo.getHumidityMax()==null){
            fb.setCode(0);
            fb.setMessage("湿度上限不能为空");
            return gson.toJson(fb);
        }
        if (addAlarmVo.getHumidityMin()==null){
            fb.setCode(0);
            fb.setMessage("湿度下限不能为空");
            return gson.toJson(fb);
        }
        if (addAlarmVo.getTilt()==null){
            fb.setCode(0);
            fb.setMessage("倾斜度不能为空");
            return gson.toJson(fb);
        }
        if (addAlarmVo.getShock()==null){
            fb.setCode(0);
            fb.setMessage("震度不能为空");
            return gson.toJson(fb);
        }
        if (alarmService.getByTypeAndScene(addAlarmVo.getAlarmType(),addAlarmVo.getAlarmScene())!=null){
            fb.setCode(0);
            fb.setMessage("添加的预警类型在该场景中已存在");
            return gson.toJson(fb);
        }
        int a=alarmService.addAlarm(addAlarmVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage("添加预警信息成功");
        }else{
            fb.setCode(0);
            fb.setMessage("添加预警信息失败");
        }
        return gson.toJson(fb);
    }

    //修改预警信息
    @ApiOperation(value = "修改预警信息",notes = "修改预警信息,传递参数：" +
            "   oldAlarmType：需要修改的预警类型，" +
            "   oldAlarmScene：需要修改的预警场景，" +
            "   alarmType：预警类型，" +
            "   alarmScene：预警场景，" +
            "   temMax：温度上限，" +
            "   temMin：温度下限，" +
            "   humidityMax：湿度上限，" +
            "   humidityMin：湿度下限，" +
            "   tilt：倾斜度，" +
            "   shock：震度，" +
            "   remark：备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/updateAlarm",method = RequestMethod.POST)
    public Object updateAlarm(@RequestBody UpdateAlarmVo updateAlarmVo){
        if (StringUtils.isEmpty(updateAlarmVo.getOldAlarmType())){
            fb.setCode(0);
            fb.setMessage("需要修改的预警类型不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateAlarmVo.getOldAlarmScene())){
            fb.setCode(0);
            fb.setMessage("需要修改的预警场景不能为空");
            return gson.toJson(fb);
        }else if (!updateAlarmVo.getOldAlarmScene().equals("仓库")&&!updateAlarmVo.getOldAlarmScene().equals("运输")){
            fb.setCode(0);
            fb.setMessage("请输入正确的需要修改预警场景");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateAlarmVo.getAlarmType())){
            fb.setCode(0);
            fb.setMessage("预警类型不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateAlarmVo.getAlarmScene())){
            fb.setCode(0);
            fb.setMessage("预警场景不能为空");
            return gson.toJson(fb);
        }
        if (!updateAlarmVo.getAlarmScene().equals("仓库")&&!updateAlarmVo.getAlarmScene().equals("运输")){
            fb.setCode(0);
            fb.setMessage("请选择正确的预警场景");
            return gson.toJson(fb);
        }
        if (updateAlarmVo.getTemMax()==null){
            fb.setCode(0);
            fb.setMessage("温度上限不能为空");
            return gson.toJson(fb);
        }
        if (updateAlarmVo.getTemMin()==null){
            fb.setCode(0);
            fb.setMessage("温度下限不能为空");
            return gson.toJson(fb);
        }
        if (updateAlarmVo.getHumidityMax()==null){
            fb.setCode(0);
            fb.setMessage("湿度上限不能为空");
            return gson.toJson(fb);
        }
        if (updateAlarmVo.getHumidityMin()==null){
            fb.setCode(0);
            fb.setMessage("湿度下限不能为空");
            return gson.toJson(fb);
        }
        if (updateAlarmVo.getTilt()==null){
            fb.setCode(0);
            fb.setMessage("倾斜度不能为空");
            return gson.toJson(fb);
        }
        if (updateAlarmVo.getShock()==null){
            fb.setCode(0);
            fb.setMessage("震度不能为空");
            return gson.toJson(fb);
        }
        if (alarmService.getByTypeAndScene(updateAlarmVo.getAlarmType(),updateAlarmVo.getAlarmType())!=null&&!updateAlarmVo.getAlarmType().equals(updateAlarmVo.getAlarmType())&&!updateAlarmVo.getAlarmScene().equals(updateAlarmVo.getAlarmScene())){
            fb.setCode(0);
            fb.setMessage("修改的预警类型在该场景中已存在");
            return gson.toJson(fb);
        }
        int a=alarmService.updateAlarm(updateAlarmVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage("修改预警信息成功");
        }else{
            fb.setCode(0);
            fb.setMessage("修改预警信息失败");
        }
        return gson.toJson(fb);

    }

    //删除预警信息
    @ApiOperation(value = "删除预警信息",notes = "删除预警信息,传值说明：" +
            "   alarmType：需要删除的预警类型，" +
            "   alarmScene：需要删除的预警场景" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/delAlarm",method = RequestMethod.POST)
    public Object delAlarm(@RequestBody DelAlarmVo delAlarmVo){
        if (StringUtils.isEmpty(delAlarmVo.getAlarmScene())){
            fb.setCode(0);
            fb.setMessage("需要删除的预警场景不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(delAlarmVo.getAlarmType())){
            fb.setCode(0);
            fb.setMessage("需要删除的预警类型不能为空");
            return gson.toJson(fb);
        }
        if (warehouseService.getByAlarm(delAlarmVo)!=null){
            fb.setCode(0);
            fb.setMessage("该预警正在被使用");
            return gson.toJson(fb);
        }
        int a=alarmService.delAlarm(delAlarmVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage("删除预警信息成功");
        }else{
            fb.setCode(0);
            fb.setMessage("删除预警信息失败");
        }
        return gson.toJson(fb);
    }

    //查询仓库场景下的预警类型
    @ApiOperation(value = "查询仓库场景下的预警类型",notes = "查询仓库场景下的预警类型")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/getForWarehouse",method = RequestMethod.GET)
    public Object getForWarehouse(){
        return alarmService.getForWarehouse();
    }

}
