package com.example.tdl.web;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.*;
import com.example.tdl.service.*;
import com.example.tdl.service.redis.TokenRedisService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/circuit")
public class CircuitController {

    private Logger logger = LoggerFactory.getLogger(CircuitController.class);

    CommFeedback fb=new CommFeedback();

    Gson gson=new Gson();

    @Autowired
    private CircuitService circuitService;

    @Autowired
    private GatewayService gatewayService;

    @Autowired
    private TDLDeviceService tdlDeviceService;

    @Autowired
    private CircuitTransferService circuitTransferService;

    @Autowired
    private AlarmLogService alarmLogService;

    //获取线路信息
    @ApiOperation(value = "获取线路信息",notes = "获取线路信息，返回值说明：" +
            "   transportationNo:运输编号," +
            "   cargoNo:货物编号" +
            "   startCity:起运城市," +
            "   startTime:起运时间," +
            "   endCity:目的城市," +
            "   endTime:目的时间," +
            "   circuitState:线路状态（0未开始，1运输中，2完成）," +
            "   alarmType:预警类型," +
            "   alarm:报警详情（比如：温度,湿度,震动）" +
            "   evaluate:运输评价")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/getAll",method = RequestMethod.GET)
    public Object getAll(){
        return circuitService.getAll();
    }



    //根据条件查询
    @ApiOperation(value = "根据条件查询",notes = "根据条件查询:" +
            "   cargoNo:货物编号" +
            "   startTime:出发时间,（时间戳）" +
            "   endTime:目的时间,（时间戳）" +
            "   circuitState:线路状态（0未开始，1运输中，2完成）,"+
            "返回值说明：" +
            "   transportationNo:运输编号," +
            "   cargoNo:货物编号" +
            "   startCity:起运城市," +
            "   startTime:起运时间," +
            "   endCity:目的城市," +
            "   endTime:目的时间," +
            "   alarmType:预警类型," +
            "   circuitState:线路状态（0未开始，1运输中，2完成）," +
            "  alarm:报警详情（比如：温度,湿度,震动，" +
            "   evaluate:运输评价")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/getByTerm",method = RequestMethod.POST)
    public Object getByTerm(@RequestBody CircuitTermVo circuitTermVo){
        if(StringUtils.isEmpty(circuitTermVo.getCargoNo())){
            circuitTermVo.setCargoNo(null);
        }
        if(circuitTermVo.getCircuitState() == null){
            circuitTermVo.setCircuitState(null);
        }
        if(circuitTermVo.getStartTime()==null){
            circuitTermVo.setStartTime(null);
        } if(circuitTermVo.getEndTime()==null){
            circuitTermVo.setEndTime(null);
        }
        return circuitService.getByTerm(circuitTermVo);
    }


    //添加线路信息
    @ApiOperation(value = "添加线路信息",notes = "添加线路信息，" +
            "       startAddressVo: 线路集合，包含" +
            "{country:国家" +
            "city:城市" +
            "addressDetail:详细地址" +
            "lng:经度" +
            "lat:纬度" +
            "postCode:邮编"+
            "expTime:预计开始时间,（传时间戳）}" +
            "   addressVoList: 线路集合，包含" +
            "{country:国家" +
            "city:城市" +
            "addressDetail:详细地址" +
            "lng:经度" +
            "lat:纬度" +
            "postCode:邮编"+
            "expTime:预计开始时间,（传时间戳）" +
            "compTime:预计结束时间,（传时间戳）}" +
            "   endAddressVo: 线路集合，包含" +
            "{country:国家" +
            "city:城市" +
            "addressDetail:详细地址" +
            "lng:经度" +
            "lat:纬度" +
            "postCode:邮编"+
            "compTime:预计结束时间,（传时间戳）}" +
            "   gSN:网关编号," +
            "   tdlAlarmList：   " +
            "{TDLSN:tdl编号," +
            " alarmType:报警类型,}" +
            "   cargoNo:货物编号," +
            "   plateNo:车牌号," +
            "   cargoName:货物别名," +
            "   remark:备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/addCircuit",method = RequestMethod.POST)
    public Object addCircuit(@RequestBody AddCircuitVo addCircuitVo){
        if(StringUtils.isEmpty(addCircuitVo.getStartAddressVo().getCountry())){
            fb.setCode(0);
            fb.setMessage("出发国家不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCircuitVo.getStartAddressVo().getCity())){
            fb.setCode(0);
            fb.setMessage("出发城市不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCircuitVo.getStartAddressVo().getAddressDetail())){
            fb.setCode(0);
            fb.setMessage("出发具体地址不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCircuitVo.getStartAddressVo().getLat())){
            fb.setCode(0);
            fb.setMessage("出发地纬度不能为空");
            return gson.toJson(fb);
        }
        if(!addCircuitVo.getStartAddressVo().getLat().matches("^-?(([1-9])((\\.\\d{1,6})?)|([1-8]\\d)((\\.\\d{1,6})?)|90)$")){
            fb.setCode(0);
            fb.setMessage("出发地纬度的范围为-90~90");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCircuitVo.getStartAddressVo().getLng())){
            fb.setCode(0);
            fb.setMessage("出发地经度不能为空");
            return gson.toJson(fb);
        }
        if(!addCircuitVo.getStartAddressVo().getLng().matches("^-?(([1-9]\\d?)((\\.\\d{1,6})?)|(1[1-7]\\d)((\\.\\d{1,6})?)|180)$")) {
            fb.setCode(0);
            fb.setMessage("出发地经度的范围为-180~180");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCircuitVo.getStartAddressVo().getPostCode())){
            fb.setCode(0);
            fb.setMessage("出发地邮编不能为空");
            return gson.toJson(fb);
        }
        if (addCircuitVo.getStartAddressVo().getExpTime()==null){
            fb.setCode(0);
            fb.setMessage("出发地预计出发时间不能为空");
            return gson.toJson(fb);
        }
        Long time = System.currentTimeMillis();
        String countryVar=addCircuitVo.getStartAddressVo().getCountry()+" ";
        String cityVar=addCircuitVo.getStartAddressVo().getCity()+ " ";
        String addressDetailVar=addCircuitVo.getStartAddressVo().getAddressDetail() +" ";
        String lngVar=addCircuitVo.getStartAddressVo().getLng()+" ";
        String latVar = addCircuitVo.getStartAddressVo().getLat()+" ";
        String postCodeVar = addCircuitVo.getStartAddressVo().getPostCode()+" ";
        String expTimeVar = addCircuitVo.getStartAddressVo().getExpTime()+" ";
        String compTimeVar = "0 ";
        String arrivalTimeVar=time +" ";
        String startTimeVar=time +" ";
        if(addCircuitVo.getAddressVoList() != null){
            if(addCircuitVo.getAddressVoList().size()>3){
                fb.setCode(0);
                fb.setMessage("途径地不能超过3个");
                return gson.toJson(fb);
            }
            for(int i =0;i<addCircuitVo.getAddressVoList().size();i++){
                if(StringUtils.isEmpty(addCircuitVo.getAddressVoList().get(i).getCountry())){
                    fb.setCode(0);
                    fb.setMessage("途径地国家不能为空");
                    return gson.toJson(fb);
                }
                if (StringUtils.isEmpty(addCircuitVo.getAddressVoList().get(i).getCity())){
                    fb.setCode(0);
                    fb.setMessage("途径地城市不能为空");
                    return gson.toJson(fb);
                }
                if (StringUtils.isEmpty(addCircuitVo.getAddressVoList().get(i).getAddressDetail())){
                    fb.setCode(0);
                    fb.setMessage("途径地具体地址不能为空");
                    return gson.toJson(fb);
                }
                if (StringUtils.isEmpty(addCircuitVo.getAddressVoList().get(i).getLat())){
                    fb.setCode(0);
                    fb.setMessage("途径地纬度不能为空");
                    return gson.toJson(fb);
                }
                if(!addCircuitVo.getAddressVoList().get(i).getLat().matches("^-?(([1-9])((\\.\\d{1,6})?)|([1-8]\\d)((\\.\\d{1,6})?)|90)$")){
                    fb.setCode(0);
                    fb.setMessage("途径地纬度的范围为-90~90");
                    return gson.toJson(fb);
                }
                if (StringUtils.isEmpty(addCircuitVo.getAddressVoList().get(i).getLng())){
                    fb.setCode(0);
                    fb.setMessage("途径地经度不能为空");
                    return gson.toJson(fb);
                }
                if(!addCircuitVo.getAddressVoList().get(i).getLng().matches("^-?(([1-9]\\d?)((\\.\\d{1,6})?)|(1[1-7]\\d)((\\.\\d{1,6})?)|180)$")) {
                    fb.setCode(0);
                    fb.setMessage("途径地经度的范围为-180~180");
                    return gson.toJson(fb);
                }
                if (StringUtils.isEmpty(addCircuitVo.getAddressVoList().get(i).getPostCode())){
                    fb.setCode(0);
                    fb.setMessage("途径地邮编不能为空");
                    return gson.toJson(fb);
                }
                if (addCircuitVo.getAddressVoList().get(i).getExpTime() == null){
                    fb.setCode(0);
                    fb.setMessage("途径地预计出发时间不能为空");
                    return gson.toJson(fb);
                }
                if (addCircuitVo.getAddressVoList().get(i).getCompTime() == null){
                    fb.setCode(0);
                    fb.setMessage("途径地预计到达时间不能为空");
                    return gson.toJson(fb);
                }
                countryVar=countryVar +addCircuitVo.getAddressVoList().get(i).getCountry() +" ";
                cityVar=cityVar +addCircuitVo.getAddressVoList().get(i).getCity() +" ";
                addressDetailVar =addressDetailVar +addCircuitVo.getAddressVoList().get(i).getAddressDetail() +" ";
                lngVar = lngVar + addCircuitVo.getAddressVoList().get(i).getLng() +" ";
                latVar = latVar + addCircuitVo.getAddressVoList().get(i).getLat() +" ";
                postCodeVar=postCodeVar + addCircuitVo.getAddressVoList().get(i).getPostCode() +" ";
                expTimeVar = expTimeVar +addCircuitVo.getAddressVoList().get(i).getExpTime() +" ";
                compTimeVar  = compTimeVar  +addCircuitVo.getAddressVoList().get(i).getCompTime() +" ";
                startTimeVar =startTimeVar + "0 ";
                arrivalTimeVar = arrivalTimeVar +"0 ";
            }
        }
        countryVar=countryVar +addCircuitVo.getEndAddressVo().getCountry() +" ";
        cityVar=cityVar +addCircuitVo.getEndAddressVo().getCity() +" ";
        addressDetailVar =addressDetailVar +addCircuitVo.getEndAddressVo().getAddressDetail() +" ";
        lngVar = lngVar + addCircuitVo.getEndAddressVo().getLng() +" ";
        latVar = latVar + addCircuitVo.getEndAddressVo().getLat() +" ";
        postCodeVar=postCodeVar + addCircuitVo.getEndAddressVo().getPostCode() +" ";
        expTimeVar = expTimeVar +"0 ";
        compTimeVar  = compTimeVar  +addCircuitVo.getEndAddressVo().getCompTime() +" ";
        startTimeVar =startTimeVar + "0 ";
        arrivalTimeVar = arrivalTimeVar +"0 ";

        if(StringUtils.isEmpty(addCircuitVo.getEndAddressVo().getCountry())){
            fb.setCode(0);
            fb.setMessage("目的地国家不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCircuitVo.getEndAddressVo().getCity())){
            fb.setCode(0);
            fb.setMessage("目的地城市不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCircuitVo.getEndAddressVo().getAddressDetail())){
            fb.setCode(0);
            fb.setMessage("目的地具体地址不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCircuitVo.getEndAddressVo().getLat())){
            fb.setCode(0);
            fb.setMessage("目的地纬度不能为空");
            return gson.toJson(fb);
        }
        if(!addCircuitVo.getEndAddressVo().getLat().matches("^-?(([1-9])((\\.\\d{1,6})?)|([1-8]\\d)((\\.\\d{1,6})?)|90)$")){
            fb.setCode(0);
            fb.setMessage("目的地纬度的范围为-90~90");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCircuitVo.getEndAddressVo().getLng())){
            fb.setCode(0);
            fb.setMessage("目的地经度不能为空");
            return gson.toJson(fb);
        }
        if(!addCircuitVo.getEndAddressVo().getLng().matches("^-?(([1-9]\\d?)((\\.\\d{1,6})?)|(1[1-7]\\d)((\\.\\d{1,6})?)|180)$")) {
            fb.setCode(0);
            fb.setMessage("目的地经度的范围为-180~180");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCircuitVo.getEndAddressVo().getPostCode())){
            fb.setCode(0);
            fb.setMessage("目的地邮编不能为空");
            return gson.toJson(fb);
        }
        if (addCircuitVo.getEndAddressVo().getCompTime()==null){
            fb.setCode(0);
            fb.setMessage("目的地预计到达时间不能为空");
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(addCircuitVo.getSN())){
            fb.setCode(0);
            fb.setMessage("网关序列号不能为空");
            return gson.toJson(fb);
        }

        String type =gatewayService.getBySN(addCircuitVo.getSN());
        //判断当前gateWay是否存在
        if(StringUtils.isEmpty(type)){
            fb.setCode(0);
            fb.setMessage("该网关序不存在");
            return gson.toJson(fb);
        }
        //获取gateway的场景
        if(gatewayService.getBySNAndType(addCircuitVo.getSN(),type).getUseScene()== 2){
            fb.setCode(0);
            fb.setMessage("该网关的使用场景与当前不符");
            return gson.toJson(fb);
        }
        String alarmTypeVar="";
        String TDLSNVar="";
        List<String> devList = tdlDeviceService.getByGatewaySN(addCircuitVo.getSN(),type);
        for(int j = 0;j<addCircuitVo.getTdlAlarmList().size();j++){
            if(StringUtils.isEmpty(addCircuitVo.getTdlAlarmList().get(j).getTDLSN())){
                fb.setCode(0);
                fb.setMessage("tag序列号不能为空");
                return gson.toJson(fb);
            }
            //判断当前tdl是否存在
            ResultTDLDeviceVo tdlDeviceVo = tdlDeviceService.getByTDLSN(addCircuitVo.getTdlAlarmList().get(j).getTDLSN());
           if(tdlDeviceVo == null){
               fb.setCode(0);
               fb.setMessage(addCircuitVo.getTdlAlarmList().get(j).getTDLSN() +"不存在");
               return gson.toJson(fb);
           }
           if(!StringUtils.isEmpty(tdlDeviceVo.getGatewaySN())){
               fb.setCode(0);
               fb.setMessage(addCircuitVo.getTdlAlarmList().get(j).getTDLSN() +"已经被使用");
               return gson.toJson(fb);
           }

            if(StringUtils.isEmpty(addCircuitVo.getTdlAlarmList().get(j).getAlarmType())){
                fb.setCode(0);
                fb.setMessage("预警类型");
                return gson.toJson(fb);
            }
            alarmTypeVar =alarmTypeVar + addCircuitVo.getTdlAlarmList().get(j).getAlarmType() +" ";
            TDLSNVar =TDLSNVar + addCircuitVo.getTdlAlarmList().get(j).getTDLSN() +" ";
            devList.add(addCircuitVo.getTdlAlarmList().get(j).getTDLSN());
        }

        if(StringUtils.isEmpty(addCircuitVo.getCargoNo())){
            fb.setCode(0);
            fb.setMessage("货物编号不能为空");
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(addCircuitVo.getPlateNo())){
            fb.setCode(0);
            fb.setMessage("车牌号不能为空");
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(addCircuitVo.getCargoName())){
            fb.setCode(0);
            fb.setMessage("货物别名不能为空");
            return gson.toJson(fb);
        }
        //根据sn获取type
        ConfigCMDVo configCMDVo = new ConfigCMDVo("config",5,15,devList,1);
        Map<Object,Object> map=new HashMap<>();
        String transportation = "汽运";
        String transportationType="本地运输";
        map.put("transportationNo",getTransportNo(transportation,transportationType));
        map.put("startTime",time);
        map.put("expTime",addCircuitVo.getStartAddressVo().getExpTime());
        map.put("compTime",addCircuitVo.getEndAddressVo().getCompTime());
        map.put("cargoNo",addCircuitVo.getCargoNo());
        map.put("cargoName",addCircuitVo.getCargoName());
        map.put("plateNo",addCircuitVo.getPlateNo());
        map.put("deliveryAddress","站点");
        map.put("transportation",transportation);
        map.put("transportationType",transportationType);
        map.put("createTime",time);
        map.put("remark",addCircuitVo.getRemark());
        map.put("counts",addCircuitVo.getAddressVoList()== null ?  0 : addCircuitVo.getAddressVoList().size());
        map.put("countryVar",countryVar);
        map.put("cityVar",cityVar);
        map.put("addressDetailVar",addressDetailVar);
        map.put("lngVar",lngVar);
        map.put("latVar",latVar);
        map.put("postCodeVar",postCodeVar);
        map.put("arrivalTimeVar",arrivalTimeVar);
        map.put("startTimeVar",startTimeVar);
        map.put("expTimeVar",expTimeVar);
        map.put("compTimeVar",compTimeVar);
        map.put("SN",addCircuitVo.getSN());
        map.put("type",type);
        map.put("TDLSNVar",TDLSNVar);
        map.put("alarmTypeVar",alarmTypeVar);
        map.put("TDLCounts",addCircuitVo.getTdlAlarmList()== null ?  0 : addCircuitVo.getTdlAlarmList().size());
        map.put("message",gson.toJson(configCMDVo));
        map.put("flag",false);
        map.put("untie",false);
        circuitService.addCircuit(map);
        Map<Object,Object> msg=new HashMap<>();
        msg.put("msg",map.get("msg"));
        if (msg.get("msg").equals("添加线路成功")){
            fb.setCode(1);
            fb.setMessage("添加线路成功");
        }else {
            fb.setCode(0);
            fb.setMessage(msg.get("msg").toString());
        }
        return gson.toJson(fb);
    }


    //修改线路结束时间
    @ApiOperation(value = "修改线路结束时间",notes = "修改线路结束时间，传值说明：" +
            "   transportationNo:运输编号," +
            "   endTime:目的时间 " )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/updateEndTime",method = RequestMethod.POST)
    public Object updateEndTime(@RequestBody UpdateCircuitVo updateCircuitVo){
        if (StringUtils.isEmpty(updateCircuitVo.getTransportationNo())){
            fb.setCode(0);
            fb.setMessage("需要修改的运输编号不能为空");
            return gson.toJson(fb);
        }
        if (updateCircuitVo.getEndTime()==null){
            fb.setCode(0);
            fb.setMessage("线路结束时间不能为空");
            return gson.toJson(fb);
        }
        if (circuitService.getByTransportationNo(updateCircuitVo.getTransportationNo()).getCircuitState()!=2){
            fb.setCode(0);
            fb.setMessage("线路未完成");
            return gson.toJson(fb);
        }
        int a = circuitService.updateEndTime(updateCircuitVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage("修改线路结束时间成功");
        }else{
            fb.setCode(0);
            fb.setMessage("修改线路结束时间失败");
        }
        return gson.toJson(fb);
    }


    //根据车牌号查线路
    @ApiOperation(value = "根据车牌号查线路",notes = "根据车牌号查线路，返回值说明：" +
            "   transportationNo:运输编号," +
            "   cargoNo:货物编号" +
            "   startCity:起运城市," +
            "   startTime:起运时间," +
            "   endCity:目的城市," +
            "   endTime:目的时间," +
            "   alarmType:预警类型," +
            "   circuitState:线路状态（0未开始，1运输中，2完成）," +
            "   alarm:报警详情（比如：温度,湿度,震动）" +
            "   evaluate:运输评价")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/getByContainerNo",method = RequestMethod.POST)
    public Object getByContainerNo(@RequestBody String cargoNo){
        JSONObject jsonObject= JSON.parseObject(cargoNo);
        cargoNo=(String)jsonObject.get("cargoNo");
        return circuitService.getByContainerNo(cargoNo);
    }



    //评价
    @ApiOperation(value = "评价",notes = "评价线路传值说明：" +
            "   transportationNo:运输编号," +
            "   evaluate:评分，" +
            "   evaluateRemark:评分备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/updateEvaluate",method = RequestMethod.POST)
    public Object updateEvaluate(@RequestBody UpdateEvaluateVo updateEvaluateVo){
        if (updateEvaluateVo.getEvaluate()==null){
            fb.setCode(0);
            fb.setMessage("评价的等级不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateEvaluateVo.getTransportationNo())){
            fb.setCode(0);
            fb.setMessage("需要评价的线路编号不能为空");
            return gson.toJson(fb);
        }
        if (circuitService.getByTransportationNo(updateEvaluateVo.getTransportationNo()).getCircuitState()!=2){
            fb.setCode(0);
            fb.setMessage("线路完成运输才能评价");
            return gson.toJson(fb);
        }
        int a = circuitService.updateEvaluate(updateEvaluateVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage("评价成功");
        }else{
            fb.setCode(0);
            fb.setMessage("评价失败");
        }
        return gson.toJson(fb);
    }



    //根据运输状态查询线路信息
    @ApiOperation(value = "根据运输状态查询线路信息",notes = "根据运输状态查询线路信息，传值说明：{\"circuitState\":2}  ,circuitState:线路状态（0未开始，1运输中，2完成），返回值说明：" +
            "   transportationNo:运输编号," +
            "   cargoNo:货物编号" +
            "   startCity:起运城市," +
            "   startTime:起运时间," +
            "   endCity:目的城市," +
            "   endTime:目的时间," +
            "   alarmType:预警类型," +
            "   circuitState:线路状态（0未开始，1运输中，2完成）," +
            "   alarm:报警详情（比如：温度,湿度,震动）" +
            "   evaluate:运输评价")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/getByCircuitState",method = RequestMethod.POST)
    public Object getByCircuitState(@RequestBody Map map){
        if (!map.containsKey("circuitState")){
            fb.setCode(0);
            fb.setMessage("需要查询的线路状态不能为空");
            return gson.toJson(fb);
        }
        return circuitService.getByCircuitState(Integer.valueOf(map.get("circuitState").toString()));
    }



    //删除线路
    @ApiOperation(value = "删除线路",notes = "删除线路，传值说明：" +
            "   transportationNo:运输编号" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/delCircuit",method = RequestMethod.POST)
    public Object delCircuit(@RequestBody String transportationNo){
        JSONObject jsonObject= JSON.parseObject(transportationNo);
        transportationNo=(String)jsonObject.get("transportationNo");
        if (StringUtils.isEmpty(transportationNo)){
            fb.setCode(0);
            fb.setMessage("需要删除的运输编号不能为空");
            return gson.toJson(fb);
        }
        if (circuitService.getByTransportationNo(transportationNo).getCircuitState()!=0){
            fb.setCode(0);
            fb.setMessage("该线路在运输中或已经完成运输");
            return gson.toJson(fb);
        }
        int a = circuitService.delCircuit(transportationNo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage("删除线路成功");
        }else{
            fb.setCode(0);
            fb.setMessage("删除线路失败");
        }
        return gson.toJson(fb);
    }


    //根据线路编号获取线路详细信息
    @ApiOperation(value = "根据线路编号获取线路详细信息",notes = "根据线路编号获取线路详细信息，传值说明：" +
            "   transportationNo:运输编号；   " +
            "返回值说明：" +
            "   transportationNo：运输编号" +
            "   cargoNo：货物编号" +
            "   startCity：出发城市" +
            "   endCity：目的城市" +
            "   circuitState：线路状态" +
            "   evaluate：评价" +
            "   list：" +
                "   {city：城市" +
                "   time：到达时间/发车时间" +
                "   last：运行时长" +
                "   alarmCount：报警数" +
                "   type：1--出发时间；2--目的时间}" +
            "   alarmLogList：" +
                "   transportationNo：运输编号" +
            "   TDLSN：tag序列号" +
            "   description：报警描述" +
            "   data：数值" +
            "   alarmTime：报警时间" +
            "   classify：分类" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @PostMapping("/getDetails")
    public Object getDetails(@RequestBody String transportationNo){
        JSONObject jsonObject= JSON.parseObject(transportationNo);
        transportationNo=(String)jsonObject.get("transportationNo");
        //获取线路信息
        ResultCircuitVo resultCircuitVo =  circuitService.getByTransportationNo(transportationNo);
        List<ResultAlarmLog> alarmLogList= alarmLogService.getByTransportationNo(transportationNo);
        CircuitMessageVo messageVo = new CircuitMessageVo(resultCircuitVo.getTransportationNo(),resultCircuitVo.getCargoNo(),resultCircuitVo.getStartCity(),resultCircuitVo.getEndCity(),resultCircuitVo.getCircuitState(),resultCircuitVo.getEvaluate(),alarmLogList);
        //获取线路情况
        List<CircuitTransferVo> list =circuitTransferService.getCity(transportationNo);
        List<ResultCityVo> cityVoList = new ArrayList<>();
        for (int i = 0; i < list.size(); i++) {
            if (list.get(i).getSequence() == 1) {
                ResultCityVo cityVo = new ResultCityVo();
                cityVo.setCity(list.get(i).getCity());
                cityVo.setType(1);
                if(list.get(i).getStartTime() == null || list.get(i).getStartTime() == 0 ){
                    cityVo.setTime(0l);
                    cityVo.setLast("0");
                    cityVo.setAlarmCount(0);
                }else{
                    cityVo.setTime(list.get(i).getStartTime());
                    if(list.get(i + 1).getArrivalTime() !=null && list.get(i + 1).getArrivalTime() !=0){
                        //根据时间和运输编号获取报警信息
                        Integer count = alarmLogService.getByTime(transportationNo, list.get(i).getStartTime(), list.get(i + 1).getArrivalTime());
                        cityVo.setAlarmCount(count);
                        cityVo.setLast(getMistiming(list.get(i + 1).getArrivalTime(),list.get(i).getStartTime()));
                    }else{
                        Integer count = alarmLogService.getByTime(transportationNo, list.get(i).getStartTime(), System.currentTimeMillis());
                        cityVo.setAlarmCount(count);
                        cityVo.setLast(getMistiming(System.currentTimeMillis(),list.get(i).getStartTime()));
                    }
                }
                cityVoList.add(cityVo);
            }else if(list.get(i).getSequence() == list.size()){
                ResultCityVo cityVo = new ResultCityVo();
                cityVo.setCity(list.get(i).getCity());
                if(list.get(i).getArrivalTime() == null  || list.get(i).getArrivalTime() == 0){
                    cityVo.setTime(0l);
                }else {
                    cityVo.setTime(list.get(i).getArrivalTime());
                }
                cityVo.setType(2);
                cityVoList.add(cityVo);
            }else{
                ResultCityVo cityVo2 = new ResultCityVo();
                cityVo2.setCity(list.get(i).getCity());
                cityVo2.setType(2);
                if(list.get(i).getArrivalTime() == null ||list.get(i).getArrivalTime() ==0){
                    cityVo2.setTime(0l);
                    cityVo2.setAlarmCount(0);
                    cityVo2.setLast("0");
                    cityVoList.add(cityVo2);
                    ResultCityVo cityVo1 = new ResultCityVo();
                    cityVo1.setCity(list.get(i).getCity());
                    cityVo1.setTime(0l);
                    cityVo1.setType(1);
                    cityVo1.setAlarmCount(0);
                    cityVo1.setLast("0");
                    cityVoList.add(cityVo1);
                }else{
                    cityVo2.setTime(list.get(i).getArrivalTime());
                    //根据时间和运输编号获取报警信息
                    if(list.get(i).getStartTime() == null ||list.get(i).getStartTime() ==0){
                        Integer count2 = alarmLogService.getByTime(transportationNo, list.get(i).getArrivalTime(), System.currentTimeMillis());
                        cityVo2.setAlarmCount(count2);
                        cityVo2.setLast(getMistiming(System.currentTimeMillis(),list.get(i).getArrivalTime()));
                        cityVoList.add(cityVo2);
                        ResultCityVo cityVo1 = new ResultCityVo();
                        cityVo1.setCity(list.get(i).getCity());
                        cityVo1.setTime(0l);
                        cityVo1.setType(1);
                        cityVo1.setAlarmCount(0);
                        cityVo1.setLast("0");
                        cityVoList.add(cityVo1);
                    }else{
                        Integer count2 = alarmLogService.getByTime(transportationNo, list.get(i).getArrivalTime(), list.get(i).getStartTime());
                        cityVo2.setAlarmCount(count2);
                        cityVo2.setLast(getMistiming(list.get(i ).getStartTime(),list.get(i).getArrivalTime()));
                        cityVoList.add(cityVo2);
                        ResultCityVo cityVo1 = new ResultCityVo();
                        cityVo1.setCity(list.get(i).getCity());
                        cityVo1.setTime(list.get(i).getStartTime());
                        cityVo1.setType(1);
                        if(list.get(i + 1).getArrivalTime() == null  ||list.get(i + 1).getArrivalTime() ==0){
                            Integer count = alarmLogService.getByTime(transportationNo, list.get(i).getStartTime(), System.currentTimeMillis());
                            cityVo1.setAlarmCount(count);
                            cityVo1.setLast(getMistiming(System.currentTimeMillis(),list.get(i).getStartTime()));
                            cityVoList.add(cityVo1);

                        }else {
                            Integer count = alarmLogService.getByTime(transportationNo, list.get(i).getStartTime(), list.get(i + 1).getArrivalTime());
                            cityVo1.setAlarmCount(count);
                            cityVo1.setLast(getMistiming(list.get(i + 1).getArrivalTime(),list.get(i).getStartTime()));
                            cityVoList.add(cityVo1);
                        }
                    }
                }
            }
        }
        messageVo.setList(cityVoList);
        return gson.toJson(messageVo);
    }


    //生成运输编号
    public String getTransportNo(String transportation,String transportationType){
        String transportationNo="";
        if(transportationType.equals("本地运输")){
            transportationNo = transportationNo+"CHN";
        }else{
            transportationNo = transportationNo+"HUN";
        }
        transportationNo=transportationNo+new SimpleDateFormat("yyyyMMdd").format(new Date());
        if(transportation.equals("汽运")){
            transportationNo = transportationNo +"G";
        } else if (transportation.equals("铁路")){
            transportationNo = transportationNo +"T";
        }else if (transportation.equals("空运")){
            transportationNo = transportationNo +"K";
        }else if (transportation.equals("快递")){
            transportationNo = transportationNo +"D";
        }else if (transportation.equals("海运")){
            transportationNo = transportationNo +"H";
        }
        transportationNo=transportationNo+((int)(Math.random()*(9999-1000+1))+1000);
        if (circuitService.getNo(transportationNo)>0){
            while (circuitService.getNo(transportationNo)>0){
                transportationNo=transportationNo+((int)(Math.random()*(9999-1000+1))+1000);
            }
        }
        return transportationNo;
    }

    //两个时间戳之间相差多少
    public String getMistiming(Long startTime,Long arrivalTime){
        long nd = 1000 * 24 * 60 * 60;
        long nh = 1000 * 60 * 60;
        long nm = 1000 * 60;
        long ns = 1000;
        // 获得两个时间的毫秒时间差异
        long diff = startTime- arrivalTime;
        // 计算差多少天
        long day = diff / nd;
        // 计算差多少小时
        long hour = diff % nd / nh;
        // 计算差多少分钟
        long min = diff % nd % nh / nm;
        // 计算差多少秒//输出结果
        long sec = diff % nd % nh % nm / ns;
        return  day + "天" + hour + "小时" + min + "分钟" +sec +"秒";
    }


    //    修改线路信息
//    @ApiOperation(value = "修改线路信息",notes = "修改线路信息，传值说明：" +
//            "   transportationNo:运输编号（自动生成，作为修改条件）" +
//            "   addressVoList: 线路集合，包含" +
//            "{sequence:顺序（出发地0，目的地从1依次排序）" +
//            "country:国家" +
//            "city:城市" +
//            "addressDetail:详细地址" +
//            "lng:经度" +
//            "lat:纬度" +
//            "postCode:邮编"+
//            "expTime:预计开始时间,（传时间戳）" +
//            "compTime:预计结束时间,（传时间戳）}" +
//            "   gSN:网关编号," +
//            "   TDLSN:tdl编号," +
//            "   alarmType:报警类型," +
//            "   cargoNo:货物编号," +
//            "   plateNo:车牌号," +
//            "   cargoName:货物别名," +
//            "   remark:备注")
//    @ApiImplicitParams({
//            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
//    })
//    @RequestMapping(value="/updateCircuit",method = RequestMethod.POST)
//    public Object updateCircuit(@RequestBody UpdateCircuitInfo updateCircuitInfo){
//        if (StringUtils.isEmpty(updateCircuitInfo.getTransportationNo())){
//            fb.setCode(0);
//            fb.setMessage("运输编号不能为空");
//            return gson.toJson(fb);
//        }
//        if(circuitService.getByTransportationNo(updateCircuitInfo.getTransportationNo()).getCircuitState()!=0){
//            fb.setCode(0);
//            fb.setMessage("只能在线路未开始运输时才能修改");
//            return gson.toJson(fb);
//        }
//        if(StringUtils.isEmpty(updateCircuitInfo.getSN())){
//            fb.setCode(0);
//            fb.setMessage("网关编号不能为空");
//            return gson.toJson(fb);
//        }
//        if (StringUtils.isEmpty(updateCircuitInfo.getTDLSN())){
//            fb.setCode(0);
//            fb.setMessage("TDL编号不能为空");
//            return gson.toJson(fb);
//        }
//        if (StringUtils.isEmpty(updateCircuitInfo.getAlarmType())){
//            fb.setCode(0);
//            fb.setMessage("报警类型不能为空");
//            return gson.toJson(fb);
//        }
//        if (StringUtils.isEmpty(updateCircuitInfo.getCargoNo())){
//            fb.setCode(0);
//            fb.setMessage("货物编号不能为空");
//            return gson.toJson(fb);
//        }
//        if (StringUtils.isEmpty(updateCircuitInfo.getPlateNo())){
//            fb.setCode(0);
//            fb.setMessage("车牌号不能为空");
//            return gson.toJson(fb);
//        }
//        if (StringUtils.isEmpty(updateCircuitInfo.getCargoName())){
//            fb.setCode(0);
//            fb.setMessage("货物别名不能为空");
//            return gson.toJson(fb);
//        }
//        if(updateCircuitInfo.getAddressVoList().size() ==0){
//            fb.setCode(0);
//            fb.setMessage("起始城市不能为空");
//            return gson.toJson(fb);
//        }
//        if(updateCircuitInfo.getAddressVoList().size()==1){
//            fb.setCode(0);
//            fb.setMessage("目的地不能为空");
//            return gson.toJson(fb);
//        }
//        if(updateCircuitInfo.getAddressVoList().size()>4) {
//            fb.setCode(0);
//            fb.setMessage("目的地不能超过3个");
//            return gson.toJson(fb);
//        }
//        String countryVar="";
//        String cityVar="";
//        String addressDetailVar="";
//        String lngVar="";
//        String latVar = "";
//        String postCodeVar = "";
//        String expTimeVar = "";
//        String compTimeVar = "";
//        String arrivalTimeVar="";
//        String startTimeVar="";
//        Long time = System.currentTimeMillis();
//        for( int i= 0; i<updateCircuitInfo.getAddressVoList().size() ; i++){
//            if (StringUtils.isEmpty(updateCircuitInfo.getAddressVoList().get(i).getCountry())){
//                fb.setCode(0);
//                fb.setMessage("国家不能为空");
//                return gson.toJson(fb);
//            }
//            if (StringUtils.isEmpty(updateCircuitInfo.getAddressVoList().get(i).getCity())){
//                fb.setCode(0);
//                fb.setMessage("城市不能为空");
//                return gson.toJson(fb);
//            }
//            if (StringUtils.isEmpty(updateCircuitInfo.getAddressVoList().get(i).getAddressDetail())){
//                fb.setCode(0);
//                fb.setMessage("详细地址不能为空");
//                return gson.toJson(fb);
//            }
//            if (StringUtils.isEmpty(updateCircuitInfo.getAddressVoList().get(i).getLng())){
//                fb.setCode(0);
//                fb.setMessage("经度不能为空");
//                return gson.toJson(fb);
//            }
//            if (StringUtils.isEmpty(updateCircuitInfo.getAddressVoList().get(i).getLat())){
//                fb.setCode(0);
//                fb.setMessage("纬度不能为空");
//                return gson.toJson(fb);
//            }
//            if(!updateCircuitInfo.getAddressVoList().get(i).getLat().matches("^-?(([1-9])((\\.\\d{1,6})?)|([1-8]\\d)((\\.\\d{1,6})?)|90)$")){
//                fb.setCode(0);
//                fb.setMessage("纬度的范围为-90~90");
//                return gson.toJson(fb);
//            }
//            if(!updateCircuitInfo.getAddressVoList().get(i).getLng().matches("^-?(([1-9]\\d?)((\\.\\d{1,6})?)|(1[1-7]\\d)((\\.\\d{1,6})?)|180)$")) {
//                fb.setCode(0);
//                fb.setMessage("经度的范围为-180~180");
//                return gson.toJson(fb);
//            }
//            if (StringUtils.isEmpty(updateCircuitInfo.getAddressVoList().get(i).getPostCode())){
//                fb.setCode(0);
//                fb.setMessage("邮编不能为空");
//                return gson.toJson(fb);
//            }
//            countryVar=countryVar +updateCircuitInfo.getAddressVoList().get(i).getCountry() +" ";
//            cityVar=cityVar +updateCircuitInfo.getAddressVoList().get(i).getCity() +" ";
//            addressDetailVar =addressDetailVar +updateCircuitInfo.getAddressVoList().get(i).getAddressDetail() +" ";
//            lngVar = lngVar + updateCircuitInfo.getAddressVoList().get(i).getLng() +" ";
//            latVar = latVar + updateCircuitInfo.getAddressVoList().get(i).getLat() +" ";
//            postCodeVar=postCodeVar + updateCircuitInfo.getAddressVoList().get(i).getPostCode() +" ";
//            if(updateCircuitInfo.getAddressVoList().get(i).getExpTime() ==null){
//                expTimeVar = expTimeVar + "0 ";
//            }else{
//                expTimeVar = expTimeVar +updateCircuitInfo.getAddressVoList().get(i).getExpTime() +" ";
//            }
//            if(updateCircuitInfo.getAddressVoList().get(i).getCompTime() == null){
//                compTimeVar  = compTimeVar  +"0 ";
//            }else{
//                compTimeVar  = compTimeVar  +updateCircuitInfo.getAddressVoList().get(i).getCompTime() +" ";
//            }
//            if(i ==0){
//                startTimeVar =startTimeVar + time +" ";
//                arrivalTimeVar = arrivalTimeVar +time +" ";
//            }else{
//                startTimeVar =startTimeVar + "0 ";
//                arrivalTimeVar = arrivalTimeVar +"0 ";
//            }
//        }
//        Map<Object,Object> map=new HashMap<>();
//        String transportation = "汽运";
//        String transportationType="本地运输";
//        map.put("transportationNo",updateCircuitInfo.getTransportationNo());
//        map.put("startTime",time);
//        map.put("expTime",updateCircuitInfo.getAddressVoList().get(0).getExpTime());
//        map.put("compTime",updateCircuitInfo.getAddressVoList().get(updateCircuitInfo.getAddressVoList().size()-1).getCompTime());
//        map.put("alarmType",updateCircuitInfo.getAlarmType());
//        map.put("cargoNo",updateCircuitInfo.getCargoNo());
//        map.put("cargoName",updateCircuitInfo.getCargoName());
//        map.put("plateNo",updateCircuitInfo.getPlateNo());
//        map.put("deliveryAddress","站点");
//        map.put("transportation",transportation);
//        map.put("transportationType",transportationType);
//        map.put("createTime",time);
//        map.put("remark",updateCircuitInfo.getRemark());
//        map.put("counts",updateCircuitInfo.getAddressVoList().size());
//        map.put("countryVar",countryVar);
//        map.put("cityVar",cityVar);
//        map.put("addressDetailVar",addressDetailVar);
//        map.put("lngVar",lngVar);
//        map.put("latVar",latVar);
//        map.put("postCodeVar",postCodeVar);
//        map.put("arrivalTimeVar",arrivalTimeVar);
//        map.put("startTimeVar",startTimeVar);
//        map.put("expTimeVar",expTimeVar);
//        map.put("compTimeVar",compTimeVar);
////        map.put("SN",addCircuitVo.getSN());
////        map.put("type",type);
////        map.put("TDLSN",addCircuitVo.getTDLSN());
////        map.put("message",gson.toJson(configCMDVo));
////        map.put("flag",false);
////        map.put("untie",false);
//        circuitService.updateCircuit(map);
//        Map<Object,Object> msg=new HashMap<>();
//        msg.put("msg",map.get("msg"));
//        if (msg.get("msg").equals("修改线路成功")){
//            fb.setCode(1);
//            fb.setMessage("修改线路成功");
//        }else{
//            fb.setCode(0);
//            fb.setMessage(msg.get("msg").toString());
//        }
//        return gson.toJson(fb);
//    }

    //    @ApiOperation(value = "根据线路编号获取线路详细信息",notes = "根据线路编号获取线路详细信息，传值说明：" +
//            "   transportationNo:运输编号；   " +
//            "返回值说明：" +
//            " " )
//    @ApiImplicitParams({
//            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
//    })
//    @PostMapping("/getAlarmData")
//    public Object getAlarmData(@RequestBody String transportationNo){
//        JSONObject jsonObject= JSON.parseObject(transportationNo);
//        transportationNo=(String)jsonObject.get("transportationNo");
//        List<ResultAlarmLog> list= alarmLogService.getByTransportationNo(transportationNo);
//        return  gson.toJson(list);
//    }
}
