package com.example.tdl.web;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.*;
import com.example.tdl.service.*;
import com.example.tdl.service.redis.InfoRedisService;
import com.example.tdl.service.redis.RealRedisService;

import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

import org.influxdb.dto.Query;
import org.influxdb.dto.QueryResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import org.springframework.data.influxdb.InfluxDBTemplate;

import java.text.ParseException;
import java.text.SimpleDateFormat;

import java.util.*;

@RestController
public class DataController {
    private Logger logger = LoggerFactory.getLogger(DataController.class);

    Gson gson = new Gson();

    CommFeedback fb = new CommFeedback();

    @Autowired
    private TDLDeviceService tdlDeviceService;

    @Autowired
    private ProbesService probesService;

    @Autowired
    private RealRedisService realRedisService;

    @Autowired
    private InfoRedisService infoRedisService;

    @Autowired
    private InfluxDBTemplate influxDBTemplate;

    @Autowired
    private AlarmLogService alarmLogService;

    @Autowired
    private CircuitService circuitService;

    @Autowired
    private TDLLogService tdlLogService;

    private final static String database ="original";

    //获取坐标数据数据
    @ApiOperation(value = "获取坐标数据数据",notes = "获取坐标数据数据" +
            "time:时间" +
            "lng：经度" +
            "lat: 纬度")
    @PostMapping("/getLocation")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public Object getLocation(@RequestBody String transportationNo){
        JSONObject jsonObject= JSON.parseObject(transportationNo);
        transportationNo=jsonObject.getString("transportationNo");
        //根据运输编号获取线路信息
        ResultCircuitVo resultCircuitVo = circuitService.getByTransportationNo(transportationNo);
        if(resultCircuitVo.getEndTime() !=0  && infoRedisService.hasHashKey("TransportConfig",transportationNo)){
            Object ret = infoRedisService.getHash("TransportConfig",transportationNo);
            return ret;
        }else{
            //获取gateway的信息
            ResultGatewayVo gatewayVo = new ResultGatewayVo();
            if(resultCircuitVo.getEndTime() == 0){
                gatewayVo = circuitService.getByNo(transportationNo);
            }else{
                gatewayVo = tdlLogService.getByNo(transportationNo);
            }
            if(gatewayVo==null){
                fb.setCode(0);
                fb.setMessage("该线路未绑定设备");
                return gson.toJson(fb);
            }
            String device = gatewayVo.getType() +"_" +gatewayVo.getSN();
            Long startTime = resultCircuitVo.getStartTime() *1000000l;
            //获取tdl信息
            String sql = "SELECT \"lng\",\"lat\" FROM \"tdl_policy\".\""+device+"\"where time >= "+startTime +"  ORDER BY time ";
            QueryResult  queryResult  = influxDBTemplate.query(new Query(sql,database));
            //解析经纬度数据
            List<LocationDataVo>  locationDataVos = new ArrayList<>();
            if(queryResult.getResults().get(0).getSeries() != null){
                locationDataVos=getLocation(queryResult);
            }
            //获取震动的数据
            String sql2 = "SELECT \"b\" FROM \"tdl_policy\".\""+device+"\"where time >= "+startTime +" ORDER BY time";
            QueryResult  queryResult2  = influxDBTemplate.query(new Query(sql2,database));
            List<String> timeList = new ArrayList<>();
            if(queryResult2.getResults().get(0).getSeries() != null){
                //获取震动的时间戳
                timeList =  getTime(queryResult2);
            }
            //获取震动数据的坐标
            String sql3 ="";
            for(String time : timeList){
                Long t = Long.valueOf(time) *1000000l;
                sql3 = "SELECT \"lng\",\"lat\" FROM \"tdl_policy\".\""+device+"\" where time >= "+startTime +" and time<="+ t +" ORDER BY time desc limit 1;" +sql3;
            }
            QueryResult  queryResult3  = influxDBTemplate.query(new Query(sql3,database));
            List<String> time= new ArrayList<>();
            if(queryResult3.getResults() !=null){
                time = getShockLocation(queryResult3);
            }
            List<LocationDataVo> locationDataVoList =new ArrayList<>();
            for(LocationDataVo locationDataVo :locationDataVos){
                if(time.contains(locationDataVo.getTime())){
                    locationDataVo.setError(1);
                    locationDataVoList.add(locationDataVo);
                }else{
                    locationDataVo.setError(0);
                    locationDataVoList.add(locationDataVo);
                }
            }
            //如果是已经完结保存到redis中
            if(resultCircuitVo.getEndTime() != 0){
                infoRedisService.setHash("TransportConfig",transportationNo,locationDataVoList.toString(),60*30);
            }
            return gson.toJson(locationDataVoList);
        }
    }


    //获取历史数据
    @ApiOperation(value = "获取历史数据",notes = "获取历史数据" +
            "返回数据格式:" +
            "   ")
    @PostMapping("/getHistoryData")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public Object getHistoryData(@RequestBody String transportationNo){
        JSONObject jsonObject= JSON.parseObject(transportationNo);
        transportationNo=jsonObject.getString("transportationNo");
        ResultCircuitVo resultCircuitVo = circuitService.getByTransportationNo(transportationNo);
        ResultGatewayVo gatewayVo = new ResultGatewayVo();
        if(resultCircuitVo.getEndTime() == 0){
            gatewayVo = circuitService.getByNo(transportationNo);
        }else{
            gatewayVo = tdlLogService.getByNo(transportationNo);
        }
        String device = gatewayVo.getType() +"_" +gatewayVo.getSN();
        //获取tdl信息
        List<String> devList = tdlDeviceService.getByGatewaySN(gatewayVo.getSN(),gatewayVo.getType());
        String devcie=gatewayVo.getType()+"_"+gatewayVo.getSN() ;
        Long startTime = resultCircuitVo.getStartTime() *1000000l;
        if(devList.size() == 0){
            fb.setCode(0);
            fb.setMessage("没有数据");
            return gson.toJson(fb);
        }
        List<String> list = new ArrayList<String>(){{
            add("\"t\"");
            add("\"h\"");
            add("\"p\"");
            add("\"a0\"");
            add("\"ta\"");
            //add("\"t\"");
        }};
        Map<String,String> map = new HashMap<>();
        map.put("\"t\"","temp");
        map.put("\"h\"","humidity");
        map.put("\"p\"","pressure");
        map.put("\"a0\"","shockEnergy");
        map.put("\"ta\"","tilt");
        List<DataVo> dataVoList = new ArrayList<>();
        for(int j = 0;j< list.size();j++){
            DataVo dataVo = new DataVo();
            dataVo.setDescription(map.get(list.get(j)));
            List<HistoryDataVo> historyDataVoList = new ArrayList<>();
            for( int i =0;i<devList.size();i++){
                HistoryDataVo historyDataVo = new HistoryDataVo();
                historyDataVo.setTDLSN(devList.get(i));
                String sql = "SELECT "+list.get(i)+" FROM \"tdl_policy\".\""+devcie+"\" where \"tdl\"=\'"+devList.get(i)+"\'  and time >="+startTime + " ORDER BY time ";
                QueryResult  queryResult  = influxDBTemplate.query(new Query(sql,database));
                List<HistoryData>  historyData = new ArrayList<>();
                if(queryResult.getResults().get(0).getSeries() != null){
                    //解析数据
                    historyData = getHistoryData(queryResult);
                }
                historyDataVo.setDataList(historyData);
                historyDataVoList.add(historyDataVo);
            }
            dataVo.setDataList(historyDataVoList);
            dataVoList.add(dataVo);
        }
        return  gson.toJson(dataVoList);
    }

    //获取温度数据
    @ApiOperation(value = "获取温度数据",notes = "获取温度数据" +
            "TDLSN:tdl编号" +
            "temperatureDataVos:" +
            "time:时间" +
            "value:温度数据")
    @PostMapping("/getTemperature")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public Object getTemperature(@RequestBody String transportationNo){
        JSONObject jsonObject= JSON.parseObject(transportationNo);
        transportationNo=jsonObject.getString("transportationNo");
        ResultCircuitVo resultCircuitVo = circuitService.getByTransportationNo(transportationNo);
        ResultGatewayVo gatewayVo = new ResultGatewayVo();
        if(resultCircuitVo.getEndTime() == 0){
            gatewayVo = circuitService.getByNo(transportationNo);
        }else{
            gatewayVo = tdlLogService.getByNo(transportationNo);
        }
        String device = gatewayVo.getType() +"_" +gatewayVo.getSN();
        //获取tdl信息
        List<String> devList = tdlDeviceService.getByGatewaySN(gatewayVo.getSN(),gatewayVo.getType());
        String devcie=gatewayVo.getType()+"_"+gatewayVo.getSN() ;
        Long startTime = resultCircuitVo.getStartTime() *1000000l;
        //SELECT "h" FROM "tdl_policy"."WTD93LG_20180314" WHERE "tdl" = 'tdl-0A0B0C0D0F' ORDER BY time DESC
        if(devList.size() == 0){
            fb.setCode(0);
            fb.setMessage("没有数据");
            return gson.toJson(fb);
        }
        List<Temperature> temperatures = new ArrayList<>();
        for( int i =0;i<devList.size();i++){
            Temperature temperature = new Temperature();
            temperature.setTDLSN(devList.get(i));
            String sql = "SELECT \"t\"  FROM \"tdl_policy\".\""+devcie+"\" where \"tdl\"=\'"+devList.get(i)+"\' and time >="+startTime+" ORDER BY time ";
            QueryResult  queryResult  = influxDBTemplate.query(new Query(sql,database));
            List<TemperatureDataVo>  temperatureDataVos = new ArrayList<>();
            if(queryResult.getResults().get(0).getSeries() != null){
                //解析数据
                temperatureDataVos = getTemperature(queryResult);
            }
            temperature.setTemperatureDataVos(temperatureDataVos);
            temperatures.add(temperature);
        }
        return  gson.toJson(temperatures);
    }

    //获取湿度数据
    @ApiOperation(value = "获取湿度数据",notes = "获取湿度数据")
    @PostMapping("/getHumidity")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public Object getHumidity(@RequestBody String transportationNo){
        JSONObject jsonObject= JSON.parseObject(transportationNo);
        transportationNo=jsonObject.getString("transportationNo");
        ResultCircuitVo resultCircuitVo = circuitService.getByTransportationNo(transportationNo);
        ResultGatewayVo gatewayVo = new ResultGatewayVo();
        if(resultCircuitVo.getEndTime() == 0){
            gatewayVo = circuitService.getByNo(transportationNo);
        }else{
            gatewayVo = tdlLogService.getByNo(transportationNo);
        }
        String device = gatewayVo.getType() +"_" +gatewayVo.getSN();
        Long startTime = resultCircuitVo.getStartTime() *1000000l;
        //获取tdl信息
        List<String> devList = tdlDeviceService.getByGatewaySN(gatewayVo.getSN(),gatewayVo.getType());
        String devcie=gatewayVo.getType()+"_"+gatewayVo.getSN() ;
        if(devList.size() == 0){
            fb.setCode(0);
            fb.setMessage("没有数据");
            return gson.toJson(fb);
        }
        List<Humidity> humidities = new ArrayList<>();
        for( int i =0;i<devList.size();i++){
            Humidity humidity = new Humidity();
            humidity.setTDLSN(devList.get(i));
            String sql = "SELECT \"h\"  FROM \"tdl_policy\".\""+devcie+"\" where \"tdl\"=\'"+devList.get(i)+"\' and time >="+startTime+" ORDER BY time ";
            QueryResult  queryResult  = influxDBTemplate.query(new Query(sql,database));
            List<HumidityDataVo>  humidityDataVos = new ArrayList<>();
            if(queryResult.getResults().get(0).getSeries() != null){
                //解析数据
                humidityDataVos = getHumidity(queryResult);
            }
            humidity.setHumidityDataVoList(humidityDataVos);
            humidities.add(humidity);
        }
//        //SELECT "h" FROM "tdl_policy"."WTD93LG_20180314" WHERE "tdl" = 'tdl-0A0B0C0D0F' ORDER BY time DESC
//        String sql = "SELECT \"h\"  FROM \"tdl_policy\".\""+devcie+"\" ORDER BY time ";
//        QueryResult  queryResult  = influxDBTemplate.query(new Query(sql,database));
//        List<HumidityDataVo>  humidityDataVos = new ArrayList<>();
//        if(queryResult.getResults().get(0).getSeries() != null){
//            //解析数据
//            humidityDataVos = getHumidity(queryResult);
//        }
        return  gson.toJson(humidities);
    }

    //获取倾斜度数据
    @ApiOperation(value = "获取倾斜度数据",notes = "获取倾斜度数据")
    @PostMapping("/getTiltAngle")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public Object getTiltAngle(@RequestBody String transportationNo){
        JSONObject jsonObject= JSON.parseObject(transportationNo);
        transportationNo=jsonObject.getString("transportationNo");
        ResultCircuitVo resultCircuitVo = circuitService.getByTransportationNo(transportationNo);
        ResultGatewayVo gatewayVo = new ResultGatewayVo();
        if(resultCircuitVo.getEndTime() == 0){
            gatewayVo = circuitService.getByNo(transportationNo);
        }else{
            gatewayVo = tdlLogService.getByNo(transportationNo);
        }
        String device = gatewayVo.getType() +"_" +gatewayVo.getSN();
        Long startTime = resultCircuitVo.getStartTime() *1000000l;
        //获取tdl信息
        List<String> devList = tdlDeviceService.getByGatewaySN(gatewayVo.getSN(),gatewayVo.getType());
        String devcie=gatewayVo.getType()+"_"+gatewayVo.getSN() ;
        if(devList.size() == 0){
            fb.setCode(0);
            fb.setMessage("没有数据");
            return gson.toJson(fb);
        }
        List<TiltAngle> tiltAngles = new ArrayList<>();
        for( int i =0;i<devList.size();i++){
            TiltAngle tiltAngle = new TiltAngle();
            tiltAngle.setTDLSN(devList.get(i));
            String sql = "SELECT \"ta\"  FROM \"tdl_policy\".\""+devcie+"\" where \"tdl\"=\'"+devList.get(i)+"\' and time >="+startTime+" ORDER BY time ";
            QueryResult  queryResult  = influxDBTemplate.query(new Query(sql,database));
            List<TiltAngleVo>  tiltAngleVos = new ArrayList<>();
            if(queryResult.getResults().get(0).getSeries() != null){
                //解析数据
                tiltAngleVos = getTiltAngle(queryResult);
            }
            tiltAngle.setShockEnergyVoList(tiltAngleVos);
            tiltAngles.add(tiltAngle);
        }
//        String sql = "SELECT \"ta\"  FROM \"tdl_policy\".\""+devcie+"\" ORDER BY time ";
//        QueryResult  queryResult  = influxDBTemplate.query(new Query(sql,database));
//        List<TiltAngleVo>  tiltAngleVos = new ArrayList<>();
//        if(queryResult.getResults().get(0).getSeries() != null){
//            //解析数据
//            tiltAngleVos = getTiltAngle(queryResult);
//        }
        return  gson.toJson(tiltAngles);
    }

    //获取震动能量
    @ApiOperation(value = "获取震动能量",notes = "获取震动能量")
    @PostMapping("/getShockEnergy")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public Object getShockEnergy(@RequestBody String transportationNo){
        JSONObject jsonObject= JSON.parseObject(transportationNo);
        transportationNo=jsonObject.getString("transportationNo");
        ResultCircuitVo resultCircuitVo = circuitService.getByTransportationNo(transportationNo);
        ResultGatewayVo gatewayVo = new ResultGatewayVo();
        if(resultCircuitVo.getEndTime() == 0){
            gatewayVo = circuitService.getByNo(transportationNo);
        }else{
            gatewayVo = tdlLogService.getByNo(transportationNo);
        }
        String device = gatewayVo.getType() +"_" +gatewayVo.getSN();
        Long startTime = resultCircuitVo.getStartTime() *1000000l;
        //获取tdl信息
        List<String> devList = tdlDeviceService.getByGatewaySN(gatewayVo.getSN(),gatewayVo.getType());
        String devcie=gatewayVo.getType()+"_"+gatewayVo.getSN() ;
        if(devList.size() == 0){
            fb.setCode(0);
            fb.setMessage("没有数据");
            return gson.toJson(fb);
        }
        List<ShockEnergy> shockEnergies = new ArrayList<>();
        for( int i =0;i<devList.size();i++){
            ShockEnergy shockEnergy = new ShockEnergy();
            shockEnergy.setTDLSN(devList.get(i));
            String sql = "SELECT \"a0\"  FROM \"tdl_policy\".\""+devcie+"\" where \"tdl\"=\'"+devList.get(i)+"\' and time >="+startTime+" ORDER BY time ";
            QueryResult  queryResult  = influxDBTemplate.query(new Query(sql,database));
            List<ShockEnergyVo>  shockEnergyVos = new ArrayList<>();
            if(queryResult.getResults().get(0).getSeries() != null){
                //解析数据
                shockEnergyVos = getShockEnergy(queryResult);
            }
            shockEnergy.setShockEnergyVoList(shockEnergyVos);
            shockEnergies.add(shockEnergy);
        }
//        String sql = "SELECT \"a0\"  FROM \"tdl_policy\".\""+devcie+"\" ORDER BY time ";
//        QueryResult  queryResult  = influxDBTemplate.query(new Query(sql,database));
//        List<ShockEnergyVo>  shockEnergyVos = new ArrayList<>();
//        if(queryResult.getResults().get(0).getSeries() != null){
//            //解析数据
//            shockEnergyVos = getShockEnergy(queryResult);
//        }
        return  gson.toJson(shockEnergies);
    }


    //解析数据
    public List<LocationDataVo> getLocation(QueryResult  queryResult){
        List<List<Object>> values = getValues(queryResult);
        List<LocationDataVo> historyDataList = new ArrayList<>();
        for(List<Object> value:values){
            LocationDataVo historyData = new LocationDataVo();
            historyData.setTime(parseTime(value.get(0).toString()));
            historyData.setLng(Double.parseDouble(value.get(1).toString()));
            historyData.setLat(Double.parseDouble(value.get(2).toString()));
            historyData.setError(0);
            historyDataList.add(historyData);
        }
        return historyDataList;
    }

    public List<HistoryData> getHistoryData(QueryResult  queryResult){
        List<List<Object>> values = getValues(queryResult);
        List<HistoryData> historyDataList = new ArrayList<>();
        for(List<Object> value:values){
            HistoryData historyData = new HistoryData();
            historyData.setTime(parseTime(value.get(0).toString()));
            historyData.setValue(value.get(1)==null  ? "" :value.get(1).toString());
            historyDataList.add(historyData);
        }
        return historyDataList;
    }

    public List<TemperatureDataVo> getTemperature(QueryResult  queryResult){
        List<List<Object>> values = getValues(queryResult);
        List<TemperatureDataVo> historyDataList = new ArrayList<>();
        for(List<Object> value:values){
            TemperatureDataVo historyData = new TemperatureDataVo();
            historyData.setTime(parseTime(value.get(0).toString()));
            historyData.setValue(value.get(1)==null  ? "" :value.get(1).toString());
            historyDataList.add(historyData);
        }
        return historyDataList;
    }

    public List<TiltAngleVo> getTiltAngle(QueryResult  queryResult){
        List<List<Object>> values = getValues(queryResult);
        List<TiltAngleVo> historyDataList = new ArrayList<>();
        for(List<Object> value:values){
            TiltAngleVo historyData = new TiltAngleVo();
            historyData.setTime(parseTime(value.get(0).toString()));
            historyData.setValue(value.get(1)==null  ? "" :value.get(1).toString());
            historyDataList.add(historyData);
        }
        return historyDataList;
    }

    public List<ShockEnergyVo> getShockEnergy(QueryResult  queryResult){
        List<List<Object>> values = getValues(queryResult);
        List<ShockEnergyVo> historyDataList = new ArrayList<>();
        for(List<Object> value:values){
            ShockEnergyVo historyData = new ShockEnergyVo();
            historyData.setTime(parseTime(value.get(0).toString()));
            historyData.setValue(value.get(1)==null  ? "" :value.get(1).toString());
            historyDataList.add(historyData);
        }
        return historyDataList;
    }

    public List<HumidityDataVo> getHumidity(QueryResult  queryResult){
        List<List<Object>> values = getValues(queryResult);
        List<HumidityDataVo> historyDataList = new ArrayList<>();
        for(List<Object> value:values){
            HumidityDataVo historyData = new HumidityDataVo();
            historyData.setTime(parseTime(value.get(0).toString()));
            historyData.setValue(value.get(1)==null  ? "" :value.get(1).toString());
            historyDataList.add(historyData);
        }
        return historyDataList;
    }

    //解析数据格式
    public List<List<Object>>  getValues( QueryResult  queryResult){
        List<List<Object>> values = new ArrayList<>();
        for(QueryResult.Result result : queryResult.getResults()){
            if(result.getSeries() !=null){
                for (QueryResult.Series series : result.getSeries()) {
                    if(series !=null){
                        values= series.getValues();
                    }
                }
            }
        }
        return values;
    }

    //将utc时间转换成时间戳
    public static String dateToStamp(String s) throws ParseException {
        Date date =null ;
        if(s.length() !=20 ) {
            date = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'").parse(s);
        }else{
            date = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'").parse(s);
        }
        return String.valueOf(date.getTime());
    }

    //解析时间
    public String parseTime(String time){
        String dateResult =null;
        try {
            dateResult = dateToStamp(time);
        } catch (ParseException e) {
            e.printStackTrace();
            logger.info(e.toString());
            fb.setCode(0);
            fb.setMessage("解析时间异常");
        }
        return dateResult;
    }

    //判断两个时间相差多少
    public static String getDistanceTime(Long time1, Long time2) {
        //获取当前时区，将时间戳转化成日期时间格式
        Date t1 = new Date(time1);
        Date t2 = new Date(time2);
        int days = differentDays(t1,t2);
        if(days>0){
            return  days +"天";
        }else{
            return  "a little minute";
        }
    }

    public static int differentDays(Date date1,Date date2) {
        Calendar cal1 = Calendar.getInstance();
        cal1.setTime(date1);

        Calendar cal2 = Calendar.getInstance();
        cal2.setTime(date2);
        int day1= cal1.get(Calendar.DAY_OF_YEAR);
        int day2 = cal2.get(Calendar.DAY_OF_YEAR);

        int year1 = cal1.get(Calendar.YEAR);
        int year2 = cal2.get(Calendar.YEAR);
        if(year1 != year2) { //不同年
            int timeDistance = 0 ;
            for(int i = year1 ; i < year2 ; i ++) {
                if(i%4==0 && i%100!=0 || i%400==0) {   //闰年
                    timeDistance += 366;
                } else{    //不是闰年
                    timeDistance += 365;
                }
            }
            return timeDistance + (day2-day1) ;
        } else{
            return day2-day1;
        }
    }

    //将时间戳转成时间
    public static String timeStamp2Date(Long seconds) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        return sdf.format(new Date(seconds));
    }

    //解析震动时间
    public List<String> getTime(QueryResult  queryResult){
        List<List<Object>> values = getValues(queryResult);
        List<String> time = new ArrayList<>();
        for(List<Object> value:values){
            try {
                time.add(UTCToCST(value.get(0).toString()));
            }catch (Exception e){
                e.printStackTrace();
                logger.info(e.toString());
            }
        }
        return time;
    }

    public  String UTCToCST(String UTCStr) throws ParseException {
        Date date =null ;
        if(UTCStr.length() !=20 ) {
            date = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'").parse(UTCStr);
        }else{
            date = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'").parse(UTCStr);
        }
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.HOUR, calendar.get(Calendar.HOUR) + 8);
        //calendar.getTime() 返回的是Date类型，也可以使用calendar.getTimeInMillis()获取时间戳
        //format的格式可以任意
        return String.valueOf(calendar.getTimeInMillis());
    }

    public List<String>  getShockLocation(QueryResult  queryResult){
        List<List<Object>> values = new ArrayList<>();
        for(QueryResult.Result result : queryResult.getResults()){
            if(result.getSeries() !=null){
                for (QueryResult.Series series : result.getSeries()) {
                    if(series !=null){
                        values.add(series.getValues().get(0));
                    }
                }
            }
        }
        List<String> timeList = new ArrayList<>();
        for(List<Object> value:values){
            timeList.add(parseTime(value.get(0).toString()));
        }
        return timeList;
    }

    public List<DeviceDataVo> getDeviceData(QueryResult  queryResult){
        List<List<Object>> values = getValues(queryResult);
        List<DeviceDataVo> historyDataList = new ArrayList<>();
        for(List<Object> value:values){
            DeviceDataVo historyData = new DeviceDataVo();
            historyData.setTime(parseTime(value.get(0).toString()));
            historyData.setTemperature(value.get(1)==null ? "" :value.get(1).toString());
            historyData.setHumidity(value.get(2)==null  ? "" :value.get(2).toString());
            historyData.setPressure(value.get(3)==null  ? "" :value.get(3).toString());
            historyData.setShockEnergy(value.get(4)==null  ? "" :value.get(4).toString());
            historyData.setTiltAngle(value.get(5)==null  ? "" :value.get(5).toString());
            historyDataList.add(historyData);
        }
        return historyDataList;
    }


    //根据运输编号获取路线以及设备信息
//    @ApiOperation(value = "获取路线以及设备信息",notes = "获取路线以及设备信息" +
//            "transportationNo:运输编号" +
//            "startCity:启运城市" +
//            "startTime:开始时间" +
//            "endCity:目的城市" +
//            "endTime:结束时间（未结束为\"\"）" +
//            "gatewaySN:网关编号" +
//            "battery:网关点量" +
//            "TDLSN:tdl编号" +
//            "batteryVoltage:tdl电压" +
//            "lastTime：运输时长（未结束为\"\"）" +
//            "circuitState:运输状态(0--未开始，1--运输中，2--完成)" +
//            "remark：备注信息")
//    @PostMapping("/getCircuit")
//    @ApiImplicitParams({
//            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
//    })
//    public Object  getCircuit(@RequestBody String transportationNo){
//        JSONObject jsonObject= JSON.parseObject(transportationNo);
//        transportationNo=jsonObject.getString("transportationNo");
//        ResultCircuit resultCircuit = new ResultCircuit();
//        resultCircuit.setTransportationNo(transportationNo);
//        //根据运输编号获取线路信息，设备信息
//        ResultCircuitVo resultCircuitVo = circuitService.getByTransportationNo(transportationNo);
//        resultCircuit.setStartCity(resultCircuitVo.getStartCity());
//        resultCircuit.setStartTime(resultCircuitVo.getStartTime());
//        resultCircuit.setEndCity(resultCircuitVo.getEndCity());
//        ResultGatewayVo gatewayVo = new ResultGatewayVo();
//        if(resultCircuitVo.getEndTime() == 0){
//            resultCircuit.setEndTime(null);
//            resultCircuit.setLastTime(null);
//            gatewayVo = circuitService.getByNo(transportationNo);
//        }else{
//            resultCircuit.setEndTime(resultCircuitVo.getEndTime());
//            resultCircuit.setLastTime(getDistanceTime(resultCircuit.getStartTime(),resultCircuit.getEndTime()));
//            gatewayVo = tdlLogService.getByNo(transportationNo);
//        }
//        //获取gateway的信息
//        if(gatewayVo==null){
//            fb.setCode(0);
//            fb.setMessage("该线路未绑定设备");
//            return gson.toJson(fb);
//        }
//        resultCircuit.setGatewaySN(gatewayVo.getSN());
//        String device = gatewayVo.getType() +"_" +gatewayVo.getSN();
//        Long startTime = resultCircuitVo.getStartTime() *1000000l;
//        //获取tdl信息
//        String sql = "SELECT \"battery\"  FROM \"tdl_policy\".\""+device+"\" where time >= "+startTime +"  ORDER BY time  desc limit 1";
//        QueryResult  queryResult  = influxDBTemplate.query(new Query(sql,database));
//        if(queryResult.getResults().get(0).getSeries() != null){
//            List<List<Object>> values = getValues(queryResult);
//            for(List<Object> value:values){
//                logger.info(value.get(1).toString());
//                resultCircuit.setBattery(value.get(1)==null ?  "": value.get(1).toString());
//            }
//        }
//        //获取tdl信息
//        List<String> devList = tdlDeviceService.getByGatewaySN(gatewayVo.getSN(),gatewayVo.getType());
//        String key=gatewayVo.getSN()+"_"+gatewayVo.getType() +":"+gatewayVo.getSN()+"_"+gatewayVo.getType() ;
//        //获取网关以及TDL的电量
//        if(devList.size() == 0 ){
//            fb.setCode(0);
//            fb.setMessage("该线路未绑定设备");
//            return gson.toJson(fb);
//        }
//        for(int i = 0;i<devList.size();i++){
//            String sql2 = "SELECT \"b\"  FROM \"tdl_policy\".\""+device+"\" where \"tdl\"=\'tdl-"+devList.get(i)+"\'and  time >= "+startTime +"  ORDER BY time desc limit 1";
//            QueryResult  queryResult2  = influxDBTemplate.query(new Query(sql2,database));
//            if(queryResult2.getResults().get(0).getSeries() != null){
//                List<List<Object>> values = getValues(queryResult2);
//                for(List<Object> value:values){
//                    resultCircuit.setTDLSN(devList.get(i));
//                    resultCircuit.setBatteryVoltage(value.get(1)==null ? "" :value.get(1).toString());
//                }
//            }
//        }
//        resultCircuit.setCircuitState(resultCircuitVo.getCircuitState());
//        return  gson.toJson(resultCircuit);
//    }


    //根据时间获取经纬度信息
//    @ApiOperation(value = "根据时间获取经纬度信息",notes = "根据时间获取经纬度信息" +
//            "transportationNo:运输编号" +
//            "TDLSN:tdl编号" +
//            "gatewaySN:网关编号" +
//            "startCity:出发城市" +
//            "endCity:目的城市" +
//            "startTime:出发时间" +
//            "endTime:到达时间" +
//            "alarmTime:报警时间" +
//            "alarmType:报警类型" +
//            "alarmValue:报警值" +
//            "lng:经度" +
//            "lat:纬度")
//    @PostMapping("/getLocationByTime")
//    @ApiImplicitParams({
//            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
//    })
//    public Object getLocationByTime(@RequestBody LocationVo locationVo ){
//        //获取线路信息
//        //获取网关以及tdl信息
//        ResultCircuitVo resultCircuitVo = circuitService.getByTransportationNo(locationVo.getTransportationNo());
//        ResultGatewayVo gatewayVo = new ResultGatewayVo();
//        if(resultCircuitVo.getEndTime() == 0){
//            gatewayVo = circuitService.getByNo(locationVo.getTransportationNo());
//        }else{
//            gatewayVo = tdlLogService.getByNo(locationVo.getTransportationNo());
//        }
//        String device = gatewayVo.getType() +"_" +gatewayVo.getSN();
//        //获取tdl信息
//        List<String> devList = tdlDeviceService.getByGatewaySN(gatewayVo.getSN(),gatewayVo.getType());
//        String devcie=gatewayVo.getType()+"_"+gatewayVo.getSN() ;
//        //根据时间获取坐标信息
//        String sql ="SELECT \"lng\",\"lat\" FROM \"tdl_policy\".\""+devcie+"\" where time = "+locationVo.getTime() *1000000l;
//        QueryResult  queryResult  = influxDBTemplate.query(new Query(sql,database));
//        List<LocationMessageVo>  locationDataVos = new ArrayList<>();
//        if(queryResult.getResults().get(0).getSeries() != null){
//            List<List<Object>> values = getValues(queryResult);
//            for(List<Object> value:values){
//                LocationMessageVo messageVo = new LocationMessageVo();
//                messageVo.setTransportationNo(locationVo.getTransportationNo());
//                messageVo.setGatewaySN(gatewayVo.getSN());
//                messageVo.setTDLSN(devList);
//                messageVo.setStartCity(resultCircuitVo.getStartCity());
//                messageVo.setEndCity(resultCircuitVo.getEndCity());
//                messageVo.setStartTime(resultCircuitVo.getStartTime());
//                messageVo.setEndTime(resultCircuitVo.getEndTime());
//                messageVo.setAlarmTime(Long.valueOf(parseTime(value.get(0).toString())));
//                messageVo.setAlarmType(locationVo.getAlarmType());
//                messageVo.setAlarmValue(locationVo.getVlaue());
//                messageVo.setLng(value.get(1)==null ?  "" : value.get(1).toString());
//                messageVo.setLat(value.get(2)==null ?  "" : value.get(2).toString());
//                locationDataVos.add(messageVo);
//            }
//        }
//        return  gson.toJson(locationDataVos);
//    }

    //获取温度、湿度、震动强度、震动能量、倾斜度
//    @PostMapping("/getDeviceData")
//    @ApiImplicitParams({
//            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
//    })
//    public Object getDeviceData(@RequestBody String transportationNo){
//        JSONObject jsonObject= JSON.parseObject(transportationNo);
//        transportationNo=jsonObject.getString("transportationNo");
//        //获取网关以及tdl信息
//        ResultCircuitVo resultCircuitVo = circuitService.getByTransportationNo(transportationNo);
//        ResultGatewayVo gatewayVo = new ResultGatewayVo();
//        if(resultCircuitVo.getEndTime() == 0){
//            gatewayVo = circuitService.getByNo(transportationNo);
//        }else{
//            gatewayVo = tdlLogService.getByNo(transportationNo);
//        }
//        String device = gatewayVo.getType() +"_" +gatewayVo.getSN();
//        Long startTime = resultCircuitVo.getStartTime() *1000000l;
//        //获取tdl信息
//        List<String> devList = tdlDeviceService.getByGatewaySN(gatewayVo.getSN(),gatewayVo.getType());
//        String devcie=gatewayVo.getType()+"_"+gatewayVo.getSN() ;
//        //SELECT "h" FROM "tdl_policy"."WTD93LG_20180314" WHERE "tdl" = 'tdl-0A0B0C0D0F' ORDER BY time DESC
//        String sql = "SELECT \"t\",\"h\",\"p\",\"a0\",\"ta\" FROM \"tdl_policy\".\""+devcie+"\" where \"tdl\" = \'"+devList.get(0)+"\' and  time>="+startTime+"  ORDER BY time ";
//        QueryResult  queryResult  = influxDBTemplate.query(new Query(sql,database));
//        //解析经纬度数据
//        List<DeviceDataVo>  deviceDataVos = new ArrayList<>();
//        if(queryResult.getResults().get(0).getSeries() != null){
//            deviceDataVos= getDeviceData(queryResult);
//        }
//        return gson.toJson(deviceDataVos);
//    }
    //获取实时数据
//    @ApiOperation(value = "获取实时数据",notes = "获取实时数据")
//    @ApiImplicitParams({
//            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
//    })
//    @PostMapping("/getRealTimeData")
//    public Object getRealTimeData(@RequestBody String transportationNo){
//        JSONObject jsonObject= JSON.parseObject(transportationNo);
//        transportationNo=jsonObject.getString("transportationNo");
//        //获取gateway的信息
//       // ResultGatewayVo gatewayVo = circuitService.getByNo(transportationNo);
//        ResultGatewayVo gatewayVo = new ResultGatewayVo();
//        if(gatewayVo==null){
//            fb.setCode(0);
//            fb.setMessage("该线路未绑定设备");
//            return gson.toJson(fb);
//        }
//        //获取tdl信息
//        List<String> devList = tdlDeviceService.getByGatewaySN(gatewayVo.getSN(),gatewayVo.getType());
//        //根据gateWay获取tdl信息
//        List<String> key = new ArrayList<>();
//        key.add(gatewayVo.getSN()+"_"+gatewayVo.getType() +":"+gatewayVo.getSN()+"_"+gatewayVo.getType());
//        for (int i = 0;i<devList.size();i++){
//            key.add(gatewayVo.getSN()+"_"+gatewayVo.getType()+":"+devList.get(i));
//        }
//        List<LiveDataVo> liveDataVos = new ArrayList<>();
//        for(int j=0;j<key.size();j++){
//            Map<Object,Object> ret = realRedisService.getHashAll(key.get(j));
//            if(ret == null){
//                continue;
//            }else{
//                Iterator entries = ret.entrySet().iterator();
//                List<String> aliasList = new ArrayList<>();
//                List<String> valueList = new ArrayList<>();
//                while(entries.hasNext()){
//                    Map.Entry entry = (Map.Entry) entries.next();
//                    aliasList.add((String) entry.getKey());
//                    valueList.add((String) entry.getValue());
//                }
//                //根据通道别名获取通道信息
//                List<ProbesVo> probesVoList =  probesService.getByAliasList(gatewayVo.getSN(),gatewayVo.getType(),aliasList);
//                for(int m = 0;m<probesVoList.size();m++){
//                    probesVoList.get(m).setValue(valueList.get(m));
//                }
//                LiveDataVo liveDataVo = new LiveDataVo();
//                liveDataVo.setKey(key.get(j).substring(key.get(j).indexOf(":"),key.get(j).length()));
//                liveDataVo.setProbesVoList(probesVoList);
//                liveDataVos.add(liveDataVo);
//            }
//        }
//        return  gson.toJson(liveDataVos);
//    }


}