package com.example.tdl.web;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.*;
import com.example.tdl.service.AlarmLogService;
import com.example.tdl.service.CircuitService;

import com.example.tdl.service.TDLLogService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.influxdb.dto.Query;
import org.influxdb.dto.QueryResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.influxdb.InfluxDBTemplate;
import org.springframework.web.bind.annotation.*;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@RestController
@RequestMapping("/homePage")
public class HomePageController {
    private Logger logger = LoggerFactory.getLogger(HomePageController.class);

    Gson gson = new Gson();

    CommFeedback fb = new CommFeedback();

    @Autowired
    private CircuitService circuitService;

    @Autowired
    private AlarmLogService alarmLogServcie;

    @Autowired
    private TDLLogService tdlLogService;

    @Autowired
    private InfluxDBTemplate influxDBTemplate;

    private final static String database ="original";

    //获取当天所有的设备数据
    @ApiOperation(value = "获取所有的运输数据",notes = "获取运输设备数据：" +
            "sum:总数量;" +
            "onRoute:在途数量;" +
            "delay:延误数量;" +
            "alarmed: 报警数量;" +
            "bump：颠簸数量;" +
            "list: " +
            "{classify:报警描述;"+
            "count:数量;}"+
            "alarmLogVoList :"+
            "{description:报警描述;" +
            "data:数值;" +
            "alarmTime:报警时间;}" +
            "circuitList：" +
            "{transportationNo:运输编号;" +
            "cargoNo：货物编号;" +
            "startCity:开始城市;" +
            "endCity:目的城市;" +
            "startTime:开始时间;" +
            "endTime:到达时间;" +
            "lng:当前经度;" +
            "lat:当前纬度;" +
            "time:当前时间;}")
    @PostMapping("/getAll")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public Object getAll(@RequestBody String time){
        JSONObject jsonObject= JSON.parseObject(time);
        time=(String) jsonObject.get("time");
        //获取数量
        HomePageDataVo dataVo = new HomePageDataVo();
        //获取所有的线路数量
        dataVo.setSum(circuitService.getCount(time));//总数量
        dataVo.setOnRoute(circuitService.getOnRoute(time));//在途数量
        dataVo.setDelay(circuitService.getDelayCount(System.currentTimeMillis(),time));//延误数量
        dataVo.setAlarmed( alarmLogServcie.getCount(time));//报警数量
        dataVo.setBump(alarmLogServcie.getBumpCount(time)); //获取颠簸数量
        dataVo.setAlarmLogVoList(alarmLogServcie.getAlarmLog(time));//报警列表
        List<HistogramData> list = alarmLogServcie.getByCount(time);//获取报警柱状图

        //获取设备的当前位置
        List<ResultCircuitVo> circuitVos =  circuitService.getByTime(time);
        List<CircuitVo> circuitList = new ArrayList<>();
        for(int i = 0;i<circuitVos.size();i++){
            CircuitVo circuitVo = new CircuitVo();
            circuitVo.setTransportationNo(circuitVos.get(i).getTransportationNo());
            circuitVo.setCargoNo(circuitVos.get(i).getCargoNo());
            circuitVo.setStartCity(circuitVos.get(i).getStartCity());
            circuitVo.setEndCity(circuitVos.get(i).getEndCity());
            circuitVo.setStartTime(circuitVos.get(i).getStartTime());
            circuitVo.setEndTime(circuitVos.get(i).getEndTime());
            ResultGatewayVo gatewayVo = new ResultGatewayVo();
            if (circuitVos.get(i).getStartTime() == null || circuitVos.get(i).getStartTime() == 0) {
                circuitList.add(circuitVo);
            } else {
                if(circuitVos.get(i).getEndTime() == null ||circuitVos.get(i).getEndTime() == 0 ){
                    gatewayVo = circuitService.getByNo(circuitVos.get(i).getTransportationNo());
                }else{
                    gatewayVo = tdlLogService.getByNo(circuitVos.get(i).getTransportationNo());
                }
                if(gatewayVo==null){
                    circuitList.add(circuitVo);
                }else {
                    String device = gatewayVo.getType() + "_" + gatewayVo.getSN();
                    Long startTime = circuitVos.get(i).getStartTime() * 1000000l;
                    String sql = "";
                    if (circuitVos.get(i).getEndTime() == null || circuitVos.get(i).getEndTime() == 0) {
                        sql = "SELECT \"lng\",\"lat\" FROM \"tdl_policy\".\"" + device + "\"where time >= " + startTime + " ORDER BY time  desc limit 1";
                    } else {
                        sql = "SELECT \"lng\",\"lat\" FROM \"tdl_policy\".\"" + device + "\"where time >= " + startTime + " and time<= " + circuitVos.get(i).getEndTime() * 1000000l + " ORDER BY time  desc limit 1";
                    }
                    QueryResult queryResult = influxDBTemplate.query(new Query(sql, database));
                    if (queryResult.getResults().get(0).getSeries() != null) {
                        String value = queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(0).toString();
                        circuitVo.setTime(parseTime(queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(0).toString()));
                        circuitVo.setLng(queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(1).toString());
                        circuitVo.setLat(queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(2).toString());
                    }
                    circuitList.add(circuitVo);
                }
            }
        }
        dataVo.setCircuitList(circuitList);
        return gson.toJson(dataVo);
    }

    //解析时间
    public String parseTime(String time){
        String dateResult =null;
        try {
            dateResult = dateToStamp(time);
        } catch (ParseException e) {
            e.printStackTrace();
            logger.info(e.toString());
            fb.setCode(0);
            fb.setMessage("解析时间异常");
        }
        return dateResult;
    }

    //将utc时间转换成时间戳
    public static String dateToStamp(String s) throws ParseException {
        Date date =null ;
        if(s.length() !=20 ) {
            date = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'").parse(s);
        }else{
            date = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'").parse(s);
        }
        return String.valueOf(date.getTime());
    }
}
