package com.example.tdl.web;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.AddModelVo;
import com.example.tdl.domain.vo.UpdateModelVo;
import com.example.tdl.service.ModelService;
import com.example.tdl.service.ProbesModelService;
import com.example.tdl.service.TopicModelService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.Map;

@RestController
@RequestMapping(path = "/model")
public class ModelController {
    private Logger logger = LoggerFactory.getLogger(this.getClass());

    private CommFeedback fb = new CommFeedback();

    private Gson gson = new Gson();

    @Autowired
    private ModelService modelService;

    @Autowired
    private TopicModelService topicModelService;

    @Autowired
    private ProbesModelService probesModelService;


    //获取设备的所有主题
    @ApiOperation(value = "查询所有配置模板",notes = "查询所有配置模板的返回值对应的意义:" +
        "      modelName: 模板名," +
        "      createTime: 模板创建时间," +
        "      updateTime: 模板最新修改时间")
    @RequestMapping(value = "/getAll",method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public Object getAll(){
        return modelService.getAll();
    }

    @ApiOperation(value = "添加模板",notes = "添加模板的同时添加通道配置信息，传递的参数:" +
        "      modelName: 添加的模板名," +
        "      probesModelName: 通道模板名," +
        "      topicModelName: 主题模板名")
    @RequestMapping(value="/addModel",method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public String addModel(@RequestBody AddModelVo addModelVo){
        if (StringUtils.isEmpty(addModelVo.getModelName())){
            fb.setCode(0);
            fb.setMessage("添加的模板名不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addModelVo.getProbesModelName())){
            fb.setCode(0);
            fb.setMessage("通道模板名不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addModelVo.getTopicModelName())){
            fb.setCode(0);
            fb.setMessage("主题模板不能为空");
            return gson.toJson(fb);
        }
        if (topicModelService.getByTopicModelName(addModelVo.getTopicModelName())==null){
            fb.setCode(0);
            fb.setMessage("该主题模板不存在");
            return gson.toJson(fb);
        }
        if (probesModelService.getByProbesModelName(addModelVo.getProbesModelName())==null){
            fb.setCode(0);
            fb.setMessage("该通道模板不存在");
            return gson.toJson(fb);
        }
        Map<String,Object> map=new HashMap<>();
        map.put("modelName",addModelVo.getModelName());
        map.put("probesModelName",addModelVo.getProbesModelName());
        map.put("topicModelName",addModelVo.getTopicModelName());
        map.put("createTime",System.currentTimeMillis());
        map.put("updateTime",System.currentTimeMillis());
        modelService.addModel(map);
        Map<String,Object> msg=new HashMap<>();
        msg.put("msg",map.get("msg"));
        if (msg.get("msg").equals("添加模板成功")){
            fb.setCode(1);
            fb.setMessage("添加模板成功");
        }else{
            fb.setCode(0);
            fb.setMessage(msg.get("msg").toString());
        }
        return gson.toJson(fb);
    }

    @ApiOperation(value = "修改主题模板",notes = "修改模板只能修改模板名，updateTime不用传，传递的参数:" +
        "      modelName: 修改后的模板名," +
        "      oldModelName: 修改以前的模板名")
    @RequestMapping(value = "/updateModel",method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public  String updateModel(@RequestBody UpdateModelVo updateModelVo){
        if (StringUtils.isEmpty(updateModelVo.getModelName())){
            fb.setCode(0);
            fb.setMessage("修改的模板名不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateModelVo.getOldModelName())){
            fb.setCode(0);
            fb.setMessage("需要修改的模板名不能为空");
            return gson.toJson(fb);
        }
        if (modelService.getByModelName(updateModelVo.getModelName())!=null&&!updateModelVo.getModelName().equals(updateModelVo.getOldModelName())){
            fb.setCode(0);
            fb.setMessage("修改的模板名不能相同");
            return gson.toJson(fb);
        }
        updateModelVo.setUpdateTime(System.currentTimeMillis());
        int a=modelService.updateModel(updateModelVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage("修改模板成功");
        }else{
            fb.setCode(0);
            fb.setMessage("修改模板失败");
        }
        return gson.toJson(fb);
    }

    //删除
    @ApiOperation(value = "删除模板",notes = "删除模板需要传的参数:" +
        "      name: 删除的模板名," +
        "      companyName: 模板所属公司")
    @RequestMapping(value = "/delModel",method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public String delModel(@RequestBody String modelName){
        JSONObject jsonObject= JSON.parseObject(modelName);
        modelName=(String)jsonObject.get("modelName");
        if (StringUtils.isEmpty(modelName)){
            fb.setCode(0);
            fb.setMessage("删除的主题模板名不能为空");
            return gson.toJson(fb);
        }
        Map<String,Object> map=new HashMap<>();
        map.put("modelName",modelName);
        modelService.delModel(map);
        Map<String,Object> msg=new HashMap<>();
        msg.put("msg",map.get("msg"));
        if (msg.get("msg").equals("删除模板成功")){
            fb.setCode(1);
            fb.setMessage("删除模板成功");
        }else{
            fb.setCode(0);
            fb.setMessage(msg.get("msg").toString());
        }
        return gson.toJson(fb);
    }
}
