package com.example.tdl.web;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.AddModelInfoVo;
import com.example.tdl.domain.vo.DelModelInfoVo;
import com.example.tdl.domain.vo.ModelInfoTermVo;
import com.example.tdl.service.ModelInfoService;
import com.example.tdl.service.ModelService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;


@RestController
@RequestMapping("/topicConfigModel")
public class ModelInfoController {
    private Logger logger = LoggerFactory.getLogger(this.getClass());

    Gson gson = new Gson();

    CommFeedback fb = new CommFeedback();

    @Autowired
    private ModelInfoService modelInfoService;

    @Autowired
    private ModelService modelService;


    @ApiOperation(value = "查询所有配置模板",notes = "modelName是查询的模板名，查询所有配置模板的返回值对应的意义:" +
        "      modelName: 模板名," +
        "      probesModelName:通道模板名," +
        "      port: 通道模板的通道号," +
        "      unit: 通道模板的通道单位," +
        "      description: 通道模板的描述," +
        "      alias: 通道模板的别名," +
        "      pType: 通道模板的通道类型," +
        "      alarmUp: 通道模板的报警上限," +
        "      alarmDown: 通道模板的通道下限," +
        "      topicModelName:主题模板名" +
        "      topicName: 主题模板的主题名," +
        "      tType: 主题模板的主题类型," +
        "      intervalTime: 主题模板的间隔时间," +
        "      frequency:发送次数")
    @RequestMapping(value = "/getAll",method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public Object getAll(@RequestBody String modelName){
        JSONObject jsonObject= JSON.parseObject(modelName);
        modelName=(String)jsonObject.get("modelName");
        if(StringUtils.isEmpty(modelName)){
            fb.setCode(0);
            fb.setMessage("模板名不能为空");
            return gson.toJson(fb);
        }
        return modelInfoService.getAll(modelName);
    }

    @ApiOperation(value = "添加配置模板",notes = "添加配置模板传递的参数:" +
        "      modelName: 需要添加的模板名," +
        "      port: 通道模板的通道号," +
        "      probesModelName: 通道模板名," +
        "      topicName: 主题模板的主题名," +
        "      topicModelName: 主题模板名")
    @RequestMapping(value="/addTopicConfigModel",method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public String addTopicConfigModel(@RequestBody AddModelInfoVo addModelInfoVo){
        if (StringUtils.isEmpty(addModelInfoVo.getModelName())){
            fb.setCode(0);
            fb.setMessage("需要操作的模板名不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addModelInfoVo.getTopicModelName())){
            fb.setCode(0);
            fb.setMessage("需要操作的主题模板名不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addModelInfoVo.getTopicName())){
            fb.setCode(0);
            fb.setMessage("需要添加的主题名不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addModelInfoVo.getProbesModelName())){
            fb.setCode(0);
            fb.setMessage("需要操作的通道模板名不能为空");
            return gson.toJson(fb);
        }
        if (addModelInfoVo.getPort()==null){
            fb.setCode(0);
            fb.setMessage("需要添加的通道编号不能为空");
            return gson.toJson(fb);
        }
        if (modelService.getByModelName(addModelInfoVo.getModelName())==null){
            fb.setCode(0);
            fb.setMessage("没有该配置模板");
            return gson.toJson(fb);
        }
        if (modelInfoService.getByPortAndTopicName(new ModelInfoTermVo(addModelInfoVo.getModelName(),addModelInfoVo.getTopicModelName(),
                addModelInfoVo.getTopicName(),addModelInfoVo.getProbesModelName(),addModelInfoVo.getPort()))!=null){
            fb.setCode(0);
            fb.setMessage("添加的配置已存在");
            return gson.toJson(fb);
        }
        int a=modelInfoService.addModelInfo(addModelInfoVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage("添加模板配置成功");
        }else{
            fb.setCode(0);
            fb.setMessage("添加模板配置失败");
        }
        return gson.toJson(fb);
    }


    @ApiOperation(value = "删除配置模板",notes = "删除配置模板传递的参数:" +
        "      modelName: 需要删除的模板名," +
        "      port: 通道模板的通道号," +
        "      probesModelName: 通道模板名," +
        "      topicName: 主题模板的主题名," +
        "      topicModelName: 主题模板名" )
    @RequestMapping(value = "/delTopicConfigModel",method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public String delTopicConfigModel(@RequestBody DelModelInfoVo delModelInfoVo){
        if (StringUtils.isEmpty(delModelInfoVo.getModelName())){
            fb.setCode(0);
            fb.setMessage("需要操作的模板名不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(delModelInfoVo.getTopicModelName())){
            fb.setCode(0);
            fb.setMessage("需要操作的主题模板名不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(delModelInfoVo.getTopicName())){
            fb.setCode(0);
            fb.setMessage("需要删除的主题名不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(delModelInfoVo.getProbesModelName())){
            fb.setCode(0);
            fb.setMessage("需要操作的通道模板名不能为空");
            return gson.toJson(fb);
        }
        if (delModelInfoVo.getPort()==null){
            fb.setCode(0);
            fb.setMessage("需要删除的通道编号不能为空");
            return gson.toJson(fb);
        }
        if (modelService.getByModelName(delModelInfoVo.getModelName())==null){
            fb.setCode(0);
            fb.setMessage("没有该配置模板");
            return gson.toJson(fb);
        }
        int a=modelInfoService.delModelInfo(delModelInfoVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage("删除模板配置成功");
        }else{
            fb.setCode(0);
            fb.setMessage("删除模板配置失败");
        }
        return gson.toJson(fb);
    }
}