package com.example.tdl.web;

import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.*;
import com.example.tdl.service.GatewayService;
import com.example.tdl.service.ProbesService;
import com.example.tdl.service.redis.TokenRedisService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.dao.DataAccessException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.Map;

@RestController
@RequestMapping(path="/probes")
public class ProbesController {
    private Logger logger = LoggerFactory.getLogger(this.getClass());

    private CommFeedback fb = new CommFeedback();

    private Gson gson = new Gson();

    @Autowired
    private ProbesService probesService;

    @Autowired
    private GatewayService gatewayService;

    @Autowired
    private TokenRedisService tokenRedisService;

    //根据网关SN,type获取通道
    @ApiOperation(value = "根据网关SN,type获取通道",notes = "根据网关SN,type获取通道")
    @RequestMapping(value="/getAll",method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public Object getProbesByGateway(@RequestBody GatewaySNAndTypeVo gatewaySNAndTypeVo){
        if (StringUtils.isEmpty(gatewaySNAndTypeVo.getSN())){
            fb.setCode(0);
            fb.setMessage("网关序列号不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(gatewaySNAndTypeVo.getType())){
            fb.setCode(0);
            fb.setMessage("网关类型不能为空");
            return gson.toJson(fb);
        }
        if (gatewayService.getBySNAndType(gatewaySNAndTypeVo.getSN(),gatewaySNAndTypeVo.getType())==null){
            fb.setCode(0);
            fb.setMessage("没有该网关");
            return gson.toJson(fb);
        }
        return probesService.getProbesByGateway(gatewaySNAndTypeVo.getSN(),gatewaySNAndTypeVo.getType());
    }


    @ApiOperation(value = "添加单个通道",notes = "添加通道:" +
        "       SN:网关序列号," +
        "       gType:网关类型," +
        "       port:通道编号," +
        "       unit:单位," +
        "       description:通道描述," +
        "       alias:通道别名," +
        "       type:通道类型," +
        "       alarmUp:设备编号," +
        "       alarmDown:设备类型")
    @RequestMapping(value = "/addProbes", method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public String addProbes(@RequestBody AddProbesVo addProbesVo){
        if(StringUtils.isEmpty(addProbesVo.getSN())){
            fb.setCode(0);
            fb.setMessage("网关序列号不能为空");
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(addProbesVo.getgType())){
            fb.setCode(0);
            fb.setMessage("网关类型不能为空");
            return gson.toJson(fb);
        }
        if(addProbesVo.getPort()==null){
            fb.setCode(0);
            fb.setMessage("通道编号不能为空");
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(addProbesVo.getUnit())){
            fb.setCode(0);
            fb.setMessage("通道单位不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addProbesVo.getDescription())) {
            fb.setCode(0);
            fb.setMessage("通道描述不能为空");
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(addProbesVo.getAlias())){
            fb.setCode(0);
            fb.setMessage("通道别名不能为空");
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(addProbesVo.getType())){
            fb.setCode(0);
            fb.setMessage("通道类型不能为空");
            return gson.toJson(fb);
        }
        if(addProbesVo.getAlarmUp()==null){
            fb.setCode(0);
            fb.setMessage("报警上限不能为空");
            return gson.toJson(fb);
        }
        if(addProbesVo.getAlarmDown()==null){
            fb.setCode(0);
            fb.setMessage("报警下限不能为空");
            return gson.toJson(fb);
        }
        //判断当前设备的通道个数是否大于100
        if (probesService.getCountByGateway(addProbesVo.getSN(),addProbesVo.getgType())<100){
            if (probesService.getByPortAndGateway(addProbesVo.getSN(),addProbesVo.getgType(),addProbesVo.getPort())!=null) {
                fb.setCode(0);
                fb.setMessage("通道编号不能相同");
                return gson.toJson(fb);
            }
            if (probesService.getByAliasAndGateway(addProbesVo.getSN(),addProbesVo.getgType(),addProbesVo.getAlias())!=null) {
                fb.setCode(0);
                fb.setMessage("通道别名不能相同");
                return gson.toJson(fb);
            }
            if (probesService.getByDescriptionAndGateway(addProbesVo.getSN(),addProbesVo.getgType(),addProbesVo.getDescription())!=null) {
                fb.setCode(0);
                fb.setMessage("通道描述不能相同");
                return gson.toJson(fb);
            }
            if(gatewayService.getBySNAndType(addProbesVo.getSN(),addProbesVo.getgType())==null){
                fb.setCode(0);
                fb.setMessage("添加的网关不存在");
                return gson.toJson(fb);
            }
            Map<String, Object> map = new HashMap<>();
            map.put("SN", addProbesVo.getSN());
            map.put("gType", addProbesVo.getgType());
            map.put("port", addProbesVo.getPort());
            map.put("unit", addProbesVo.getUnit());
            map.put("description", addProbesVo.getDescription());
            map.put("alias", addProbesVo.getAlias());
            map.put("type", addProbesVo.getType());
            map.put("alarmUp", addProbesVo.getAlarmUp());
            map.put("alarmDown", addProbesVo.getAlarmDown());
            try {
                probesService.addProbes(map);
                Map<String, Object> msg = new HashMap<>();
                msg.put("msg", map.get("msg"));
                if (msg.get("msg").equals("添加成功")) {
                    fb.setCode(1);
                    fb.setMessage(msg.get("msg").toString());
                } else {
                    fb.setCode(0);
                    fb.setMessage(msg.get("msg").toString());
                }
            } catch (Exception e) {
                e.printStackTrace();
                logger.info(e.toString());
                fb.setCode(0);
                fb.setMessage("添加失败");
            }
        } else{
            fb.setCode(0);
            fb.setMessage("每个网关最多100个通道");
        }
        return gson.toJson(fb);
    }

    @ApiOperation(value = "删除单个通道",notes = "删除单个通道的时候根据网关和通道号删除" +
        "       SN:设备编号," +
        "       type:设备类型," +
        "       port:通道编号")
    @RequestMapping(value = "/delAProbes", method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public String delAProbes(@RequestBody DelAProbesVo delAProbesVo){
        if(StringUtils.isEmpty(delAProbesVo.getType())){
            fb.setCode(0);
            fb.setMessage("网关类型不能为空");
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(delAProbesVo.getSN())){
            fb.setCode(0);
            fb.setMessage("网关序列号不能为空");
            return gson.toJson(fb);
        }
        if(delAProbesVo.getPort()==null){
            fb.setCode(0);
            fb.setMessage("通道编号不能为空");
            return gson.toJson(fb);
        }
        if(gatewayService.getBySNAndType(delAProbesVo.getSN(),delAProbesVo.getType())==null){
            fb.setCode(0);
            fb.setMessage("该网关不存在");
            return gson.toJson(fb);
        }
        Map<String,Object> map=new HashMap<>();
        map.put("SN",delAProbesVo.getSN());
        map.put("type",delAProbesVo.getType());
        map.put("port",delAProbesVo.getPort());
        try {
            probesService.delAProbes(map);
            Map<String,Object> msg=new HashMap<>();
            msg.put("msg",map.get("msg"));
            if (msg.get("msg").equals("删除通道成功")){
                fb.setCode(1);
                fb.setMessage("删除通道成功");
            }else{
                fb.setCode(0);
                fb.setMessage(msg.get("msg").toString());
            }
        }catch (DataAccessException e){
            e.printStackTrace();
            fb.setCode(0);
            fb.setMessage("删除失败");
        }
        return gson.toJson(fb);
    }


    @ApiOperation(value = "删除所有通道",notes = "根据设备信息删除该设备下所有通道" +
        "       SN:设备编号," +
        "       type:设备类型")
    @RequestMapping(value = "/delAllProbes", method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public String delAllProbes(@RequestBody GatewaySNAndTypeVo gatewaySNAndTypeVo){
        if(StringUtils.isEmpty(gatewaySNAndTypeVo.getType())){
            fb.setCode(0);
            fb.setMessage("网关类型不能为空");
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(gatewaySNAndTypeVo.getSN())){
            fb.setCode(0);
            fb.setMessage("网关序列号不能为空");
            return gson.toJson(fb);
        }
        if(gatewayService.getBySNAndType(gatewaySNAndTypeVo.getSN(),gatewaySNAndTypeVo.getType())==null){
            fb.setCode(0);
            fb.setMessage("该网关不存在");
            return gson.toJson(fb);
        }
        Map<String,Object> map=new HashMap<>();
        map.put("SN",gatewaySNAndTypeVo.getSN());
        map.put("type",gatewaySNAndTypeVo.getType());
        try {
            probesService.delAllProbes(map);
            Map<String,Object> msg=new HashMap<>();
            msg.put("msg",map.get("msg"));
            if (msg.get("msg").equals("删除通道成功")){
                fb.setCode(1);
                fb.setMessage(msg.get("msg").toString());
            }else{
                fb.setCode(0);
                fb.setMessage(msg.get("msg").toString());
            }
        }catch (DataAccessException e){
            e.printStackTrace();
            fb.setCode(0);
            fb.setMessage("删除失败");
        }
        return gson.toJson(fb);
    }

    @ApiOperation(value = "修改通道",notes = "根据网关信息和通道号修改通道" +
        "       port:需要修改的通道编号," +
        "       unit：修改后的单位," +
        "       alias:修改后的别名," +
        "       description：修改后的通道描述," +
        "       type:修改后的通道类型," +
        "       alarmUp：修改后的报警上线," +
        "       alarmDown：修改后的报警下线," +
        "       SN：需要修改的网关序列号," +
        "       gType：需要修改的网关类型")
    @RequestMapping(value = "/updateProbes" ,method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public String updateProbes(@RequestBody UpdateProbesVo updateProbesVo){
        if(StringUtils.isEmpty(updateProbesVo.getSN())){
            fb.setCode(0);
            fb.setMessage("网关序列号不能为空");
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(updateProbesVo.getgType())){
            fb.setCode(0);
            fb.setMessage("网关类型不能为空");
            return gson.toJson(fb);
        }
        if(updateProbesVo.getPort()==null){
            fb.setCode(0);
            fb.setMessage("通道编号不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateProbesVo.getDescription())) {
            fb.setCode(0);
            fb.setMessage("通道描述不能为空");
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(updateProbesVo.getAlias())){
            fb.setCode(0);
            fb.setMessage("通道别名不能为空");
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(updateProbesVo.getType())){
            fb.setCode(0);
            fb.setMessage("通道类型不能为空");
            return gson.toJson(fb);
        }
        if (updateProbesVo.getAlarmUp()==null){
            fb.setCode(0);
            fb.setMessage("报警上限不能为空");
            return gson.toJson(fb);
        }
        if (updateProbesVo.getAlarmDown()==null){
            fb.setCode(0);
            fb.setMessage("报警下限不能为空");
            return gson.toJson(fb);
        }
        if(gatewayService.getBySNAndType(updateProbesVo.getSN(),updateProbesVo.getgType())==null){
            fb.setCode(0);
            fb.setMessage("该网关不存在");
            return gson.toJson(fb);
        }
        //根据SN,type获取设备状态
        boolean state = gatewayService.getBySNAndType(updateProbesVo.getSN(),updateProbesVo.getgType()).getState();
        if(state){
            fb.setCode(0);
            fb.setMessage("设备在挂载，请先卸载设备");
            return gson.toJson(fb);
        }
        ResultProbesVo resultProbesVo1= probesService.getByAliasAndGateway(updateProbesVo.getSN(),updateProbesVo.getgType(),updateProbesVo.getAlias());
        ResultProbesVo resultProbesVo2= probesService.getByDescriptionAndGateway(updateProbesVo.getSN(),updateProbesVo.getgType(),updateProbesVo.getDescription());
        if(resultProbesVo1!=null && !resultProbesVo1.getPort().equals(updateProbesVo.getPort())){
            fb.setCode(0);
            fb.setMessage("该别名已经存在");
            return gson.toJson(fb);
        }
        if(resultProbesVo2!=null && !resultProbesVo2.getPort().equals(updateProbesVo.getPort())){
            fb.setCode(0);
            fb.setMessage("该描述已经存在");
            return gson.toJson(fb);
        }
        try {
            int d=probesService.updateProbes(updateProbesVo);
            if (d>0){
                fb.setCode(1);
                fb.setMessage("修改成功");
            }else{
                fb.setCode(0);
                fb.setMessage("修改失败");
            }
        }catch (DataAccessException e){
            e.printStackTrace();
            fb.setCode(0);
            fb.setMessage("修改失败");
        }
        return gson.toJson(fb);
    }


}
