package com.example.tdl.web;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.AddProbesModelVo;
import com.example.tdl.domain.vo.UpdateProbesModelVo;
import com.example.tdl.service.ProbesModelService;
import com.example.tdl.service.redis.TokenRedisService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.Map;


@RestController
@RequestMapping(path="/probesModel")
public class ProbesModelController {
    Gson gson = new Gson();

    CommFeedback fb = new CommFeedback();

    private Logger logger = LoggerFactory.getLogger(this.getClass());

    @Autowired
    private ProbesModelService probesModelService;

    @Autowired
    private TokenRedisService tokenRedisService;

    //获取所有的通道模板
    @ApiOperation(value = "获取所有通道模板",notes = "返回参数:" +
        "      probesModelName: 模板名," +
        "      countNum: 模板下通道数量," +
        "      createTime: 创建时间," +
        "      updateTime: 修改时间")
    @RequestMapping(value = "/getAll",method = RequestMethod.GET)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public Object getAll(){
        return probesModelService.getAll();
    }

    //新增模板
    @ApiOperation(value = "新增通道模板" ,notes = "新增模板的同时，新增相应数量的模板通道，模板不能超过20个，每个模板的模板通道不能超过100个" +
        "       probesModelName:模板名，" +
        "       countNum：通道数量，")
    @RequestMapping(value = "/addProbesModel" ,method =  RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public String  addProbesModel(@RequestBody AddProbesModelVo addProbesModelVo){
        if(StringUtils.isEmpty(addProbesModelVo.getProbesModelName())){
            fb.setCode(0);
            fb.setMessage("模板名不能为空");
            return gson.toJson(fb);
        }
        if (addProbesModelVo.getCountNum()==null){
            fb.setCode(0);
            fb.setMessage("新增的模板通道数量不能为空");
            return gson.toJson(fb);
        }
        if(addProbesModelVo.getCountNum()>100){
            fb.setCode(0);
            fb.setMessage("新增的模板通道数量不能超过100");
            return gson.toJson(fb);
        }
        int a=probesModelService.getProbesModelCount();
        if (a>20){
            fb.setCode(0);
            fb.setMessage("对不起，只能添加20个模板");
            return gson.toJson(fb);
        }
        Map<String, Object> map = new HashMap<>();
        map.put("probesModelName", addProbesModelVo.getProbesModelName());
        map.put("countNum", addProbesModelVo.getCountNum());
        map.put("createTime",System.currentTimeMillis());
        map.put("updateTime",System.currentTimeMillis());
        map.put("alarmUp", 0);
        map.put("alarmDown", 0);
        try {
            probesModelService.addProbesModel(map);
            Map<String, Object> ms = new HashMap<>();
            ms.put("msg", map.get("msg"));
            if (ms.get("msg").equals("添加模板成功")) {
                fb.setCode(1);
                fb.setMessage(map.get("msg").toString());
            } else {
                fb.setCode(0);
                fb.setMessage(map.get("msg").toString());
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.info(e.toString());
            fb.setCode(0);
            fb.setMessage("添加失败");
        }
        return  gson.toJson(fb);
    }

    //修改模板
    @ApiOperation(value = "修改通道模板",notes = "修改只能修改模板名，updateTime不用传，以下必传" +
        "      probesModelName: 模板名," +
        "      oldProbesModelName: 需要修改的模板名")
    @RequestMapping(value = "/updateProbesModel",method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public String updateProbesModel(@RequestBody UpdateProbesModelVo updateProbesModelVo){
        if (StringUtils.isEmpty(updateProbesModelVo.getOldProbesModelName())){
            fb.setCode(0);
            fb.setMessage("需要修改的模板名不能为空");
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(updateProbesModelVo.getProbesModelName())){
            fb.setCode(0);
            fb.setMessage("模板名称不能为空");
            return gson.toJson(fb);
        }
        if (probesModelService.getByProbesModelName(updateProbesModelVo.getProbesModelName())!=null&&
                !updateProbesModelVo.getProbesModelName().equals(updateProbesModelVo.getOldProbesModelName())){
            fb.setCode(0);
            fb.setMessage("该模板已存在");
            return gson.toJson(fb);
        }
        updateProbesModelVo.setUpdateTime(System.currentTimeMillis());
        int a=probesModelService.updateChannelModule(updateProbesModelVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage("修改成功");
        }else{
            fb.setCode(0);
            fb.setMessage("修改模板失败");
        }
        return gson.toJson(fb);
    }

    //删除模板
    @ApiOperation(value = "删除通道模板",notes = "删除模板的传参，以下必传" +
        "      name: 模板名")
    @RequestMapping(value = "/delProbesModel",method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public String  delProbesModel(@RequestBody String probesModelName){
        JSONObject jsonObject= JSON.parseObject(probesModelName);
        probesModelName=(String)jsonObject.get("probesModelName");
        if(StringUtils.isEmpty(probesModelName)){
            fb.setCode(0);
            fb.setMessage("模板名不能为空");
            return gson.toJson(fb);
        }
        Map<String,Object> params=new HashMap<String, Object>();
        params.put("probesModelName",probesModelName);
        try {
            probesModelService.delProbesModel(params);
            Map<String,Object> ms=new HashMap<>();
            ms.put("msg",params.get("msg"));
            if(ms.get("msg").equals("删除模板成功")){
                fb.setCode(1);
                fb.setMessage(ms.get("msg").toString());
            }else{
                fb.setCode(0);
                fb.setMessage(ms.get("msg").toString());
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.info(e.toString());
            fb.setCode(0);
            fb.setMessage("删除失败");
        }
        return gson.toJson(fb);
    }
}
