package com.example.tdl.web;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.*;
import com.example.tdl.service.ProbesModelInfoService;
import com.example.tdl.service.ProbesModelService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.Map;

@RestController
@RequestMapping(path="/probesModelInfo")
public class ProbesModelInfoController {
    private Logger logger = LoggerFactory.getLogger(this.getClass());

    private Gson gson = new Gson();

    private CommFeedback fb = new CommFeedback();

    @Autowired
    private ProbesModelInfoService probesModelInfoService;

    @Autowired
    private ProbesModelService probesModelService;

    //根据模板名获取所有的通道信息
    @ApiOperation(value = "根据通道模板名获取所有的通道模板信息",notes = "根据通道模板名获取所有的通道模板信息：" +
        "    probesModelName: 通道模板名," +
        "    port: 通道模板编号," +
        "    unit: 通道模板单位" +
        "    description: 通道模板描述," +
        "    alias: 通道模板别名," +
        "    type: 通道模板类型," +
        "    alarmUp: 通道模板报警上限," +
        "    alarmDown: 通道模板报警下限")
    @RequestMapping(value = "/getAll",method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public Object getAll(@RequestBody String probesModelName){
        JSONObject jsonObject= JSON.parseObject(probesModelName);
        probesModelName=(String) jsonObject.get("probesModelName");
        if (StringUtils.isEmpty(probesModelName)){
            fb.setCode(0);
            fb.setMessage("需要查询的通道模板不能为空");
            return gson.toJson(fb);
        }
        return probesModelInfoService.getAll(probesModelName);
    }

    //新增通道
    @ApiOperation(value = "添加单个通道模板",notes = "添加单个通道模板传递参数：" +
        "    probesModelName: 新增模板通道的模板名," +
        "    port: 通道模板编号," +
        "    unit: 通道模板单位," +
        "    description: 通道模板描述," +
        "    alias: 通道模板别名," +
        "    type: 通道模板类型," +
        "    alarmUp: 通道模板报警上限," +
        "    alarmDown: 通道模板报警下限")
    @RequestMapping(value="/addProbesModelInfo",method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public String  addProbesModelInfo(@RequestBody ResultProbesModelInfoVo resultProbesModelInfoVo){
        if (StringUtils.isEmpty(resultProbesModelInfoVo.getProbesModelName())){
            fb.setCode(0);
            fb.setMessage("添加的模板名不能为空");
            return gson.toJson(fb);
        }
        if (resultProbesModelInfoVo.getPort()==null){
            fb.setCode(0);
            fb.setMessage("通道模板通道编号不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(resultProbesModelInfoVo.getUnit())){
            fb.setCode(0);
            fb.setMessage("通道模板通道单位不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(resultProbesModelInfoVo.getDescription())){
            fb.setCode(0);
            fb.setMessage("通道模板通道描述不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(resultProbesModelInfoVo.getAlias())){
            fb.setCode(0);
            fb.setMessage("通道模板通道别名不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(resultProbesModelInfoVo.getType())){
            fb.setCode(0);
            fb.setMessage("通道模板通道类型不能为空");
            return gson.toJson(fb);
        }
        if (resultProbesModelInfoVo.getAlarmUp()==null){
            fb.setCode(0);
            fb.setMessage("通道模板通道报警上限不能为空");
            return gson.toJson(fb);
        }
        if (resultProbesModelInfoVo.getAlarmDown()==null){
            fb.setCode(0);
            fb.setMessage("通道模板通道报警下限不能为空");
            return gson.toJson(fb);
        }
        if (probesModelService.getByProbesModelName(resultProbesModelInfoVo.getProbesModelName())==null){
            fb.setCode(0);
            fb.setMessage("添加的模板名不存在");
            return gson.toJson(fb);
        }
        Integer probesModelId=probesModelService.getIdByProbesModel(resultProbesModelInfoVo.getProbesModelName());
        if(probesModelInfoService.getCountByProbesModelName(probesModelId)>=100) {
            fb.setCode(0);
            fb.setMessage("每个模板只能添加100个通道模板");
            return gson.toJson(fb);
        }
        if (probesModelInfoService.getByPort(probesModelId,resultProbesModelInfoVo.getPort())!=null){
            fb.setCode(0);
            fb.setMessage("添加的模板通道编号不能重复");
            return gson.toJson(fb);
        }
        if (probesModelInfoService.getByAlias(probesModelId,resultProbesModelInfoVo.getAlias())!=null){
            fb.setCode(0);
            fb.setMessage("添加的模板通道别名不能重复");
            return gson.toJson(fb);
        }
        int a = probesModelInfoService.addProbesModelInfo(new AddProbesModelInfoVo(probesModelId,resultProbesModelInfoVo.getPort(),
                resultProbesModelInfoVo.getUnit(),resultProbesModelInfoVo.getDescription(),resultProbesModelInfoVo.getAlias(),
                resultProbesModelInfoVo.getType(),resultProbesModelInfoVo.getAlarmUp(),resultProbesModelInfoVo.getAlarmDown()));
        if (a>0){
            fb.setCode(1);
            fb.setMessage("添加单个通道模板成功");
        }else{
            fb.setCode(0);
            fb.setMessage("添加单个通道模板失败");
        }
        return gson.toJson(fb);
    }

    //删除单个通道模板
    @ApiOperation(value = "删除单个通道模板",notes = "删除单个通道模板传递参数改变" +
        "    port: 需要删除的通道模板编号" +
        "    probesModelName: 需要删除的通道模板名")
    @RequestMapping(value="/delProbesModelInfo",method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public String delProbesModelInfo(@RequestBody DelProbesModelInfoVo delProbesModelInfoVo){
        if (StringUtils.isEmpty(delProbesModelInfoVo.getProbesModelName())){
            fb.setCode(0);
            fb.setMessage("需要删除的模板名不能为空");
            return gson.toJson(fb);
        }
        if (delProbesModelInfoVo.getPort()==null){
            fb.setCode(0);
            fb.setMessage("需要删除的通道模板编号不能为空");
            return gson.toJson(fb);
        }

        if (probesModelService.getByProbesModelName(delProbesModelInfoVo.getProbesModelName())==null){
            fb.setCode(0);
            fb.setMessage("需要删除的模板名不存在");
            return gson.toJson(fb);
        }
        Map<String,Object> params=new HashMap<String, Object>();
        params.put("probesModelName",delProbesModelInfoVo.getProbesModelName());
        params.put("port",delProbesModelInfoVo.getPort());
        try {
            probesModelInfoService.delProbesModelInfo(params);
            Map<String,Object> ms=new HashMap<>();
            ms.put("msg",params.get("msg"));
            if(ms.get("msg").equals("删除通道模板成功")){
                fb.setCode(1);
                fb.setMessage(ms.get("msg").toString());
            }else{
                fb.setCode(0);
                fb.setMessage(ms.get("msg").toString());
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.toString();
            fb.setCode(0);
            fb.setMessage("删除失败");
        }
        return gson.toJson(fb);
    }


    //修改单个通道模板
    @ApiOperation(value = "修改单个通道模板",notes = "修改单个通道模板,port不能改：" +
        "    probesModelName: 需要修改的模板名," +
        "    port: 需要修改的通道模板编号," +
        "    unit: 通道模板单位," +
        "    description: 通道模板描述," +
        "    alias: 通道模板别名," +
        "    type: 通道模板类型," +
        "    alarmUp: 通道模板报警上限," +
        "    alarmDown: 通道模板报警下限")
    @RequestMapping(value = "/updateProbesModelInfo",method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public String updateProbesModelInfo(@RequestBody ResultProbesModelInfoVo resultProbesModelInfoVo){
        if (StringUtils.isEmpty(resultProbesModelInfoVo.getProbesModelName())){
            fb.setCode(0);
            fb.setMessage("需要修改的模板名不能为空");
            return gson.toJson(fb);
        }
        if (resultProbesModelInfoVo.getPort()==null){
            fb.setCode(0);
            fb.setMessage("需要修改的通道模板编号不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(resultProbesModelInfoVo.getUnit())){
            fb.setCode(0);
            fb.setMessage("通道模板单位不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(resultProbesModelInfoVo.getDescription())){
            fb.setCode(0);
            fb.setMessage("通道模板描述不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(resultProbesModelInfoVo.getAlias())){
            fb.setCode(0);
            fb.setMessage("通道模板别名不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(resultProbesModelInfoVo.getType())){
            fb.setCode(0);
            fb.setMessage("通道模板类型不能为空");
            return gson.toJson(fb);
        }
        if (resultProbesModelInfoVo.getAlarmUp()==null){
            fb.setCode(0);
            fb.setMessage("通道模板报警上限不能为空");
            return gson.toJson(fb);
        }
        if (resultProbesModelInfoVo.getAlarmDown()==null){
            fb.setCode(0);
            fb.setMessage("通道模板报警下限不能为空");
            return gson.toJson(fb);
        }
        if (probesModelService.getByProbesModelName(resultProbesModelInfoVo.getProbesModelName())==null){
            fb.setCode(0);
            fb.setMessage("需要修改的模板名不存在");
            return gson.toJson(fb);
        }
        Integer probesModelId=probesModelService.getIdByProbesModel(resultProbesModelInfoVo.getProbesModelName());
        ResultProbesModelInfoVo resultAlias=probesModelInfoService.getByAlias(probesModelId,resultProbesModelInfoVo.getAlias());
        if (resultAlias!=null&&resultAlias.getPort()!=resultProbesModelInfoVo.getPort()){
            fb.setCode(0);
            fb.setMessage("修改的别名不能重复");
            return gson.toJson(fb);
        }

        int a = probesModelInfoService.updateProbesModelInfo(new UpdateProbesModelInfoVo(probesModelId,resultProbesModelInfoVo.getPort(),
                resultProbesModelInfoVo.getUnit(),resultProbesModelInfoVo.getDescription(),resultProbesModelInfoVo.getAlias(),
                resultProbesModelInfoVo.getType(),resultProbesModelInfoVo.getAlarmUp(),resultProbesModelInfoVo.getAlarmDown()));
        if (a>0) {
            fb.setCode(1);
            fb.setMessage("修改通道模板成功");
        } else {
            fb.setCode(0);
            fb.setMessage("修改通道模板失败");
        }
        return gson.toJson(fb);
    }

    //修改多个通道模板
//    @ApiOperation(value = "修改多个通道模板",notes = "修改多个通道模板需传递多个对象")
//    @RequestMapping(value = "/updateMoreDevChannelModule",method = RequestMethod.POST)
//    public String updateMoreDevChannelModule(@RequestBody List<UpdateDevChannelModule> updateDevChannelModule) {
//        String portVar="";
//        String unitVar="";
//        String descriptionVar="";
//        String typeVar="";
//        int num=updateDevChannelModule.size();
//        User user = (User) SecurityUtils.getSubject().getSession().getAttribute("user");
//        if (user==null){
//            fb.setCode(0);
//            fb.setMessage("您没有登录，请先登录");
//        }else{
//            Role role = roleService.getRoleId(user.getUserName());
//            if (role==null){
//                fb.setCode(0);
//                fb.setMessage("对不起，您没有此权限");
//            }else{
//                List<String> actions = roleHasActionService.getActionByRoleName(role.getRoleName(), user.getCompanyName());
//                if (actions.contains("模板修改")) {
//                    if (updateDevChannelModule.get(0).getCompanyName().equals(user.getCompanyName())){
//                        for (int i=0;i<num;i++){
//                            portVar+=updateDevChannelModule.get(i).getPort()+" ";
//                            unitVar+=updateDevChannelModule.get(i).getUnit()+" ";
//                            descriptionVar+=updateDevChannelModule.get(i).getDevChannelName()+"("+updateDevChannelModule.get(i).getDescription()+")"+" ";
//                            typeVar+=updateDevChannelModule.get(i).getType()+" ";
//                        }
//                        Map<String,Object> map=new HashMap<>();
//                        map.put("name",updateDevChannelModule.get(0).getName());
//                        map.put("companyName",user.getCompanyName());
//                        map.put("num",num);
//                        map.put("portVar",portVar);
//                        map.put("unitVar",unitVar);
//                        map.put("descriptionVar",descriptionVar);
//                        map.put("typeVar",typeVar);
//                        devChannelModuleService.updateMoreDevChannelModule(map);
//                        Map<String,Object> msg=new HashMap<>();
//                        msg.put("msg",map.get("msg"));
//                        if (msg.get("msg").equals("修改成功")){
//                            fb.setCode(1);
//                            fb.setMessage(map.get("msg").toString());
//                        }else{
//                            fb.setCode(0);
//                            fb.setMessage(map.get("msg").toString());
//                        }
//                    }else{
//                        fb.setCode(0);
//                        fb.setMessage("只能修改自己公司的通道模板");
//                    }
//                } else {
//                    fb.setCode(0);
//                    fb.setMessage("对不起，您没有此权限");
//                    return gson.toJson(fb);
//                }
//            }
//        }
//        return gson.toJson(fb);
//    }


    //    //删除所有通道模板
//    @ApiOperation(value = "删除多个通道模板",notes = "删除某模板下所有的通道模板只需传模板名")
//    @RequestMapping(value="/delAllDevChannelModule",method = RequestMethod.POST)
//    public String  delAllDevChannelModule(@RequestBody String channelModelName){
//        JSONObject jsonObject=JSONObject.parseObject(channelModelName);
//        String name=jsonObject.getString("channelModelName");
//        User user = (User) SecurityUtils.getSubject().getSession().getAttribute("user");
//        if (user==null){
//            fb.setCode(0);
//            fb.setMessage("您没有登录，请先登录");
//        }else{
//            Role role = roleService.getRoleId(user.getUserName());
//            if (role==null){
//                fb.setCode(0);
//                fb.setMessage("对不起，您没有此权限");
//            }else{
//                ResultChannelModuleVo cmForID=channelModuleServcie.getByName(name,user.getCompanyName());//获取需要删除的id
//                if (cmForID!=null){
//                    List<String> actions = roleHasActionService.getActionByRoleName(role.getRoleName(), user.getCompanyName());
//                    if (actions.contains("模板删除")) {
//                        Map<String,String> params=new HashMap<String, String>();
//                        params.put("name",name);
//                        params.put("companyName",user.getCompanyName());
//                        try {
//                            devChannelModuleService.delAllDevChannelModule(params);
//                            Map<String,String> ms=new HashMap<>();
//                            ms.put("msg",params.get("msg"));
//                            if(ms.get("msg").equals("删除成功")){
//                                fb.setCode(1);
//                                fb.setMessage(ms.get("msg").toString());
//                            }else{
//                                fb.setCode(0);
//                                fb.setMessage(ms.get("msg").toString());
//                            }
//                        }catch (Exception e){
//                            fb.setCode(0);
//                            fb.setMessage("删除失败");
//                        }
//                    } else {
//                        fb.setCode(0);
//                        fb.setMessage("对不起，您没有此权限");
//                        return gson.toJson(fb);
//                    }
//                }else{
//                    fb.setCode(0);
//                    fb.setMessage("对不起，该模板不属于贵公司");
//                }
//            }
//        }
//        return gson.toJson(fb);
//    }

}
