package com.example.tdl.web;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.*;
import com.example.tdl.service.TDLDeviceService;
import com.example.tdl.service.redis.TokenRedisService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;


@RestController
@RequestMapping("/tdlDevice")
public class TDLDeviceController {

    private Logger logger = LoggerFactory.getLogger(TDLDeviceController.class);

    CommFeedback fb=new CommFeedback();

    Gson gson=new Gson();

    @Autowired
    private TDLDeviceService tdlDeviceService=new TDLDeviceService();

    @Autowired
    private TokenRedisService tokenRedisService;

    //查询TDL信息
    @ApiOperation(value = "查询TDL信息",notes = "查询TDL信息，返回值说明：" +
            "   TDLName:TDL名字," +
            "   TDLSN:TDL序列号," +
            "   counts:使用次数," +
            "   lastTime:最后使用时间," +
            "   useScene:使用场景（0为空闲，1为仓库，2为运输）," +
            "   warehouseName:仓库名," +
            "   transportationNo:运输编号," +
            "   gatewaySN:网关序列号," +
            "   gatewayType:网关类型。" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/getAll",method = RequestMethod.GET)
    public Object getAll(){
        return tdlDeviceService.getAll();
    }

    //根据条件查询TDL信息
    @ApiOperation(value = "根据条件查询TDL信息",notes = "根据条件查询TDL信息，传值说明：" +
            "   TDLName:TDL名字," +
            "   TDLSN:TDL序列号," +
            "   useScene:使用场景（0为空闲，1为仓库，2为运输）。" +
            "返回值说明：" +
            "   TDLName:TDL名字," +
            "   TDLSN:TDL序列号," +
            "   counts:使用次数," +
            "   lastTime:最后使用时间," +
            "   useScene:使用场景（0为空闲，1为仓库，2为运输）," +
            "   warehouseName:仓库名," +
            "   transportationNo:运输编号," +
            "   gatewaySN:网关序列号," +
            "   gatewayType:网关类型。" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/getByTerm",method = RequestMethod.POST)
    public Object getByTerm(@RequestBody TDLDeviceTermVo tdlDeviceTermVo){
        return tdlDeviceService.getByTerm(tdlDeviceTermVo);
    }

    //添加TDL信息
    @ApiOperation(value = "添加TDL信息",notes = "添加TDL信息，传值说明：" +
            "   TDLName:TDL名字," +
            "   TDLSN:TDL序列号")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/addTDLDevice",method = RequestMethod.POST)
    public Object addTDLDevice(@RequestBody AddTDLDeviceVo addTDLDeviceVo){
        if (StringUtils.isEmpty(addTDLDeviceVo.getTDLName())){
            fb.setCode(0);
            fb.setMessage("TDL名称不能为空");
            return  gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addTDLDeviceVo.getTDLSN())){
            fb.setCode(0);
            fb.setMessage("TDL序列号不能为空");
            return  gson.toJson(fb);
        }
        if (tdlDeviceService.getByTDLSN(addTDLDeviceVo.getTDLSN())!=null){
            fb.setCode(0);
            fb.setMessage("添加的TDL序列号已存在");
            return gson.toJson(fb);
        }
        int a = tdlDeviceService.addTDLDevice(addTDLDeviceVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage("添加TDL信息成功");
        }else{
            fb.setCode(0);
            fb.setMessage("添加TDL信息失败");
        }
        return gson.toJson(fb);
    }

    //修改TDL信息
    @ApiOperation(value = "修改TDL信息",notes = "修改TDL信息，TDLSN不能改，传值说明：" +
            "   TDLName:TDL名字," +
            "   TDLSN:需要修改的TDL序列号")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/updateTDLDevice",method = RequestMethod.POST)
    public Object updateTDLDevice(@RequestBody UpdateTDLDeviceVo updateTDLDeviceVo){

        if (StringUtils.isEmpty(updateTDLDeviceVo.getTDLName())){
            fb.setCode(0);
            fb.setMessage("TDL名称不能为空");
            return  gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateTDLDeviceVo.getTDLSN())){
            fb.setCode(0);
            fb.setMessage("需要修改的TDL序列号不能为空");
            return  gson.toJson(fb);
        }
        int a = tdlDeviceService.updateTDLDevice(updateTDLDeviceVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage("修改TDL信息成功");
        }else{
            fb.setCode(0);
            fb.setMessage("修改TDL信息失败");
        }
        return gson.toJson(fb);
    }

    //删除TDL信息
    @ApiOperation(value = "删除TDL信息",notes = "删除TDL信息，传值说明：" +
            "   TDLSN:需要删除的TDL序列号")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/delTDLDevice",method = RequestMethod.POST)
    public Object delTDLDevice(@RequestBody String TDLSN){
        JSONObject jsonObject=JSON.parseObject(TDLSN);
        TDLSN=(String)jsonObject.get("TDLSN");
        if (StringUtils.isEmpty(TDLSN)){
            fb.setCode(0);
            fb.setMessage("需要删除的TDL序列号不能为空");
            return  gson.toJson(fb);
        }
        int a = tdlDeviceService.delTDLDevice(TDLSN);
        if (a>0){
            fb.setCode(1);
            fb.setMessage("删除TDL信息成功");
        }else{
            fb.setCode(0);
            fb.setMessage("删除TDL信息失败");
        }
        return gson.toJson(fb);

    }
}
