package com.example.tdl.web;

import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.AddTopicConfigVo;
import com.example.tdl.domain.vo.ResultTopicConfigVo;
import com.example.tdl.service.TopicConfigService;
import com.example.tdl.service.redis.TokenRedisService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping("/topicConfig")
public class TopicConfigController {
    private Logger logger = LoggerFactory.getLogger(this.getClass());

    Gson gson = new Gson();

    CommFeedback fb = new CommFeedback();

    @Autowired
    private TopicConfigService topicConfigService;

    @Autowired
    private TokenRedisService tokenRedisService;

    @ApiOperation(value = "根据主题名查看该主题下所有的通道",notes = "需要传主题名,返回值：" +
            "   port:通道编号," +
            "   unit:通道单位," +
            "   description:通道描述," +
            "   alias:通道别名," +
            "   type:通道类型," +
            "   alarmUp:报警上限," +
            "   alarmDown:报警下限," +
            "   topicName:主题名," +
            "   tType:主题类型," +
            "   intervalTime:间隔时间," +
            "   frequency:发送次数")
    @RequestMapping(value = "/getAll",method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public Object getAll(@RequestBody String topicName){
        //创建JSONObject对象
        JSONObject json =JSONObject.parseObject(topicName);
        topicName=(String)json.get("topicName");
        if (StringUtils.isEmpty(topicName)){
            fb.setCode(0);
            fb.setMessage("查询的主题名不能为空");
            return gson.toJson(fb);
        }
        List<ResultTopicConfigVo> list = topicConfigService.getByTopicName(topicName);
        return list;
    }

    @ApiOperation(value = "添加主题配置",notes = "根据主题名和通道编号添加主题配置，sn,type不用传，传递参数：" +
            "   topicName:主题名," +
            "   port:通道编号,")
    @RequestMapping(value="/addTopicConfig",method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public String addTopicConfig(@RequestBody AddTopicConfigVo addTopicConfigVo){
        if (StringUtils.isEmpty(addTopicConfigVo.getTopicName())){
            fb.setCode(0);
            fb.setMessage("添加的主题名不能为空");
            return gson.toJson(fb);
        }
        if (addTopicConfigVo.getPort()==null){
            fb.setCode(0);
            fb.setMessage("添加的通道编号不能为空");
            return gson.toJson(fb);
        }
        String SN =addTopicConfigVo.getTopicName().split("/")[2];
        String type =addTopicConfigVo.getTopicName().split("/")[1];
        addTopicConfigVo.setSN(SN);
        addTopicConfigVo.setType(type);
        int a =topicConfigService.addTopicConfig(addTopicConfigVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage("添加主题配置成功");
        }else {
            fb.setCode(0);
            fb.setMessage("添加主题配置失败");
        }
        return gson.toJson(fb);
    }

    @ApiOperation(value = "删除主题配置",notes = "根据主题名和通道编号删除主题配置，SN,type不用传,传递参数：" +
            "   topicName:主题名," +
            "   port:通道编号")
    @RequestMapping(value = "/delTopicConfig",method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public String delTopicConfig(@RequestBody AddTopicConfigVo addTopicConfigVo){
        if (StringUtils.isEmpty(addTopicConfigVo.getTopicName())){
            fb.setCode(0);
            fb.setMessage("删除的主题名不能为空");
            return gson.toJson(fb);
        }
        if (addTopicConfigVo.getPort()==null){
            fb.setCode(0);
            fb.setMessage("删除的通道编号不能为空");
            return gson.toJson(fb);
        }
        String SN =addTopicConfigVo.getTopicName().split("/")[2];
        String type =addTopicConfigVo.getTopicName().split("/")[1];
        addTopicConfigVo.setSN(SN);
        addTopicConfigVo.setType(type);
        int i =topicConfigService.delTopicConfig(addTopicConfigVo);
        if(i>0){
            fb.setCode(1);
            fb.setMessage("删除成功");
        }else{
            fb.setCode(0);
            fb.setMessage("删除失败");
        }
        return gson.toJson(fb);
    }

//    @RequestMapping(value = "/updateTopicConfig",method = RequestMethod.POST)
//    public String updateTopicConfig(@RequestBody TopicConfigVo topicConfigVo,HttpServletRequest request){
//        String token = request.getHeader("Account_token");
//        String datum = tokenRedisService.get("TOKEN_" +token);
//        if(datum == null){
//            fb.setCode(0);
//            fb.setMessage("对不起您未登录");
//            return gson.toJson(fb);
//        }
//        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
//        TopicConfig topicConfig = new TopicConfig();
//        int i = topicConfigService.updateTopicConfig(topicConfig);
//        if(i == 0){
//            fb.setCode(0);
//            fb.setMessage("修改失败");
//        }else{
//            fb.setCode(1);
//            fb.setMessage("修改成功");
//        }
//        return gson.toJson(fb);
//    }

}
