package com.example.tdl.web;

import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.AddTopicVo;
import com.example.tdl.domain.vo.DelTopicVo;
import com.example.tdl.domain.vo.GatewaySNAndTypeVo;
import com.example.tdl.domain.vo.UpdateTopicVo;
import com.example.tdl.service.GatewayService;

import com.example.tdl.service.TopicService;
import com.example.tdl.service.redis.TokenRedisService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.Map;

@RestController
@RequestMapping(path = "/topic")
public class TopicController {
    private Logger logger = LoggerFactory.getLogger(this.getClass());

    private CommFeedback fb = new CommFeedback();

    private Gson gson = new Gson();

    @Autowired
    private TopicService topicService;

    @Autowired
    private GatewayService gatewayService;

    @Autowired
    private TokenRedisService tokenRedisService;

    //获取设备的所有主题
    @ApiOperation(value = "根据设备查询主题",notes = "根据设备查询主题：" +
        "   sn:设备的SN，" +
        "   type:设备类型")
    @RequestMapping(value = "/getAllByDevice",method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public Object getAllByDevice(@RequestBody GatewaySNAndTypeVo gatewaySNAndTypeVo){
        if(StringUtils.isEmpty(gatewaySNAndTypeVo.getSN())){
            fb.setCode(0);
            fb.setMessage("查询的网关SN不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(gatewaySNAndTypeVo.getType())){
            fb.setCode(0);
            fb.setMessage("查询的网关类型不能为空");
            return gson.toJson(fb);
        }
        return topicService.getByGateway(gatewaySNAndTypeVo);
    }

    @ApiOperation(value = "添加主题",notes = "添加所需要传递的参数：" +
        "   SN:需要添加的网关SN，" +
        "   gType:需要添加的网关类型，" +
        "   topicName:添加的主题名，" +
        "   type:添加的主题类型，" +
        "   intervalTime:添加的主题间隔时间，" +
        "   frequency:添加的主题发送次数")
    @RequestMapping(value="/addTopic",method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public  String addTopic(@RequestBody AddTopicVo addTopicVo){
        if (StringUtils.isEmpty(addTopicVo.getSN())){
            fb.setCode(0);
            fb.setMessage("需要添加主题的网关SN不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addTopicVo.getgType())){
            fb.setCode(0);
            fb.setMessage("需要添加主题的网关类型不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addTopicVo.getTopicName())){
            fb.setCode(0);
            fb.setMessage("添加的主题名不能为空");
            return gson.toJson(fb);
        }
        if (addTopicVo.getType()==null){
            fb.setCode(0);
            fb.setMessage("添加的主题类型不能为空");
            return gson.toJson(fb);
        }
        if (addTopicVo.getIntervalTime()==null){
            fb.setCode(0);
            fb.setMessage("需要添加的主题间隔时间不能为空");
            return gson.toJson(fb);
        }
        if (addTopicVo.getFrequency()==null){
            fb.setCode(0);
            fb.setMessage("需要添加的主题发送次数不能为空");
            return gson.toJson(fb);
        }
        if (gatewayService.getBySNAndType(addTopicVo.getSN(),addTopicVo.getgType())==null){
            fb.setCode(0);
            fb.setMessage("该网关不存在");
            return gson.toJson(fb);
        }
        if(topicService.getByTopicName(addTopicVo.getSN(),addTopicVo.getgType(),addTopicVo.getTopicName()) != null ){
            fb.setCode(0);
            fb.setMessage("该主题名已存在");
        }else{
            if (addTopicVo.getIntervalTime()>=100){
                Map<String,Object> map=new HashMap<>();
                map.put("SN",addTopicVo.getSN());
                map.put("gType",addTopicVo.getgType());
                map.put("topicName",addTopicVo.getTopicName());
                map.put("type",addTopicVo.getType());
                map.put("intervalTime",addTopicVo.getIntervalTime());
                map.put("frequency",addTopicVo.getFrequency());
                topicService.addTopic(map);
                Map<String,Object> msg=new HashMap<>();
                msg.put("msg",map.get("msg"));
                if (msg.get("msg").equals("添加成功")){
                    fb.setCode(1);
                    fb.setMessage("添加成功");
                }else{
                    fb.setCode(0);
                    fb.setMessage(msg.get("msg").toString());
                }
            }else{
                fb.setCode(0);
                fb.setMessage("间隔时间不能低于100毫秒");
            }
        }
        return gson.toJson(fb);
    }

    @ApiOperation(value = "修改主题",notes = "修改所需要传递的参数：" +
        "   topicName:修改后的主题名，" +
        "   type:修改的主题类型，" +
        "   intervalTime:修改的主题间隔时间，" +
        "   frequency:修改的主题发送次数，" +
        "   oldTopicName:需要修改的主题名，" +
        "   SN:需要修改的网关SN，" +
        "   gType:需要修改的网关类型")
    @RequestMapping(value = "/updateTopic",method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public  String updateTopic(@RequestBody UpdateTopicVo updateTopicVo){

        if (StringUtils.isEmpty(updateTopicVo.getTopicName())){
            fb.setCode(0);
            fb.setMessage("修改后的主题名不能为空");
            return gson.toJson(fb);
        }
        if (updateTopicVo.getType()==null){
            fb.setCode(0);
            fb.setMessage("修改的主题类型不能为空");
        }
        if (updateTopicVo.getIntervalTime()==null){
            fb.setCode(0);
            fb.setMessage("修改的主题间隔时间不能为空");
            return gson.toJson(fb);
        }
        if (updateTopicVo.getFrequency()==null){
            fb.setCode(0);
            fb.setMessage("修改的主题发送次数不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateTopicVo.getOldTopicName())){
            fb.setCode(0);
            fb.setMessage("需要修改的主题名不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateTopicVo.getSN())){
            fb.setCode(0);
            fb.setMessage("需要修改的网关SN不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateTopicVo.getgType())){
            fb.setCode(0);
            fb.setMessage("需要修改的网关类型不能为空");
            return gson.toJson(fb);
        }
        if (topicService.getByTopicName(updateTopicVo.getSN(),updateTopicVo.getgType(),updateTopicVo.getTopicName())!=null&&!updateTopicVo.getTopicName().equals(updateTopicVo.getOldTopicName())){
            fb.setCode(0);
            fb.setMessage("修改的主题已存在");
            return gson.toJson(fb);
        }
        if (updateTopicVo.getIntervalTime()>=100){
            int i = topicService.updateTopic(updateTopicVo);
            if(i >= 0 ){
                fb.setCode(1);
                fb.setMessage("修改主题成功");
            }else{
                fb.setCode(0);
                fb.setMessage("修改失败");
            }
        }else{
            fb.setCode(0);
            fb.setMessage("间隔时间不能低于100毫秒");
        }
        return gson.toJson(fb);
    }

    //删除
    @ApiOperation(value = "删除主题",notes = "删除所需要传递的参数：" +
        "   topicName:主题名，" +
        "   SN:删除的网关SN，" +
        "   type:需要删除的网关类型")
    @RequestMapping(value = "/delTopic",method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public String delTopic(@RequestBody DelTopicVo delTopicVo){
        if (StringUtils.isEmpty(delTopicVo.getTopicName())){
            fb.setCode(0);
            fb.setMessage("删除的主题名不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(delTopicVo.getSN())){
            fb.setCode(0);
            fb.setMessage("删除的网关SN不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(delTopicVo.getType())){
            fb.setCode(0);
            fb.setMessage("需要删除的网关类型不能为空");
            return gson.toJson(fb);
        }
        Map<String,Object> map=new HashMap<>();
        map.put("SN",delTopicVo.getSN());
        map.put("type",delTopicVo.getType());
        map.put("topicName",delTopicVo.getTopicName());
        topicService.delTopic(map);
        Map<String,Object> msg=new HashMap<>();
        msg.put("msg",map.get("msg"));
        if (msg.get("msg").equals("删除主题成功")){
            fb.setCode(1);
            fb.setMessage(msg.get("msg").toString());
        }else{
            fb.setCode(0);
            fb.setMessage(msg.get("msg").toString());
        }

        return gson.toJson(fb);
    }
}
