package com.example.tdl.web;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.AddTopicModelInfoVo;
import com.example.tdl.domain.vo.DelTopicModelInfoVo;
import com.example.tdl.domain.vo.UpdateTopicModelInfoVo;

import com.example.tdl.service.TopicModelInfoService;
import com.example.tdl.service.TopicModelService;
import com.example.tdl.service.redis.TokenRedisService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.Map;

@RestController
@RequestMapping(path = "/topicModelInfo")
public class TopicModelInfoController {

    private Logger logger = LoggerFactory.getLogger(this.getClass());

    private CommFeedback fb = new CommFeedback();

    private Gson gson = new Gson();

    @Autowired
    private TopicModelInfoService topicModelInfoService;

    @Autowired
    private TopicModelService topicModelService;


    @Autowired
    private TokenRedisService tokenRedisService;

    //获取设备的所有主题
    @ApiOperation(value = "查询某个主题模板下的主题配置信息",notes = "查询某个主题模板下的主题配置信息的返回值对应的意义:" +
        "      topicModelName: 主题模板名," +
        "      topicName: 主题名," +
        "      type: 主题类型," +
        "      intervalTime: 间隔时间," +
        "      frequency:发送次数")
    @RequestMapping(value = "/getAll",method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public Object getAll(@RequestBody String topicModelName){
        JSONObject jsonObject= JSON.parseObject(topicModelName);
        topicModelName=(String)jsonObject.get("topicModelName");
        if (StringUtils.isEmpty(topicModelName)){
            fb.setCode(0);
            fb.setMessage("查询的主题模板名不能为空");
            return gson.toJson(fb);
        }
        return topicModelInfoService.getAll(topicModelName);

    }

    @ApiOperation(value = "添加某个主题模板下的主题配置信息",notes = "添加某个主题模板下的主题配置信息，传递参数:" +
        "      topicModelName: 主题模板名," +
        "      topicName: 主题名," +
        "      type: 主题类型," +
        "      intervalTime: 间隔时间," +
        "      frequency:发送次数")
    @RequestMapping(value="/addTopicModelInfo",method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public String addTopicModelInfo(@RequestBody AddTopicModelInfoVo topicModelInfoVo){
        if (StringUtils.isEmpty(topicModelInfoVo.getTopicModelName())){
            fb.setCode(0);
            fb.setMessage("添加的主题模板名不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(topicModelInfoVo.getTopicName())){
            fb.setCode(0);
            fb.setMessage("添加的主题名不能为空");
            return gson.toJson(fb);
        }
        if (topicModelInfoVo.getType() == null){
            fb.setCode(0);
            fb.setMessage("添加的主题类型不能为空");
            return gson.toJson(fb);
        }
        if (topicModelInfoVo.getIntervalTime() == null){
            fb.setCode(0);
            fb.setMessage("添加的间隔时间不能为空");
            return gson.toJson(fb);
        }
        if (topicModelInfoVo.getIntervalTime()<100){
            fb.setCode(0);
            fb.setMessage("添加的间隔时间不能小于100");
            return gson.toJson(fb);
        }
        if (topicModelInfoService.getByTopicModelNameAndTopicName(topicModelInfoVo.getTopicModelName(),topicModelInfoVo.getTopicName())!=null){
            fb.setCode(0);
            fb.setMessage("添加模板的主题名不能相同");
            return gson.toJson(fb);
        }
        if (topicModelService.getByTopicModelName(topicModelInfoVo.getTopicModelName())==null){
            fb.setCode(0);
            fb.setMessage("没有该主题模板");
            return gson.toJson(fb);
        }
        int a=topicModelInfoService.addTopicModelInfo(topicModelInfoVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage("添加模板主题成功");
        }else{
            fb.setCode(0);
            fb.setMessage("添加模板主题失败");
        }
        return gson.toJson(fb);
    }

    @ApiOperation(value = "修改某个主题模板下的主题配置信息",notes = "修改某个主题模板下的主题配置信息，传递参数:" +
        "      topicModelName: 修改的主题模板名," +
        "      oldTopicName: 修改之前的主题名," +
        "      topicName: 修改以后的主题名," +
        "      type: 主题类型," +
        "      intervalTime: 间隔时间," +
        "      frequency:发送次数")
    @RequestMapping(value = "/updateTopicModelInfo",method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public  String updateTopicModelInfo(@RequestBody UpdateTopicModelInfoVo updateTopicModelInfoVo){
        if (StringUtils.isEmpty(updateTopicModelInfoVo.getTopicModelName())){
            fb.setCode(0);
            fb.setMessage("需要修改的主题模板名不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateTopicModelInfoVo.getOldTopicName())){
            fb.setCode(0);
            fb.setMessage("需要修改的主题名不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateTopicModelInfoVo.getTopicName())){
            fb.setCode(0);
            fb.setMessage("修改的主题名不能为空");
            return gson.toJson(fb);
        }
        if (updateTopicModelInfoVo.getType()== null){
            fb.setCode(0);
            fb.setMessage("修改的主题类型不能为空");
            return gson.toJson(fb);
        }
        if (updateTopicModelInfoVo.getIntervalTime()== null){
            fb.setCode(0);
            fb.setMessage("修改的间隔时间不能为空");
            return gson.toJson(fb);
        }
        if (updateTopicModelInfoVo.getIntervalTime()<100){
            fb.setCode(0);
            fb.setMessage("修改的间隔时间不能小于100");
            return gson.toJson(fb);
        }
        if (topicModelInfoService.getByTopicModelNameAndTopicName(updateTopicModelInfoVo.getTopicModelName(),updateTopicModelInfoVo.getTopicName())!=null&&!updateTopicModelInfoVo.getOldTopicName().equals(updateTopicModelInfoVo.getTopicName())){
            fb.setCode(0);
            fb.setMessage("修改的主题名不能相同");
            return gson.toJson(fb);
        }
        if (topicModelService.getByTopicModelName(updateTopicModelInfoVo.getTopicModelName())==null){
            fb.setCode(0);
            fb.setMessage("没有该主题模板");
            return gson.toJson(fb);
        }
        int a=topicModelInfoService.updateTopicModelInfo(updateTopicModelInfoVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage("修改主题模板信息成功");
        }else{
            fb.setCode(0);
            fb.setMessage("修改主题模板失败");
        }


        return gson.toJson(fb);
    }

    //删除
    @ApiOperation(value = "删除某个主题模板下的主题配置信息",notes = "删除某个主题模板下的主题配置信息，传递参数:" +
        "      topicModelName: 需要删除的主题模板名," +
        "      topicName: 需要删除的主题名")
    @RequestMapping(value = "/delTopicModel",method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public String delTopicModel(@RequestBody DelTopicModelInfoVo delTopicModelInfoVo){
        if (StringUtils.isEmpty(delTopicModelInfoVo.getTopicModelName())){
            fb.setCode(0);
            fb.setMessage("删除的主题模板名不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(delTopicModelInfoVo.getTopicName())){
            fb.setCode(0);
            fb.setMessage("删除的主题名不能为空");
            return gson.toJson(fb);
        }
        if (topicModelService.getByTopicModelName(delTopicModelInfoVo.getTopicModelName())==null){
            fb.setCode(0);
            fb.setMessage("没有该主题模板");
            return gson.toJson(fb);
        }
        try {
            Map<String,Object> map=new HashMap<>();
            map.put("topicModelName",delTopicModelInfoVo.getTopicModelName());
            map.put("topicName",delTopicModelInfoVo.getTopicName());
            topicModelInfoService.delTopicModelInfo(map);
            Map<String,Object> msg=new HashMap<>();
            msg.put("msg",map.get("msg"));
            if(msg.get("msg").equals("删除成功")){
                fb.setCode(1);
                fb.setMessage("删除成功");
            }else{
                fb.setCode(0);
                fb.setMessage(msg.get("msg").toString());
            }
        }catch (Exception e){
            logger.info(e.toString());
            e.printStackTrace();
            fb.setCode(0);
            fb.setMessage("删除失败");
        }
        return gson.toJson(fb);
    }
}
