package com.example.tdl.web;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.UpdateTransferModelConfigVo;
import com.example.tdl.service.TransferModelConfigService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.Map;


@RestController
@RequestMapping(path = "/transferModelConfig")
public class TransferModelConfigController {
    private Logger logger = LoggerFactory.getLogger(this.getClass());

    private CommFeedback fb = new CommFeedback();

    private Gson gson = new Gson();

    @Autowired
    private TransferModelConfigService transferModelConfigService;


    @ApiOperation(value = "查询线路配置模板",notes = "查询线路配置模板的返回值对应的意义:" +
        "      transferNo：线路模板编号" +
        "      transferModelName: 线路途径地模板名," +
        "      country: 国家，" +
        "      city：城市，" +
        "      addressDetail：详细地址，" +
        "      lng：经度，" +
        "      lat：纬度，" +
        "      postCode：邮编，" +
        "      sequence：顺序，" +
        "      remark：备注。")
    @RequestMapping(value = "/getAll",method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public Object getAll(@RequestBody String transferNo){
        JSONObject jsonObject= JSON.parseObject(transferNo);
        transferNo=(String)jsonObject.get("transferNo");
        return transferModelConfigService.getAll(transferNo);
    }

    @ApiOperation(value = "修改线路配置模板",notes = "修改线路配置模板，传递的参数:" +
            "      transferNo：线路模板编号," +
            "      info{" +
            "           country: 国家，" +
            "           city：城市，" +
            "           addressDetail：详细地址，" +
            "           lng：经度，" +
            "           lat：纬度，" +
            "           postCode：邮编，" +
            "           remark：备注" +
            "       }")
    @RequestMapping(value = "/updateTransferModelConfig",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public Object updateTransferModelConfig(@RequestBody UpdateTransferModelConfigVo updateTransferModelConfigVo) {
        if (StringUtils.isEmpty(updateTransferModelConfigVo.getTransferNo())) {
            fb.setCode(0);
            fb.setMessage("需要修改的线路模板编号不能为空");
            return gson.toJson(fb);
        }
        if (updateTransferModelConfigVo.getInfo().size() >= 2 && updateTransferModelConfigVo.getInfo().size() < 5) {
            String country = "";
            String city = "";
            String addressDetail = "";
            String lng = "";
            String lat = "";
            String postCode = "";
            String remark = "";
            for (int i = 0; i < updateTransferModelConfigVo.getInfo().size(); i++) {
                if (StringUtils.isEmpty(updateTransferModelConfigVo.getInfo().get(i).getCountry())) {
                    fb.setCode(0);
                    fb.setMessage("国家不能为空");
                    return gson.toJson(fb);
                }
                if (StringUtils.isEmpty(updateTransferModelConfigVo.getInfo().get(i).getCity())) {
                    fb.setCode(0);
                    fb.setMessage("城市不能为空");
                    return gson.toJson(fb);
                }
                if (StringUtils.isEmpty(updateTransferModelConfigVo.getInfo().get(i).getAddressDetail())) {
                    fb.setCode(0);
                    fb.setMessage("具体地址不能为空");
                    return gson.toJson(fb);
                }
                if (StringUtils.isEmpty(updateTransferModelConfigVo.getInfo().get(i).getLat())) {
                    fb.setCode(0);
                    fb.setMessage("纬度不能为空");
                    return gson.toJson(fb);
                }
                if (!updateTransferModelConfigVo.getInfo().get(i).getLat().matches("^-?(([1-9])((\\.\\d{1,6})?)|([1-8]\\d)((\\.\\d{1,6})?)|90)$")) {
                    fb.setCode(0);
                    fb.setMessage("纬度的范围为-90~90");
                    return gson.toJson(fb);
                }
                if (StringUtils.isEmpty(updateTransferModelConfigVo.getInfo().get(i).getLng())) {
                    fb.setCode(0);
                    fb.setMessage("经度不能为空");
                    return gson.toJson(fb);
                }
                if (!updateTransferModelConfigVo.getInfo().get(i).getLng().matches("^-?(([1-9]\\d?)((\\.\\d{1,6})?)|(1[1-7]\\d)((\\.\\d{1,6})?)|180)$")) {
                    fb.setCode(0);
                    fb.setMessage("经度的范围为-180~180");
                    return gson.toJson(fb);
                }
                if (StringUtils.isEmpty(updateTransferModelConfigVo.getInfo().get(i).getPostCode())) {
                    fb.setCode(0);
                    fb.setMessage("邮编不能为空");
                    return gson.toJson(fb);
                }
                country = country + updateTransferModelConfigVo.getInfo().get(i).getCountry() + " ";
                city = city + updateTransferModelConfigVo.getInfo().get(i).getCity() + " ";
                addressDetail = addressDetail + updateTransferModelConfigVo.getInfo().get(i).getAddressDetail() + " ";
                lng = lng + updateTransferModelConfigVo.getInfo().get(i).getLng() + " ";
                lat = lat + updateTransferModelConfigVo.getInfo().get(i).getLat() + " ";
                postCode = postCode + updateTransferModelConfigVo.getInfo().get(i).getPostCode() + " ";
                remark = remark + updateTransferModelConfigVo.getInfo().get(i).getRemark() + " ";
            }
            Map<Object,Object> map=new HashMap<>();
            map.put("transferNo",updateTransferModelConfigVo.getTransferNo());
            map.put("counts",updateTransferModelConfigVo.getInfo().size());
            map.put("country",country);
            map.put("city",city);
            map.put("addressDetail",addressDetail);
            map.put("lng",lng);
            map.put("lat",lat);
            map.put("postCode",postCode);
            map.put("remark",remark);
            transferModelConfigService.updateTransferModelConfig(map);
            Map<String,Object> msg=new HashMap<>();
            msg.put("msg",map.get("msg"));
            if (msg.get("msg").equals("修改线路配置模板成功")){
                fb.setCode(1);
                fb.setMessage("修改线路配置模板成功");
            }else{
                fb.setCode(0);
                fb.setMessage(msg.get("msg").toString());
            }
        }
        return gson.toJson(fb);
    }
}
