package com.example.tdl.web;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.AddModelVo;
import com.example.tdl.domain.vo.AddTransferModelVo;
import com.example.tdl.domain.vo.UpdateModelVo;
import com.example.tdl.domain.vo.UpdateTransferModelVo;
import com.example.tdl.service.ProbesModelService;
import com.example.tdl.service.TopicModelService;
import com.example.tdl.service.TransferModelService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

@RestController
@RequestMapping(path = "/transferModel")
public class TransferModelController {
    private Logger logger = LoggerFactory.getLogger(this.getClass());

    private CommFeedback fb = new CommFeedback();

    private Gson gson = new Gson();

    @Autowired
    private TransferModelService transferModelService;


    @ApiOperation(value = "查询线路模板",notes = "查询线路模板的返回值对应的意义:" +
        "      transferNo: 线路途径地模板编号," +
        "      transferModelName: 线路途径地模板名," +
        "      counts: 途径地个数")
    @RequestMapping(value = "/getAll",method = RequestMethod.GET)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public Object getAll(){
        return transferModelService.getAll();
    }

    @ApiOperation(value = "添加模板",notes = "添加模板的同时添加通道配置信息，传递的参数:" +
        "      transferModelName: 添加的模板名," +
        "      info:{" +
        "           country:国家，" +
        "           city:城市，" +
        "           addressDetail:具体地址，" +
        "           lng:经度，" +
        "           lat:纬度，" +
        "           postCode:邮编，" +
        "           remark:备注" +
        "       }")
    @RequestMapping(value="/addTransferModel",method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public String addTransferModel(@RequestBody AddTransferModelVo addTransferModelVo){
        if (StringUtils.isEmpty(addTransferModelVo.getTransferModelName())){
            fb.setCode(0);
            fb.setMessage("添加的线路模板名不能为空");
            return gson.toJson(fb);
        }
        if (transferModelService.getByTransferModelName(addTransferModelVo.getTransferModelName())!=null){
            fb.setCode(0);
            fb.setMessage("添加的线路模板名不能重复");
            return gson.toJson(fb);
        }
        if (addTransferModelVo.getInfo().size()>=2&&addTransferModelVo.getInfo().size()<5){
            String country="";
            String city="";
            String addressDetail="";
            String lng="";
            String lat = "";
            String postCode = "";
            String remark="";
            for(int i =0;i<addTransferModelVo.getInfo().size();i++){
                if(StringUtils.isEmpty(addTransferModelVo.getInfo().get(i).getCountry())){
                    fb.setCode(0);
                    fb.setMessage("国家不能为空");
                    return gson.toJson(fb);
                }
                if (StringUtils.isEmpty(addTransferModelVo.getInfo().get(i).getCity())){
                    fb.setCode(0);
                    fb.setMessage("城市不能为空");
                    return gson.toJson(fb);
                }
                if (StringUtils.isEmpty(addTransferModelVo.getInfo().get(i).getAddressDetail())){
                    fb.setCode(0);
                    fb.setMessage("具体地址不能为空");
                    return gson.toJson(fb);
                }
                if (StringUtils.isEmpty(addTransferModelVo.getInfo().get(i).getLat())){
                    fb.setCode(0);
                    fb.setMessage("纬度不能为空");
                    return gson.toJson(fb);
                }
                if(!addTransferModelVo.getInfo().get(i).getLat().matches("^-?(([1-9])((\\.\\d{1,6})?)|([1-8]\\d)((\\.\\d{1,6})?)|90)$")){
                    fb.setCode(0);
                    fb.setMessage("纬度的范围为-90~90");
                    return gson.toJson(fb);
                }
                if (StringUtils.isEmpty(addTransferModelVo.getInfo().get(i).getLng())){
                    fb.setCode(0);
                    fb.setMessage("经度不能为空");
                    return gson.toJson(fb);
                }
                if(!addTransferModelVo.getInfo().get(i).getLng().matches("^-?(([1-9]\\d?)((\\.\\d{1,6})?)|(1[1-7]\\d)((\\.\\d{1,6})?)|180)$")) {
                    fb.setCode(0);
                    fb.setMessage("经度的范围为-180~180");
                    return gson.toJson(fb);
                }
                if (StringUtils.isEmpty(addTransferModelVo.getInfo().get(i).getPostCode())){
                    fb.setCode(0);
                    fb.setMessage("邮编不能为空");
                    return gson.toJson(fb);
                }
                country=country +addTransferModelVo.getInfo().get(i).getCountry() +" ";
                city=city +addTransferModelVo.getInfo().get(i).getCity() +" ";
                addressDetail =addressDetail +addTransferModelVo.getInfo().get(i).getAddressDetail() +" ";
                lng = lng + addTransferModelVo.getInfo().get(i).getLng() +" ";
                lat = lat + addTransferModelVo.getInfo().get(i).getLat() +" ";
                postCode=postCode + addTransferModelVo.getInfo().get(i).getPostCode() +" ";
                remark=remark+addTransferModelVo.getInfo().get(i).getRemark()+" ";
            }
            Date day=new Date();
            SimpleDateFormat df = new SimpleDateFormat("yyyyMMdd");
            String transferNo="TM"+df.format(day);
            String transferNos=transferNo+((int)(Math.random()*(9999-1000+1))+1000);
            if (transferModelService.getByTransferNo(transferNos)!=null) {
                while (transferModelService.getByTransferNo(transferNos) != null) {
                    transferNos = transferNo + ((int) (Math.random() * (9999 - 1000 + 1)) + 1000);
                }
            }

            Map<String,Object> map=new HashMap<>();
            map.put("transferNo",transferNos);
            map.put("transferModelName",addTransferModelVo.getTransferModelName());
            map.put("counts",addTransferModelVo.getInfo().size());
            map.put("country",country);
            map.put("city",city);
            map.put("addressDetail",addressDetail);
            map.put("lng",lng);
            map.put("lat",lat);
            map.put("postCode",postCode);
            map.put("remark",remark);
            transferModelService.addTransferModel(map);
            Map<String,Object> msg=new HashMap<>();
            msg.put("msg",map.get("msg"));
            if (msg.get("msg").equals("添加线路模板成功")){
                fb.setCode(1);
                fb.setMessage("添加线路模板成功");
            }else{
                fb.setCode(0);
                fb.setMessage(msg.get("msg").toString());
            }
        }else{
            fb.setCode(0);
            fb.setMessage("添加的线路模板途径城市个数在2-4之间");
            return gson.toJson(fb);
        }
        return gson.toJson(fb);
    }

    @ApiOperation(value = "修改主题模板",notes = "修改模板只能修改模板名，updateTime不用传，传递的参数:" +
        "      transferNo: 需要修改的线路模板编号," +
        "      transferModelName: 修改后的线路模板名")
    @RequestMapping(value = "/updateModel",method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public  String updateModel(@RequestBody UpdateTransferModelVo updateTransferModelVo){
        if (StringUtils.isEmpty(updateTransferModelVo.getTransferNo())){
            fb.setCode(0);
            fb.setMessage("需要修改的线路模板编号不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateTransferModelVo.getTransferModelName())){
            fb.setCode(0);
            fb.setMessage("修改后的线路模板名不能为空");
            return gson.toJson(fb);
        }
        int a=transferModelService.updateTransferModel(updateTransferModelVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage("修改模板成功");
        }else{
            fb.setCode(0);
            fb.setMessage("修改模板失败");
        }
        return gson.toJson(fb);
    }

    //删除
    @ApiOperation(value = "删除线路模板",notes = "删除线路模板需要传的参数:" +
        "      transferNo: 线路模板编号")
    @RequestMapping(value = "/delModel",method = RequestMethod.POST)
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public String delModel(@RequestBody String transferNo){
        JSONObject jsonObject= JSON.parseObject(transferNo);
        transferNo=(String)jsonObject.get("transferNo");
        if (StringUtils.isEmpty(transferNo)){
            fb.setCode(0);
            fb.setMessage("删除的线路模板名不能为空");
            return gson.toJson(fb);
        }
        Map<String,Object> map=new HashMap<>();
        map.put("transferNo",transferNo);
        transferModelService.delTransferModel(map);
        Map<String,Object> msg=new HashMap<>();
        msg.put("msg",map.get("msg"));
        if (msg.get("msg").equals("删除线路模板成功")){
            fb.setCode(1);
            fb.setMessage("删除线路模板成功");
        }else{
            fb.setCode(0);
            fb.setMessage(msg.get("msg").toString());
        }
        return gson.toJson(fb);
    }
}
