package com.example.tdl.web;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.*;
import com.example.tdl.service.RoleHasPermissionService;
import com.example.tdl.service.RoleService;

import com.example.tdl.service.UserService;
import com.example.tdl.service.redis.TokenRedisService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;


@RestController
@RequestMapping("/user")
public class UserController {

    private Logger logger = LoggerFactory.getLogger(UserController.class);

    private CommFeedback fb = new CommFeedback();

    private Gson gson = new Gson();

    @Autowired
    private UserService userService;

    @Autowired
    private RoleHasPermissionService roleHasPermissionService;

    @Autowired
    private TokenRedisService tokenRedisService;

    @Autowired
    private RoleService roleService;


    //获取所有用户信息
    @ApiOperation(value = "获取所有用户信息",notes = "获取所有用户信息")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/getAll",method = RequestMethod.GET)
    public Object getAll(){
        return userService.getAll();
    }

    //根据条件查询
    @ApiOperation(value = "根据条件查询",notes = "根据条件查询")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/getByTerm",method = RequestMethod.POST)
    public Object getByTerm(@RequestBody UserTermVo userTermVo,HttpServletRequest request){
        return userService.getByTerm(userTermVo);
    }

    //新增用户
    @ApiOperation(value = "新增用户",notes = "新增用户时，userNumber,createTime,updateTime不用传。传递参数:" +
            "   userName：用户名，" +
            "   password：密码，" +
            "   nickName：昵称，" +
            "   phone：电话，" +
            "   email：邮件，" +
            "   roleName：角色名")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/addUser",method = RequestMethod.POST)
    public Object addUser(@RequestBody AddUserVo userVo){
        if (StringUtils.isEmpty(userVo.getUserName())){
            fb.setCode(0);
            fb.setMessage("用户名不能为空");
            return gson.toJson(fb);
        }
        if (!userVo.getUserName().matches("^[a-zA-Z][a-zA-Z0-9_]{5,19}$")) {
            fb.setCode(0);
            fb.setMessage("用户名以字母开头,可包含数字和_,6-20个字符");
            return gson.toJson(fb);
        }
        if(userService.getByUserName(userVo.getUserName()) !=null){
            fb.setCode(0);
            fb.setMessage("用户名已被占用");
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(userVo.getPassword())){
            fb.setCode(0);
            fb.setMessage("密码不能为空");
            return gson.toJson(fb);
        }
        if(!userVo.getPassword().matches("^[a-zA-Z\\d_]{6,20}$")){
            fb.setCode(0);
            fb.setMessage("密码必须是6-20位的数字、字母和_");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(userVo.getNickName())){
            fb.setCode(0);
            fb.setMessage("昵称不能为空");
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(userVo.getPhone())){
            fb.setCode(0);
            fb.setMessage("电话不能为空");
            return gson.toJson(fb);
        }
        if( !userVo.getPhone().matches("^1[34578]\\d{9}$")){
            fb.setCode(0);
            fb.setMessage("手机号不规范");
            return gson.toJson(fb);
        }
        if ((userService.getByPhone(userVo.getPhone()))!=null){
            fb.setCode(0);
            fb.setMessage("该手机号已被注册");
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(userVo.getEmail())){
            fb.setCode(0);
            fb.setMessage("邮箱不能为空");
            return gson.toJson(fb);
        }
        if( !userVo.getEmail().matches("\\w+(\\.\\w*)*@\\w+(\\.\\w{2,3}){1,3}")){
            fb.setCode(0);
            fb.setMessage("邮箱格式不规范");
            return gson.toJson(fb);
        }
        if ((userService.getByEmail(userVo.getEmail()))!=null){
            fb.setCode(0);
            fb.setMessage("该邮箱已被注册");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(userVo.getRoleName())){
            fb.setCode(0);
            fb.setMessage("角色名不能为空");
            return gson.toJson(fb);
        }
        if (roleService.getByRoleName(userVo.getRoleName())==0){
            fb.setCode(0);
            fb.setMessage("没有该角色");
            return gson.toJson(fb);
        }
        String userNumber="U"+System.currentTimeMillis();
        userVo.setUserNumber(userNumber);
        userVo.setCreateTime(System.currentTimeMillis());
        userVo.setUpdateTime(System.currentTimeMillis());
        int a=userService.addUser(userVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage("添加用户成功");
        }else{
            fb.setCode(0);
            fb.setMessage("添加用户失败");
        }
        return gson.toJson(fb);
    }

    //修改用户
    @ApiOperation(value = "修改用户",notes = "修改用户都要传" +
            "   userNumber：用户编号，" +
            "   userName:用户名," +
            "   nickName：昵称，" +
            "   phone：电话，" +
            "   email：邮件，" +
            "   roleName：角色名")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/updateUser",method = RequestMethod.POST)
    public Object updateUser(@RequestBody UpdateUserVo userVo){
        ResultUserVo userInfo=userService.getByUserNumber(userVo.getUserNumber());
        if (StringUtils.isEmpty(userVo.getNickName())){
            fb.setCode(0);
            fb.setMessage("昵称不能为空");
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(userVo.getPhone())){
            fb.setCode(0);
            fb.setMessage("电话不能为空");
            return gson.toJson(fb);
        }
        if( !userVo.getPhone().matches("^1[34578]\\d{9}$")){
            fb.setCode(0);
            fb.setMessage("手机号不规范");
            return gson.toJson(fb);
        }
        if (userInfo!=null&&userInfo.getPhone().equals(userVo.getPhone())){
            fb.setCode(0);
            fb.setMessage("该手机号已被注册");
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(userVo.getEmail())){
            fb.setCode(0);
            fb.setMessage("邮箱不能为空");
            return gson.toJson(fb);
        }
        if( !userVo.getEmail().matches("\\w+(\\.\\w*)*@\\w+(\\.\\w{2,3}){1,3}")){
            fb.setCode(0);
            fb.setMessage("邮箱格式不规范");
            return gson.toJson(fb);
        }
        if (userInfo!=null&&userInfo.getEmail().equals(userVo.getEmail())){
            fb.setCode(0);
            fb.setMessage("该邮箱已被注册");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(userVo.getRoleName())){
            fb.setCode(0);
            fb.setMessage("角色名不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(userVo.getUserNumber())){
            fb.setCode(0);
            fb.setMessage("需要修改的用户编号不能为空");
            return gson.toJson(fb);
        }
        if (roleService.getByRoleName(userVo.getRoleName())==0){
            fb.setCode(0);
            fb.setMessage("修改的角色名不存在");
            return gson.toJson(fb);
        }
        int a=userService.updateUser(userVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage("修改用户成功");
        }else{
            fb.setCode(0);
            fb.setMessage("修改用户失败");
        }
        return gson.toJson(fb);
    }

    //删除用户
    @ApiOperation(value = "删除用户",notes = "删除用户")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/delUser",method = RequestMethod.POST)
    public Object delUser(@RequestBody String userNumber){
        JSONObject jsonObject= JSON.parseObject(userNumber);
        userNumber=jsonObject.getString("userNumber");
        if (StringUtils.isEmpty(userNumber)){
            fb.setCode(0);
            fb.setMessage("需要删除的用户编号不能为空");
            return gson.toJson(fb);
        }
        int a = userService.delUser(userNumber);
        if (a>0){
            fb.setCode(1);
            fb.setMessage("删除用户成功");
        }else{
            fb.setCode(0);
            fb.setMessage("删除用户失败");
        }
        return gson.toJson(fb);
    }

    //重置密码
    @ApiOperation(value = "重置密码",notes = "重置密码")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/resetPassword",method = RequestMethod.POST)
    public Object resetPassword(@RequestBody String userNumber){
        JSONObject jsonObject= JSON.parseObject(userNumber);
        userNumber=jsonObject.getString("userNumber");
        if (StringUtils.isEmpty(userNumber)) {
            fb.setCode(0);
            fb.setMessage("需要重置密码的用户编号不能为空");
            return gson.toJson(fb);
        }
        String pwd=DigestUtils.sha256Hex("Witium123");
        int a = userService.updatePassword(new UpdatePasswordVo(userNumber,pwd));
        if (a>0){
            fb.setCode(1);
            fb.setMessage("重置密码成功");
        }else{
            fb.setCode(0);
            fb.setMessage("重置密码失败");
        }
        return gson.toJson(fb);
    }

    //修改密码
    @ApiOperation(value = "修改密码",notes = "修改密码")
    @ApiImplicitParams({
        @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/updatePassword",method = RequestMethod.POST)
    public Object updatePassword(@RequestBody UpdatePasswordVo updatePasswordVo){
        if (StringUtils.isEmpty(updatePasswordVo.getUserNumber())) {
            fb.setCode(0);
            fb.setMessage("需要重置密码的用户编号不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updatePasswordVo.getPassword())) {
            fb.setCode(0);
            fb.setMessage("修改的密码不能为空");
            return gson.toJson(fb);
        }
        String pwd=DigestUtils.sha256Hex(updatePasswordVo.getPassword());
        updatePasswordVo.setPassword(pwd);
        int a = userService.updatePassword(updatePasswordVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage("修改密码成功");
        }else{
            fb.setCode(0);
            fb.setMessage("修改密码失败");
        }
        return gson.toJson(fb);
    }

    //获取当前的用户名
    @ApiOperation(value = "获取当前的用户名",notes = "获取当前的用户名")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @GetMapping(value="/getUserName")
    public Object getUserName(HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        return new UserNameVo(gson.fromJson(datum,UserRedisVo.class).getUserName());
    }

}
