package com.example.tdl.web;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.*;
import com.example.tdl.entity.Warehouse;
import com.example.tdl.service.RoleService;
import com.example.tdl.service.WarehouseService;
import com.example.tdl.service.redis.TokenRedisService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;

@RestController
@RequestMapping("/warehouse")
public class WarehouseController {

    private Logger logger = LoggerFactory.getLogger(WarehouseController.class);

    CommFeedback fb=new CommFeedback();

    Gson gson=new Gson();

    @Autowired
    private WarehouseService warehouseService=new WarehouseService();

    @Autowired
    private TokenRedisService tokenRedisService;

    //查询所有仓库信息
    @ApiOperation(value = "查询所有仓库信息",notes = "查询所有仓库信息,返回值说明：" +
            "   warehouseName:仓库名," +
            "   warehouseNo:仓库编号," +
            "   country:国家," +
            "   province:省," +
            "   city:市," +
            "   district:区," +
            "   address:具体地址," +
            "   regionCN:区域中文," +
            "   regionEN:区域英文," +
            "   longitude:经度," +
            "   latitude:纬度," +
            "   property:属性," +
            "   type:类型," +
            "   alarmType:预警类型," +
            "   alarmScene:预警场景," +
            "   hardwareLevel:硬件等级," +
            "   areaIndoor:室内平面面积," +
            "   areaRack:货架面积," +
            "   areaOutdoor:室外面积," +
            "   areaPlanIndoor:实际室内总面积," +
            "   office:办事处," +
            "   timeZone:时区," +
            "   weatherInfo:区域气象信息," +
            "   warehouseInfo:基础描述," +
            "   remark:备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/getAll",method = RequestMethod.GET)
    public Object getAll(HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        if(datum == null){
            fb.setCode(0);
            fb.setMessage("您没有登录，请先登录");
            return gson.toJson(fb);
        }else{
            return warehouseService.getAll();
        }
    }

    //根据条件查询仓库信息
    @ApiOperation(value = "根据条件查询仓库信息",notes = "根据条件查询仓库信息,传值说明：" +
            "   warehouseName:仓库名," +
            "   country:国家," +
            "   province:省," +
            "   city:市," +
            "   property:属性," +
            "   type:,仓库类型" +
            "   hardwareLevel:硬件等级。" +
            "返回值说明：" +
            "   warehouseName:仓库名," +
            "   warehouseNo:仓库编号," +
            "   country:国家," +
            "   province:省," +
            "   city:市," +
            "   district:区," +
            "   address:具体地址," +
            "   regionCN:区域中文," +
            "   regionEN:区域英文," +
            "   longitude:经度," +
            "   latitude:纬度," +
            "   property:属性," +
            "   type:类型," +
            "   alarmType:预警类型," +
            "   alarmScene:预警场景," +
            "   hardwareLevel:硬件等级," +
            "   areaIndoor:室内平面面积," +
            "   areaRack:货架面积," +
            "   areaOutdoor:室外面积," +
            "   areaPlanIndoor:实际室内总面积," +
            "   office:办事处," +
            "   timeZone:时区," +
            "   weatherInfo:区域气象信息," +
            "   warehouseInfo:基础描述," +
            "   remark:备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/getAllByTerm",method = RequestMethod.POST)
    public Object getAllByTerm(@RequestBody WarehouseTermVo warehouseTermVo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        if(datum == null){
            fb.setCode(0);
            fb.setMessage("您没有登录，请先登录");
            return gson.toJson(fb);
        }else{
            return warehouseService.getAllByTerm(warehouseTermVo);
        }
    }

    //添加仓库信息
    @ApiOperation(value = "添加仓库信息",notes = "添加仓库信息,传值说明：" +
            "   warehouseName:仓库名," +
            "   country:国家," +
            "   province:省," +
            "   city:市," +
            "   district:区," +
            "   address:具体地址," +
            "   regionCN:区域中文," +
            "   regionEN:区域英文," +
            "   longitude:经度," +
            "   latitude:纬度," +
            "   property:属性," +
            "   type:类型," +
            "   alarmType:预警类型," +
            "   hardwareLevel:硬件等级," +
            "   areaIndoor:室内平面面积," +
            "   areaRack:货架面积," +
            "   areaOutdoor:室外面积," +
            "   areaPlanIndoor:实际室内总面积," +
            "   office:办事处," +
            "   timeZone:时区," +
            "   weatherInfo:区域气象信息," +
            "   warehouseInfo:基础描述," +
            "   remark:备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/addWarehouse",method = RequestMethod.POST)
    public Object addWarehouse(@RequestBody AddWarehouseVo addWarehouseVo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        if(datum == null){
            fb.setCode(0);
            fb.setMessage("您没有登录，请先登录");
            return gson.toJson(fb);
        }else{
            if (StringUtils.isEmpty(addWarehouseVo.getWarehouseName())){
                fb.setCode(0);
                fb.setMessage("仓库名不能为空");
                return gson.toJson(fb);
            }
            if (StringUtils.isEmpty(addWarehouseVo.getCountry())){
                fb.setCode(0);
                fb.setMessage("国家不能为空");
                return gson.toJson(fb);
            }
            if (StringUtils.isEmpty(addWarehouseVo.getProvince())){
                fb.setCode(0);
                fb.setMessage("省不能为空");
                return gson.toJson(fb);
            }
            if (StringUtils.isEmpty(addWarehouseVo.getCity())){
                fb.setCode(0);
                fb.setMessage("市不能为空");
                return gson.toJson(fb);
            }
            if (StringUtils.isEmpty(addWarehouseVo.getDistrict())){
                fb.setCode(0);
                fb.setMessage("区不能为空");
                return gson.toJson(fb);
            }
            if (StringUtils.isEmpty(addWarehouseVo.getAddress())){
                fb.setCode(0);
                fb.setMessage("具体地址不能为空");
                return gson.toJson(fb);
            }
            if (StringUtils.isEmpty(addWarehouseVo.getRegionCN())){
                fb.setCode(0);
                fb.setMessage("区域中文不能为空");
                return gson.toJson(fb);
            }
            if (StringUtils.isEmpty(addWarehouseVo.getRegionEN())){
                fb.setCode(0);
                fb.setMessage("区域英文不能为空");
                return gson.toJson(fb);
            }
            if (StringUtils.isEmpty(addWarehouseVo.getLatitude())){
                fb.setCode(0);
                fb.setMessage("纬度不能为空");
                return gson.toJson(fb);
            }
            if (StringUtils.isEmpty(addWarehouseVo.getLongitude())){
                fb.setCode(0);
                fb.setMessage("经度不能为空");
                return gson.toJson(fb);
            }
            if(!addWarehouseVo.getLatitude().matches("^-?(([1-9])((\\.\\d{1,6})?)|([1-8]\\d)((\\.\\d{1,6})?)|90)$")){
                fb.setCode(0);
                fb.setMessage("纬度的范围为-90~90");
                return gson.toJson(fb);
            }
            if(!addWarehouseVo.getLongitude().matches("^-?(([1-9]\\d?)((\\.\\d{1,6})?)|(1[1-7]\\d)((\\.\\d{1,6})?)|180)$")) {
                fb.setCode(0);
                fb.setMessage("经度的范围为-180~180");
                return gson.toJson(fb);
            }
            if (StringUtils.isEmpty(addWarehouseVo.getProperty())){
                fb.setCode(0);
                fb.setMessage("属性不能为空");
                return gson.toJson(fb);
            }else if (!addWarehouseVo.getProperty().equals("供应中心")&&!addWarehouseVo.getProperty().equals("项目仓")&&
                    !addWarehouseVo.getProperty().equals("中心仓")&&!addWarehouseVo.getProperty().equals("X-dock")){
                fb.setCode(0);
                fb.setMessage("请输入正确的属性");
                return gson.toJson(fb);
            }
            if (StringUtils.isEmpty(addWarehouseVo.getType())){
                fb.setCode(0);
                fb.setMessage("类型不能为空");
                return gson.toJson(fb);
            }else if (!addWarehouseVo.getType().equals("平面仓")&&!addWarehouseVo.getType().equals("立体仓")){
                fb.setCode(0);
                fb.setMessage("请输入正确的仓库类型");
                return gson.toJson(fb);
            }
            if (StringUtils.isEmpty(addWarehouseVo.getAlarmType())){
                fb.setCode(0);
                fb.setMessage("预警类型不能为空");
                return gson.toJson(fb);
            }
            if (StringUtils.isEmpty(addWarehouseVo.getHardwareLevel())){
                fb.setCode(0);
                fb.setMessage("硬件等级不能为空");
                return gson.toJson(fb);
            }else if (!addWarehouseVo.getHardwareLevel().equals("好")&&!addWarehouseVo.getHardwareLevel().equals("一般")&&
                    !addWarehouseVo.getHardwareLevel().equals("较差")&&!addWarehouseVo.getHardwareLevel().equals("极差")){
                fb.setCode(0);
                fb.setMessage("请选择正确的硬件等级");
                return gson.toJson(fb);
            }
            if (addWarehouseVo.getAreaIndoor()==null){
                fb.setCode(0);
                fb.setMessage("室内平面面积不能为空");
                return gson.toJson(fb);
            }
            if (addWarehouseVo.getAreaRack()==null){
                fb.setCode(0);
                fb.setMessage("货架面积不能为空");
                return gson.toJson(fb);
            }
            if (addWarehouseVo.getAreaOutdoor()==null){
                fb.setCode(0);
                fb.setMessage("室外面积不能为空");
                return gson.toJson(fb);
            }
            if (addWarehouseVo.getAreaPlanIndoor() == null){
                fb.setCode(0);
                fb.setMessage("实际室内总面积不能为空");
                return gson.toJson(fb);
            }
            if (StringUtils.isEmpty(addWarehouseVo.getOffice())){
                fb.setCode(0);
                fb.setMessage("办事处不能为空");
                return gson.toJson(fb);
            }
            if (addWarehouseVo.getTimeZone()==null){
                fb.setCode(0);
                fb.setMessage("时区不能为空");
                return gson.toJson(fb);
            }
            if (StringUtils.isEmpty(addWarehouseVo.getWeatherInfo())){
                fb.setCode(0);
                fb.setMessage("区域气象信息不能为空");
                return gson.toJson(fb);
            }
            if (StringUtils.isEmpty(addWarehouseVo.getWarehouseInfo())){
                fb.setCode(0);
                fb.setMessage("基础描述不能为空");
                return gson.toJson(fb);
            }
            if (warehouseService.getByWarehouseName(addWarehouseVo.getWarehouseName())!=null){
                fb.setCode(0);
                fb.setMessage("仓库名不能重复");
                return gson.toJson(fb);
            }
            int a = warehouseService.addWarehouse(new Warehouse(addWarehouseVo.getWarehouseName(),String.valueOf(System.currentTimeMillis()),
                    addWarehouseVo.getCountry(),addWarehouseVo.getProvince(),addWarehouseVo.getCity(),addWarehouseVo.getDistrict(),
                    addWarehouseVo.getAddress(),addWarehouseVo.getRegionCN(),addWarehouseVo.getRegionEN(),addWarehouseVo.getLongitude(),
                    addWarehouseVo.getLatitude(),addWarehouseVo.getProperty(),addWarehouseVo.getType(),addWarehouseVo.getAlarmType(),
                    addWarehouseVo.getHardwareLevel(),addWarehouseVo.getAreaIndoor(),addWarehouseVo.getAreaRack(),addWarehouseVo.getAreaOutdoor(),
                    addWarehouseVo.getAreaPlanIndoor(),addWarehouseVo.getOffice(),addWarehouseVo.getTimeZone(),addWarehouseVo.getWeatherInfo(),
                    addWarehouseVo.getWarehouseInfo(),System.currentTimeMillis(),System.currentTimeMillis(),addWarehouseVo.getRemark()));
            if (a>0){
                fb.setCode(1);
                fb.setMessage("添加仓库信息成功");
            }else {
                fb.setCode(0);
                fb.setMessage("添加仓库信息失败");
            }
            return gson.toJson(fb);
        }
    }

    //根据仓库编号修改仓库信息
    @ApiOperation(value = "根据仓库编号修改仓库信息",notes = "根据仓库编号修改仓库信息,updatTime不用传，传值说明：" +
            "   warehouseNo:需要修改的仓库编号," +
            "   warehouseName:仓库名," +
            "   country:国家," +
            "   province:省," +
            "   city:市," +
            "   district:区," +
            "   address:具体地址," +
            "   regionCN:区域中文," +
            "   regionEN:区域英文," +
            "   longitude:经度," +
            "   latitude:纬度," +
            "   property:属性," +
            "   type:类型," +
            "   alarmType:预警类型," +
            "   hardwareLevel:硬件等级," +
            "   areaIndoor:室内平面面积," +
            "   areaRack:货架面积," +
            "   areaOutdoor:室外面积," +
            "   areaPlanIndoor:实际室内总面积," +
            "   office:办事处," +
            "   timeZone:时区," +
            "   weatherInfo:区域气象信息," +
            "   warehouseInfo:基础描述," +
            "   remark:备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/updateWarehouse",method = RequestMethod.POST)
    public Object updateWarehouse(@RequestBody UpdateWarehouseVo updateWarehouseVo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        if(datum == null){
            fb.setCode(0);
            fb.setMessage("您没有登录，请先登录");
            return gson.toJson(fb);
        }else{
            if (StringUtils.isEmpty(updateWarehouseVo.getWarehouseNo())){
                fb.setCode(0);
                fb.setMessage("需要修改的仓库编号不能为空");
            }
            if (StringUtils.isEmpty(updateWarehouseVo.getWarehouseName())){
                fb.setCode(0);
                fb.setMessage("仓库名不能为空");
                return gson.toJson(fb);
            }
            if (StringUtils.isEmpty(updateWarehouseVo.getCountry())){
                fb.setCode(0);
                fb.setMessage("国家不能为空");
                return gson.toJson(fb);
            }
            if (StringUtils.isEmpty(updateWarehouseVo.getProvince())){
                fb.setCode(0);
                fb.setMessage("省不能为空");
                return gson.toJson(fb);
            }
            if (StringUtils.isEmpty(updateWarehouseVo.getCity())){
                fb.setCode(0);
                fb.setMessage("市不能为空");
                return gson.toJson(fb);
            }
            if (StringUtils.isEmpty(updateWarehouseVo.getDistrict())){
                fb.setCode(0);
                fb.setMessage("区不能为空");
                return gson.toJson(fb);
            }
            if (StringUtils.isEmpty(updateWarehouseVo.getAddress())){
                fb.setCode(0);
                fb.setMessage("具体地址不能为空");
                return gson.toJson(fb);
            }
            if (StringUtils.isEmpty(updateWarehouseVo.getRegionCN())){
                fb.setCode(0);
                fb.setMessage("区域中文不能为空");
                return gson.toJson(fb);
            }
            if (StringUtils.isEmpty(updateWarehouseVo.getRegionEN())){
                fb.setCode(0);
                fb.setMessage("区域英文不能为空");
                return gson.toJson(fb);
            }
            if (StringUtils.isEmpty(updateWarehouseVo.getLatitude())){
                fb.setCode(0);
                fb.setMessage("纬度不能为空");
                return gson.toJson(fb);
            }
            if (StringUtils.isEmpty(updateWarehouseVo.getLongitude())){
                fb.setCode(0);
                fb.setMessage("经度不能为空");
                return gson.toJson(fb);
            }
            if(!updateWarehouseVo.getLatitude().matches("^-?(([1-9])((\\.\\d{1,6})?)|([1-8]\\d)((\\.\\d{1,6})?)|90)$")){
                fb.setCode(0);
                fb.setMessage("纬度的范围为-90~90");
                return gson.toJson(fb);
            }
            if(!updateWarehouseVo.getLongitude().matches("^-?(([1-9]\\d?)((\\.\\d{1,6})?)|(1[1-7]\\d)((\\.\\d{1,6})?)|180)$")) {
                fb.setCode(0);
                fb.setMessage("经度的范围为-180~180");
                return gson.toJson(fb);
            }
            if (StringUtils.isEmpty(updateWarehouseVo.getProperty())){
                fb.setCode(0);
                fb.setMessage("属性不能为空");
                return gson.toJson(fb);
            }else if (!updateWarehouseVo.getProperty().equals("供应中心")&&!updateWarehouseVo.getProperty().equals("项目仓")&&
                    !updateWarehouseVo.getProperty().equals("中心仓")&&!updateWarehouseVo.getProperty().equals("X-dock")){
                fb.setCode(0);
                fb.setMessage("请输入正确的属性");
                return gson.toJson(fb);
            }
            if (StringUtils.isEmpty(updateWarehouseVo.getType())){
                fb.setCode(0);
                fb.setMessage("类型不能为空");
                return gson.toJson(fb);
            }else if (!updateWarehouseVo.getType().equals("平面仓")&&!updateWarehouseVo.getType().equals("立体仓")){
                fb.setCode(0);
                fb.setMessage("请输入正确的仓库类型");
                return gson.toJson(fb);
            }
            if (StringUtils.isEmpty(updateWarehouseVo.getAlarmType())){
                fb.setCode(0);
                fb.setMessage("预警类型不能为空");
                return gson.toJson(fb);
            }
            if (StringUtils.isEmpty(updateWarehouseVo.getHardwareLevel())){
                fb.setCode(0);
                fb.setMessage("硬件等级不能为空");
                return gson.toJson(fb);
            }else if (!updateWarehouseVo.getHardwareLevel().equals("好")&&!updateWarehouseVo.getHardwareLevel().equals("一般")&&
                    !updateWarehouseVo.getHardwareLevel().equals("较差")&&!updateWarehouseVo.getHardwareLevel().equals("极差")){
                fb.setCode(0);
                fb.setMessage("请选择正确的硬件等级");
                return gson.toJson(fb);
            }
            if (updateWarehouseVo.getAreaIndoor() == null){
                fb.setCode(0);
                fb.setMessage("室内平面面积不能为空");
                return gson.toJson(fb);
            }
            if (updateWarehouseVo.getAreaRack()== null){
                fb.setCode(0);
                fb.setMessage("货架面积不能为空");
                return gson.toJson(fb);
            }
            if (updateWarehouseVo.getAreaOutdoor()== null){
                fb.setCode(0);
                fb.setMessage("室外面积不能为空");
                return gson.toJson(fb);
            }
            if (updateWarehouseVo.getAreaPlanIndoor()== null){
                fb.setCode(0);
                fb.setMessage("实际室内总面积不能为空");
                return gson.toJson(fb);
            }
            if (StringUtils.isEmpty(updateWarehouseVo.getOffice())){
                fb.setCode(0);
                fb.setMessage("办事处不能为空");
                return gson.toJson(fb);
            }
            if (updateWarehouseVo.getTimeZone()== null){
                fb.setCode(0);
                fb.setMessage("时区不能为空");
                return gson.toJson(fb);
            }
            if (StringUtils.isEmpty(updateWarehouseVo.getWeatherInfo())){
                fb.setCode(0);
                fb.setMessage("区域气象信息不能为空");
                return gson.toJson(fb);
            }
            if (StringUtils.isEmpty(updateWarehouseVo.getWarehouseInfo())){
                fb.setCode(0);
                fb.setMessage("基础描述不能为空");
                return gson.toJson(fb);
            }
            ResultWarehouseVo resultWarehouseVo=warehouseService.getByWarehouseName(updateWarehouseVo.getWarehouseName());
            if (resultWarehouseVo!=null&&!resultWarehouseVo.getWarehouseNo().equals(updateWarehouseVo.getWarehouseNo())){
                fb.setCode(0);
                fb.setMessage("修改的仓库名已存在");
                return gson.toJson(fb);
            }
            updateWarehouseVo.setUpdateTime(System.currentTimeMillis());
            int a = warehouseService.updateWarehouse(updateWarehouseVo);
            if (a>0){
                fb.setCode(1);
                fb.setMessage("修改仓库信息成功");
            }else {
                fb.setCode(0);
                fb.setMessage("修改仓库信息失败");
            }
            return gson.toJson(fb);
        }
    }

    //删除仓库信息
    @ApiOperation(value = "删除仓库信息",notes = "删除仓库信息,只用传需要删除的仓库编号")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/delWarehouse",method = RequestMethod.POST)
    public Object delWarehouse(@RequestBody String warehouseNo, HttpServletRequest request){
        JSONObject jsonObject= JSON.parseObject(warehouseNo);
        warehouseNo=(String)jsonObject.get("warehouseNo");
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        if(datum == null){
            fb.setCode(0);
            fb.setMessage("您没有登录，请先登录");
            return gson.toJson(fb);
        }else{
            if (StringUtils.isEmpty(warehouseNo)){
                fb.setCode(0);
                fb.setMessage("需要删除的仓库编号不能为空");
                return gson.toJson(fb);
            }
            int a = warehouseService.delWarehouse(warehouseNo);
            if (a>0){
                fb.setCode(1);
                fb.setMessage("删除仓库信息成功");
            }else {
                fb.setCode(0);
                fb.setMessage("删除仓库信息失败");
            }
            return gson.toJson(fb);
        }
    }


}
